@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005-2006 by Albert Astals Cid <aacid@kde.org>          *
+ *   Copyright (C) 2005-2007 by Albert Astals Cid <aacid@kde.org>          *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -42,7 +42,7 @@ static const double ellipseBigAxisY = 2.61;
 static const double nonButtonRibbonX = 150.0;
 static const double nonButtonRibbonY = 125.0;
 
-blinken::blinken() : QWidget(0), m_overHighscore(false), m_overQuit(false), m_overCentralText(false), m_overMenu(false), m_overAboutKDE(false), m_overAboutBlinken(false), m_overManual(false), m_overCentralLetters(false), m_overCounter(false), m_overFont(false), m_overSound(false), m_showPreferences(false), m_updateButtonHighlighting(false), m_highlighted(blinkenGame::none)
+blinken::blinken() : KMainWindow(), m_overHighscore(false), m_overQuit(false), m_overCentralText(false), m_overMenu(false), m_overAboutKDE(false), m_overAboutBlinken(false), m_overManual(false), m_overCentralLetters(false), m_overCounter(false), m_overFont(false), m_overSound(false), m_showPreferences(false), m_updateButtonHighlighting(false), m_highlighted(blinkenGame::none)
 {
 	m_renderer = new QSvgRenderer(KStandardDirs::locate("appdata", "images/blinken.svg"));
 	
@@ -71,9 +71,10 @@ blinken::blinken() : QWidget(0), m_overHighscore(false), m_overQuit(false), m_ov
 	
 	for (int i = 0; i < 3; i++) m_overLevels[i] = false;
 	
-	QString aux = i18nc("If the Steve font that is used by blinKen by default to show status messages does not support any of the characters of your language, please translate that message to 1 and KDE standard font will be used to show the texts, if not translate it to 0", "0");
+	QString aux = i18nc("If the Steve font that is used by Blinken by default to show status messages does not support any of the characters of your language, please translate that message to 1 and KDE standard font will be used to show the texts, if not translate it to 0", "0");
 	
 	m_alwaysUseNonCoolFont = aux == "1";
+	setAutoSaveSettings();
 }
 
 blinken::~blinken()
@@ -83,6 +84,11 @@ blinken::~blinken()
 	delete m_helpMenu;
 }
 
+QSize blinken::sizeHint() const
+{
+	return QSize(600, 490);
+}
+
 void blinken::paintEvent(QPaintEvent *)
 {
 	m_centralLettersRect = QRectF((double)width() / 3.35,
@@ -97,40 +103,60 @@ void blinken::paintEvent(QPaintEvent *)
 	QPainter p(this);
 	p.setRenderHint(QPainter::Antialiasing, true);
 	
+	if (m_lastSize != size())
+	{
+		m_pixmapCache.clear();
+		m_lastSize = size();
+	}
+	
 	// Base
-	m_renderer->render(&p, "blinkenBase");
+	// This -1 +1 suck but without them i got blank lines on borders
+	p.drawPixmap(-1, 0, getPixmap("blinkenBase", size()+QSize(1,1)));
 	
 	double xScaleButtons = 374.625 / 814.062;
 	double yScaleButtons = 250.344 / 664.062;
 	
 	// Red button
-	p.translate( (double)width() / 2, (double)height() / 25);
-	p.scale(xScaleButtons, yScaleButtons);
-	if (m_highlighted & blinkenGame::red) m_renderer->render(&p, "red_highlight");
-	else m_renderer->render(&p, "red_normal");
+	QSize sz = QSize((int)(width() * xScaleButtons), (int)(height() * yScaleButtons));
+	if (m_highlighted & blinkenGame::red)
+	{
+		p.drawPixmap(QPointF( (double)width() / 2, (double)height() / 25), getPixmap("red_highlight", sz));
+	}
+	else
+	{
+		p.drawPixmap(QPointF( (double)width() / 2, (double)height() / 25), getPixmap("red_normal", sz));
+	}
 	
 	// Green button
-	p.resetMatrix();
-	p.translate( (double)width() / 2, (double)height() / 2.339);
-	p.scale(xScaleButtons, yScaleButtons);
-	if (m_highlighted & blinkenGame::green) m_renderer->render(&p, "green_highlight");
-	else m_renderer->render(&p, "green_normal");
+	if (m_highlighted & blinkenGame::green)
+	{
+		p.drawPixmap(QPointF( (double)width() / 2, (double)height() / 2.339), getPixmap("green_highlight", sz));
+	}
+	else
+	{
+		p.drawPixmap(QPointF( (double)width() / 2, (double)height() / 2.339), getPixmap("green_normal", sz));
+	}
 	
 	// Yellow button
-	p.resetMatrix();
-	p.translate( (double)width() / 33.5, (double)height() / 25);
-	p.scale(xScaleButtons, yScaleButtons);
-	if (m_highlighted & blinkenGame::yellow) m_renderer->render(&p, "yellow_highlight");
-	else m_renderer->render(&p, "yellow_normal");
+	if (m_highlighted & blinkenGame::yellow)
+	{
+		p.drawPixmap(QPointF( (double)width() / 33.5, (double)height() / 25), getPixmap("yellow_highlight", sz));
+	}
+	else
+	{
+		p.drawPixmap(QPointF( (double)width() / 33.5, (double)height() / 25), getPixmap("yellow_normal", sz));
+	}
 	
 	// Blue button
-	p.resetMatrix();
-	p.translate( (double)width() / 33.5, (double)height() / 2.339);
-	p.scale(xScaleButtons, yScaleButtons);
-	if (m_highlighted & blinkenGame::blue) m_renderer->render(&p, "blue_highlight");
-	else m_renderer->render(&p, "blue_normal");
+	if (m_highlighted & blinkenGame::blue)
+	{
+		p.drawPixmap(QPointF( (double)width() / 33.5, (double)height() / 2.339), getPixmap("blue_highlight", sz));
+	}
+	else
+	{
+		p.drawPixmap(QPointF( (double)width() / 33.5, (double)height() / 2.339), getPixmap("blue_normal", sz));
+	}
 	
-	p.resetMatrix();
 	drawMenuQuit(p);
 	p.resetMatrix();
 	
@@ -167,16 +193,16 @@ void blinken::paintEvent(QPaintEvent *)
 		p.drawRect(m_soundRect);
 		if (blinkenSettings::playSounds())
 		{
-			p.drawLine(186, 214, 199, 227); 
-			p.drawLine(186, 227, 199, 214); 
+			p.drawLine(186, 214, 201, 229);
+			p.drawLine(186, 229, 201, 214);
 		}
 		if (!m_alwaysUseNonCoolFont)
 		{
 			p.drawRect(m_fontRect);
 			if (blinkenSettings::customFont())
 			{
-				p.drawLine(437, 214, 450, 227); 
-				p.drawLine(437, 227, 450, 214); 
+				p.drawLine(437, 214, 452, 229);
+				p.drawLine(437, 229, 452, 214);
 			}
 		}
 
@@ -332,7 +358,7 @@ void blinken::mousePressEvent(QMouseEvent *e)
 		blinkenSettings::self()->writeConfig();
 		update();
 	}
-	else if (m_overQuit) qApp->quit();
+	else if (m_overQuit) close();
 	else if (m_overAboutBlinken || m_overCentralLetters) m_helpMenu -> aboutApplication();
 	else if (m_overAboutKDE) m_helpMenu -> aboutKDE();
 	else if (m_overManual) m_helpMenu -> appHelpActivated();
@@ -536,9 +562,6 @@ void blinken::drawMenuQuit(QPainter &p)
 	double xScaleSquareButtons = 90.0 / 814.062;
 	double yScaleSquareButtons = 90.0 / 664.062;
 	
-	double xScaleSmallSquareButtons = 68.0 / 814.062;
-	double yScaleSmallSquareButtons = 68.0 / 664.062;
-	
 	// Highscore button
 	p.resetMatrix();
 	p.translate( (double)width() / 50.875, (double)height() / 30);
@@ -606,18 +629,12 @@ void blinken::drawMenuQuit(QPainter &p)
 				qRound((double)height() / 1.2),
 				qRound((double)width() * xScaleSquareButtons),
 				qRound((double)height() * yScaleSquareButtons));
-	m_aboutKDERect = QRect(qRound((double)width() / 1.421),
-				qRound((double)height() / 1.153),
-				qRound((double)width() * xScaleSmallSquareButtons),
-				qRound((double)height() * yScaleSmallSquareButtons));
-	m_aboutBlinkenRect = QRect(qRound((double)width() / 1.271),
-					qRound((double)height() / 1.153),
-					qRound((double)width() * xScaleSmallSquareButtons),
-					qRound((double)height() * yScaleSmallSquareButtons));
-	m_manualRect = QRect(qRound((double)width() / 1.1146),
-				qRound((double)height() / 1.3667),
-				qRound((double)width() * xScaleSmallSquareButtons),
-				qRound((double)height() * yScaleSmallSquareButtons));
+	m_aboutBlinkenRect = QRect(QPoint(qRound((double)width() / 1.271), qRound((double)height() / 1.153)),
+	                           QPoint(m_menuRect.left(), m_menuRect.bottom()));
+	m_manualRect = QRect(QPoint(qRound((double)width() / 1.1146), qRound((double)height() / 1.3667)),
+	                     QPoint(m_menuRect.right(), m_menuRect.top()));
+	m_aboutKDERect = QRect(QPoint(qRound((double)width() / 1.421), qRound((double)height() / 1.153)),
+	                       QPoint(m_aboutBlinkenRect.left(), m_aboutBlinkenRect.bottom()));
 }
 
 void blinken::drawScoreAndCounter(QPainter &p)
@@ -665,7 +682,7 @@ void blinken::drawStatusText(QPainter &p)
 
 	QString restartText = i18n("Restart the game");
 	QString text;
-	if (m_overQuit) text = i18n("Quit blinKen");
+	if (m_overQuit) text = i18n("Quit Blinken");
 	else if (m_overHighscore || m_overCounter) text = i18n("View Highscore Table");
 	else if (m_overAboutBlinken || m_overCentralLetters) text = m_helpMenu->action( KHelpMenu::menuAboutApp )->text().remove("&");
 	else if (m_overAboutKDE) text = m_helpMenu->action( KHelpMenu::menuAboutKDE )->text().remove("&"); 
@@ -974,4 +991,18 @@ void blinken::updateCursor(const QPoint &p)
 	else setCursor(Qt::ArrowCursor);
 }
 
+QPixmap blinken::getPixmap(const QString &element, const QSize &imageSize)
+{
+	QMap<QString, QPixmap>::const_iterator it = m_pixmapCache.constFind(element);
+	if (it == m_pixmapCache.constEnd())
+	{
+		QPixmap pix(imageSize);
+		pix.fill(Qt::transparent);
+		QPainter p(&pix);
+		m_renderer->render(&p, element);
+		it = m_pixmapCache.insert(element, pix);
+	}
+	return it.value();
+}
+
 #include "blinken.moc"
