@@ -1,9 +1,10 @@
 /*
- * Copyright (C) 2001-2006 Anne-Marie Mahfouf <annma@kde.org>
+ * Copyright 2001-2007 Anne-Marie Mahfouf <annma@kde.org>
 
-    This program is free software; you can redistribute it and/or
-    modify it under the terms of version 2 of the GNU General Public
-    License as published by the Free Software Foundation.
+     This program is free software; you can redistribute it and/or modify  
+     it under the terms of the GNU General Public License as published by  
+     the Free Software Foundation; either version 2 of the License, or     
+     (at your option) any later version.                                   
 
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,6 +25,7 @@
 #include <kpassivepopup.h>
 #include <kpushbutton.h>
 #include <kstandarddirs.h>
+#include <ktoggleaction.h>
 #include <kvbox.h>
 
 #include <Phonon/MediaObject>
@@ -39,7 +41,7 @@
 #include <QWidget>
 #include <QMouseEvent>
 #include <QSvgRenderer>
- #include <QRegExpValidator>
+#include <QRegExpValidator>
 //project headers
 #include "prefs.h"
 #include "khangman.h"
@@ -47,8 +49,8 @@
 #include "langutils.h"
 
 
-KHangManView::KHangManView(KHangMan*parent)
-    : QWidget(parent /*WStaticContents | WNoAutoErase*/)
+KHangManView::KHangManView(KHangMan*parent) 
+        : QWidget(parent /*WStaticContents | WNoAutoErase*/), m_showhint(false)
 {
     setAttribute(Qt::WA_StaticContents);
     khangman = parent;
@@ -81,11 +83,12 @@ KHangManView::KHangManView(KHangMan*parent)
     m_posSecondSpace   = -1;
     m_numMissedLetters = 0;
     m_accentedLetters  = true;
-    m_hintExists       = true;	// Assume hint exists
+    m_hintExists       = true;
     m_doc              = 0;
     m_theme            = 0; // essential
     m_player           = 0;
     m_randomInt        = -1;
+
     connect( m_letterInput, SIGNAL( returnPressed() ), this, SLOT( slotTry() ) );
     connect( m_guessButton, SIGNAL( clicked() ), this, SLOT( slotTry() ));
     connect( this, SIGNAL(signalChangeStatusbar(const QString&, int)), khangman, SLOT(changeStatusbar(const QString&, int)));
@@ -110,7 +113,6 @@ KHangManView::~KHangManView()
 
 // Handle a guess of the letter in sChar.
 //
-
 void KHangManView::replaceLetters(const QString& sChar)
 {
     int   index = 0;
@@ -132,11 +134,13 @@ void KHangManView::replaceLetters(const QString& sChar)
                 b_end = true;
                 break;
             }
-            else
+            else {
                 index++;
+            }
 
-            if (count == m_word.count(sChar)-1)
-            b_end = true;
+            if (count == m_word.count(sChar)-1) {
+                b_end = true;
+            }
         }
     }
     else {
@@ -163,29 +167,32 @@ void KHangManView::replaceLetters(const QString& sChar)
         if (sChar=="e") replaceLetters(QString::fromUtf8("é"));
         if (sChar=="u") replaceLetters(QString::fromUtf8("ù"));
     }
-    if (!Prefs::oneLetter())
+    if (!Prefs::oneLetter()) {
         m_guessedLetters << sChar; //appends the list only if not in One Letter only mode...
+    }
 
-    if (m_word.count(sChar) == 1)
+    if (m_word.count(sChar) == 1) {
         m_guessedLetters << sChar; //append if only one instance
+    }
 
-    if (Prefs::oneLetter() && b_end)
+    if (Prefs::oneLetter() && b_end) {
         m_guessedLetters << sChar;
+    }
 }
 
 void KHangManView::play(const QString& soundFile)
 {
-    if (soundFile.isEmpty())
+    if (soundFile.isEmpty()) {
         return;
+    }
 
-    if (!m_player)
-    {
+    if (!m_player) {
         m_player = Phonon::createPlayer(Phonon::GameCategory);
     }
-    else
-    {
+    else {
         m_player->stop();
     }
+
     m_player->setCurrentSource(soundFile);
     m_player->play();
 }
@@ -195,60 +202,36 @@ bool KHangManView::containsChar(const QString &sChar)
 {
     bool b_isInWord = false;
     if (m_accentedLetters && !Prefs::accentedLetters()) {
-        if (sChar=="i")
+        if (sChar=="i") {
             b_isInWord = m_word.contains(QString::fromUtf8("í"));
+        }
 
-        if (sChar=="a")
+        if (sChar=="a") {
             b_isInWord = m_word.contains(QString::fromUtf8("à"))
-                || m_word.contains(QString::fromUtf8("á"))
-                || m_word.contains(QString::fromUtf8("ã"));
+                    || m_word.contains(QString::fromUtf8("á"))
+                    || m_word.contains(QString::fromUtf8("ã"));
+        }
 
-        if (sChar=="u")
+        if (sChar=="u") {
             b_isInWord = m_word.contains(QString::fromUtf8("ü"))
-                || m_word.contains(QString::fromUtf8("ù"));
+                    || m_word.contains(QString::fromUtf8("ù"));
+        }
 
-        if (sChar=="o")
+        if (sChar=="o") {
             b_isInWord = m_word.contains(QString::fromUtf8("ò"))
-                || m_word.contains(QString::fromUtf8("ó"))
-                || m_word.contains(QString::fromUtf8("õ"));
+                    || m_word.contains(QString::fromUtf8("ó"))
+                    || m_word.contains(QString::fromUtf8("õ"));
+        }
 
-        if (sChar=="e")
+        if (sChar=="e") {
             b_isInWord = m_word.contains(QString::fromUtf8("è"))
-                || m_word.contains(QString::fromUtf8("é"));
+                    || m_word.contains(QString::fromUtf8("é"));
+        }
     }
 
     return (b_isInWord || m_word.contains(sChar));
 }
 
-
-// ----------------------------------------------------------------
-//                           Event handlers
-
-
-void KHangManView::mousePressEvent(QMouseEvent *mouse)
-{
-    if (mouse->button() == Qt::RightButton && m_hintExists && Prefs::hint()) {
-
-        KPassivePopup *myPopup = new KPassivePopup( m_letterInput);
-        myPopup->setView(i18n("Hint"), m_hint );
-        myPopup->setTimeout(Prefs::hintTimer()*1000); //show for 4 seconds as default
-        int x=0, y=0;
-
-        QPoint abspos = mapToGlobal( QPoint( 0, 0 ) );
-        x = abspos.x() + width()*70/700;
-        y = abspos.y() + height()*150/535;
-
-        QPoint  point = QPoint(x, y);
-        myPopup->show(mapToGlobal(point));
-
-        // Maybe it's better to popup where the mouse clicks, in that
-        // case kill the popup before new click
-        //myPopup->move(mouse->pos());
-    }
-    //update();
-}
-
-
 // ----------------------------------------------------------------
 
 // FIXME: Move this function somewhere logical
@@ -257,13 +240,15 @@ void KHangManView::mousePressEvent(QMouseEvent *mouse)
 void KHangManView::setTheme(KHMTheme *theme)
 {
     // we don't allow null themes
-    if (!theme)
+    if (!theme) {
         return;
+    }
 
     QString svgpath = KStandardDirs::locate("data", QString("khangman/pics/%1/%2").arg(theme->name(), theme->svgFileName()));
     // we don't allow themes with no svg installed
-    if (!QFile::exists(svgpath))
+    if (!QFile::exists(svgpath)) {
         return;
+    }
 
     delete m_theme;
     m_theme = theme;
@@ -282,6 +267,10 @@ void KHangManView::setTheme(KHMTheme *theme)
     m_theme->letterInputTextColor().getRgb(&r1, &g1, &b1);
     m_letterInput->setStyleSheet(QString("QLineEdit{border-style: solid; background-color: white; color: rgb(%1, %2, %3) ; border-bottom-right-radius:10; border-radius: 15px; border-width: 3px; padding: 3px; border-color: rgb(%1, %2, %3)}").arg(r1).arg(g1).arg(b1));
     m_letterInput->setFocus();
+    m_guessButton->setFont(QFont("Dustimo Roman", height()/22));
+    QFont charWrite_font( m_letterInput->font() );
+    charWrite_font.setPointSize( height()/18 );
+    m_letterInput->setFont(charWrite_font);
     update();
 }
 
@@ -296,6 +285,7 @@ void KHangManView::paintEvent( QPaintEvent * e )
     paintHangman(p, e->rect());
     paintWord(p, e->rect());
     paintMisses(p, e->rect());
+    paintHint(p, e->rect());
 }
 
 
@@ -307,13 +297,14 @@ void KHangManView::paintHangman(QPainter &p, const QRect& rect)
         m_backgroundCache = QPixmap(size());
         QPainter aux(&m_backgroundCache);
         m_renderer->render(&aux, "background");
-
     }
     p.drawPixmap(rect.topLeft(), m_backgroundCache, rect);
     // Draw the animated hanged K
     QRect myRect = m_theme->kRect(size());
-    if (!myRect.intersects(rect))
+    if (!myRect.intersects(rect)) {
         return;
+    }
+
     m_renderer->render(&p, QString("ani%2").arg(m_numMissedLetters), myRect);
 }
 
@@ -321,8 +312,9 @@ void KHangManView::paintHangman(QPainter &p, const QRect& rect)
 void KHangManView::paintWord(QPainter &p, const QRect& rect)
 {
     QRect myRect = m_theme->wordRect(size());
-    if (!myRect.intersects(rect))
+    if (!myRect.intersects(rect)) {
         return;
+    }
 
     p.setPen(m_theme->letterColor());
 
@@ -332,6 +324,8 @@ void KHangManView::paintWord(QPainter &p, const QRect& rect)
     tFont.setPixelSize( 28 );
 
     p.setFont(tFont);
+    // Set first letter as upper case for German
+    goodWord.replace(0, 1, LangUtils::capitalize(goodWord.at(0), Prefs::selectedLanguage()));
     p.drawText(myRect, Qt::AlignCenter|Qt::AlignCenter, goodWord);
 }
 
@@ -359,16 +353,53 @@ void KHangManView::paintMisses(QPainter &p, const QRect& )
     mFont.setPointSize(30);
     p.setFont(mFont);
 
-    QFontMetrics  fm2(mFont);
-    QRect         fmRect2(fm2.boundingRect(misses));
-    QRect         myRect2(width()- fmRect.width() - fmRect2.width() - 15,
-                      15 - fm2.height() + fm.height(),
-                      fmRect2.width(), fm2.height());
+    QFontMetrics    fm2(mFont);
+    QRect           fmRect2(fm2.boundingRect(misses));
+    QRect           myRect2(width()- fmRect.width() - fmRect2.width() - 15,
+                            15 - fm2.height() + fm.height(),
+                            fmRect2.width(), fm2.height());
     p.setPen( letterColor );
     p.drawText(myRect2, Qt::AlignLeft|Qt::AlignCenter, misses);
 }
 
 
+void KHangManView::paintHint(QPainter &p, const QRect &rect )
+{
+    if(!m_showhint)
+        return;
+
+    QRect myRect = m_theme->hintRect(size());
+    if(!myRect.intersects(rect))
+        return;
+
+/* Debug */  //p.fillRect(myRect, QBrush(Qt::cyan) );
+
+    QColor letterColor = m_theme->letterColor();
+    p.setPen(letterColor);
+
+    QString hint=i18n("Hint");
+    QFont hFont = QFont("Domestic Manners");
+    hFont.setPointSize(14);
+    QFontMetrics fm1(hFont);
+    QRect fm1Rect(fm1.boundingRect(hint));
+    
+    QFont tFont = LangUtils::fontForLanguage(Prefs::selectedLanguage());
+    tFont.setPixelSize(14);
+    QFontMetrics fm2(tFont);
+    QRect fm2Rect(fm2.boundingRect(m_hint));
+
+    QRect myRect1 = QRect(myRect.x(), myRect.y(), myRect.width(), fm1Rect.height());
+
+    QRect myRect2 = QRect(myRect.x(), myRect.y()+fm1Rect.height(), myRect.width(), myRect.height()-fm1Rect.height());
+
+    p.setFont(hFont);
+    p.drawText(myRect1, Qt::AlignHCenter, hint);
+
+    p.setFont(tFont);
+    p.drawText(myRect2, Qt::AlignHCenter | Qt::TextWordWrap, m_hint);
+}
+
+
 void KHangManView::resizeEvent(QResizeEvent *)
 {
     m_letterInput->setMinimumSize( QSize( height()/18, 0 ) );
@@ -379,12 +410,12 @@ void KHangManView::resizeEvent(QResizeEvent *)
 
     m_letterInput->setFont( charWrite_font );
     m_letterInput->setGeometry(width()-2*height()/12, height()-2*height()/16,
-                               height()/10, height()/10);
+                               height()/9, height()/9);
     m_guessButton->setFont(QFont("Dustimo Roman", height()/22));
     m_guessButton->setGeometry(width() - 2*height()/12
                                - m_guessButton->sizeHint().width()-7,
                                height() - 2*height()/16,
-                               m_guessButton->sizeHint().width(), height()/10);
+                               m_guessButton->sizeHint().width(), height()/9);
 }
 
 
@@ -395,125 +426,128 @@ void KHangManView::slotTry()
 {
     QString guess = m_letterInput->text();
     if (guess.isEmpty()) {
-	m_letterInput->setFocus();
-	return;
+        m_letterInput->setFocus();
+        return;
     }
     kDebug() << "guess as entered: " << guess;
 
-    guess = LangUtils::capitalize(guess, Prefs::selectedLanguage(), Prefs::upperCase());
-
     // Handle the guess.
     if (!m_guessedLetters.contains(guess)) {
         // The letter is not already guessed.
 
-    if (containsChar(guess)) {
-        replaceLetters(guess);
+        if (containsChar(guess)) {
+            replaceLetters(guess);
 
-        // This is needed because of the white spaces.
-        QString  stripWord = goodWord;
-        QString  sword     = m_word;
-        if (m_posSecondSpace > 0)  {
-            stripWord.replace(2*m_posFirstSpace,   1, "");
-            stripWord.replace(2*m_posFirstSpace-1, 1, "");
+            // This is needed because of the white spaces.
+            QString  stripWord = goodWord;
+            QString  sword     = m_word;
+            if (m_posSecondSpace > 0)  {
+                stripWord.replace(2*m_posFirstSpace,   1, "");
+                stripWord.replace(2*m_posFirstSpace-1, 1, "");
 
-            stripWord.replace(2*(m_posSecondSpace-1),   1, "");
-            stripWord.replace(2*(m_posSecondSpace-1)-1, 1, "");
-        }
-
-        QStringList  rightChars =  stripWord.split(" ");
-        QString      rightWord  = rightChars.join("");
-        update();
-        sword.remove(QRegExp(" "));
-
-        // If the user made it...
-        if (rightWord.trimmed().toLower() == sword.trimmed().toLower()) {
-
-            // We reset everything...
-            //TODO find a better way to finish
-            //
-
-            if (Prefs::sound()) {
-                QString soundFile = KStandardDirs::locate("data", "khangman/sounds/EW_Dialogue_Appear.ogg");
-                play(soundFile);
+                stripWord.replace(2*(m_posSecondSpace-1),   1, "");
+                stripWord.replace(2*(m_posSecondSpace-1)-1, 1, "");
             }
 
-            if (Prefs::wonDialog()) {
-                // TODO: hide Hint KPassivePopup if any
-                QPoint point;
-                KPassivePopup *popup = new KPassivePopup( this);
-                popup->setAutoDelete( true );
-                popup->setTimeout( 4*1000 );
-                popup->setView(i18n("Congratulations,\nyou won!") );
+            QStringList  rightChars =  stripWord.split(" ");
+            QString      rightWord  = rightChars.join("");
+            update();
+            sword.remove(QRegExp(" "));
 
-                int x =0, y = 0;
-                QPoint abspos = popup->pos();
-                x = abspos.x() + width()*50/700;
-                y = abspos.y() + height()*20/535;
-                point = QPoint(x, y);
-                popup->show(mapToGlobal(point));
-                QTimer::singleShot( 4*1000, this, SLOT(newGame()) );
-            }
-            else if (KMessageBox::questionYesNo(this,
+            // If the user made it...
+            if (rightWord.trimmed().toLower() == sword.trimmed().toLower()) {
+
+                // We reset everything...
+                //TODO find a better way to finish
+                //
+
+                if (Prefs::sound()) {
+                    QString soundFile = KStandardDirs::locate("data", "khangman/sounds/EW_Dialogue_Appear.ogg");
+                    play(soundFile);
+                }
+
+                if (Prefs::wonDialog()) {
+                    // TODO: hide Hint KPassivePopup if any
+                    QPoint point;
+                    KPassivePopup *popup = new KPassivePopup( this);
+                    popup->setAutoDelete( true );
+                    popup->setTimeout( 4*1000 );
+                    popup->setView(i18n("Congratulations,\nyou won!") );
+
+                    int x =0, y = 0;
+                    QPoint abspos = popup->pos();
+                    x = abspos.x() + width()*50/700;
+                    y = abspos.y() + height()*20/535;
+                    point = QPoint(x, y);
+                    popup->show(mapToGlobal(point));
+                    QTimer::singleShot( 4*1000, this, SLOT(newGame()) );
+                }
+                else if (KMessageBox::questionYesNo(this,
                                         i18n("Congratulations! You won! Do you want to play again?"),
                                         QString(),KGuiItem(i18n("Play Again")), KGuiItem(i18n("Do Not Play"))) == 3)
-                newGame();
-            else
-                qApp->quit();
+                {
+                    newGame();
+                }
+                else {
+                    qApp->quit();
+                }
 
-            ++winCount;
-            setGameCount();
-
+                ++winCount;
+                setGameCount();
+            }
         }
-    }
-    else {
-        // The char is missed.
-
-        m_guessedLetters << guess;
-        m_missedLetters = m_missedLetters.replace((2 * m_numMissedLetters), 1, guess);
-
-        m_numMissedLetters++;
-        update();
-        // Check if we have reached the limit of wrong guesses.
-        if (m_numMissedLetters >= MAXWRONGGUESSES) {
-
-            //TODO sequence to finish when hanged
-            QStringList charList=m_word.split(" ");
-            QString theWord=charList.join(" ");
-            goodWord = theWord;
-
-            //usability: find another way to start a new game
-            QString newGameString = i18n("You lost. Do you want to play again?");
-            if (Prefs::wonDialog()) {
-                // TODO: hide Hint KPassivePopup if any
-                QPoint point;
-                KPassivePopup *popup = new KPassivePopup( this);
-                popup->setAutoDelete( true );
-                popup->setTimeout( 4*1000 );
-
-                KVBox *vb = new KVBox( popup );
-
-                QLabel *popLabel = new QLabel( vb);
-                popLabel->setFont(QFont("Sans Serif", 14, QFont::Normal));
-                popLabel->setText(i18n("<qt>You lost!\nThe word was\n<b>%1</b></qt>", m_word));
-                popup->setView( vb );
-
-                QPoint abspos = popup->pos();
-                int  x = abspos.x() + width()  * 50 / 700;
-                int  y = abspos.y() + height() * 20 / 535;
-                popup->show(mapToGlobal(QPoint(x, y)));
-
-                QTimer::singleShot( 4 * 1000, this, SLOT(newGame()) );
+        else {
+            // The char is missed.
+
+            m_guessedLetters << guess;
+            m_missedLetters = m_missedLetters.replace((2 * m_numMissedLetters), 1, guess);
+
+            m_numMissedLetters++;
+            update();
+            // Check if we have reached the limit of wrong guesses.
+            if (m_numMissedLetters >= MAXWRONGGUESSES) {
+
+                //TODO sequence to finish when hanged
+                QStringList charList=m_word.split(" ");
+                QString theWord=charList.join(" ");
+                goodWord = theWord;
+
+                //usability: find another way to start a new game
+                QString newGameString = i18n("You lost. Do you want to play again?");
+                if (Prefs::wonDialog()) {
+                    // TODO: hide Hint KPassivePopup if any
+                    QPoint point;
+                    KPassivePopup *popup = new KPassivePopup( this);
+                    popup->setAutoDelete( true );
+                    popup->setTimeout( 4*1000 );
+
+                    KVBox *vb = new KVBox( popup );
+
+                    QLabel *popLabel = new QLabel( vb);
+                    popLabel->setFont(QFont("Sans Serif", 14, QFont::Normal));
+                    popLabel->setText(i18n("<qt>You lost!\nThe word was\n<b>%1</b></qt>", m_word));
+                    popup->setView( vb );
+
+                    QPoint abspos = popup->pos();
+                    int  x = abspos.x() + width()  * 50 / 700;
+                    int  y = abspos.y() + height() * 20 / 535;
+                    popup->show(mapToGlobal(QPoint(x, y)));
+
+                    QTimer::singleShot( 4 * 1000, this, SLOT(newGame()) );
+                }
+                else if (KMessageBox::questionYesNo(this, newGameString, QString(),
+                                                KGuiItem(i18n("Play Again")), KGuiItem(i18n("Do Not Play"))) == 3)
+                {
+                    newGame();
+                }
+                else {
+                    qApp->quit();
+                }
+
+                ++lossCount;
+                setGameCount();
             }
-            else if (KMessageBox::questionYesNo(this, newGameString, QString(),
-                                                KGuiItem(i18n("Play Again")), KGuiItem(i18n("Do Not Play"))) == 3)
-                newGame();
-            else
-                qApp->quit();
-
-            ++lossCount;
-            setGameCount();
         }
-    }
     }
     else {
         // The letter is already guessed.
@@ -573,6 +607,12 @@ void KHangManView::enableUserInput()
 
 void KHangManView::newGame()
 {
+    //reset Hint action
+    khangman->hintAct->setChecked(false);
+    slotSetHint(khangman->hintAct->isChecked());
+
+    khangman->hintAct->setEnabled(m_hintExists);
+
     setGameCount();
     if (Prefs::sound()) {
         QString soundFile = KStandardDirs::locate("data", "khangman/sounds/new_game.ogg");
@@ -599,11 +639,11 @@ void KHangManView::readFile()
                                  Prefs::selectedLanguage(),
                                  Prefs::levelFile());
         KMessageBox::sorry( this, mString, i18n("Error") );
-        qApp->quit();
+        //qApp->quit();
     }
 
     // Detects if file is a kvtml file so that it's a hint enable file
-	slotSetWordsSequence();
+    slotSetWordsSequence();
 }
 
 void KHangManView::reset()
@@ -623,29 +663,27 @@ void KHangManView::reset()
 void KHangManView::game()
 {
     kDebug() << "language " << Prefs::selectedLanguage();
-    kDebug() << "level "    << Prefs::levelFile()       ;
+    kDebug() << "level "    << Prefs::levelFile();
 
     m_word = m_randomList[m_randomInt%NumberOfWords].first;
+    m_word = m_word.toLower();
     m_hint = m_randomList[m_randomInt%NumberOfWords].second;
 
-    if (m_word.isEmpty()){
+    if (m_word.isEmpty()) {
         m_randomInt++;
-		game();
-	}
+        game();
+    }
 
     if (m_hint.isEmpty()) {
-        m_hintExists = false;	// Hint does not exist.
-        emit signalChangeStatusbar("", 103);
+        m_hintExists = false;   // Hint does not exist.
+        //TODO refresh hint action
     }
     else {
         m_hintExists = true;
-        khangman->setMessages();
+        //TODO refresh hint action
     }
-
-    if (Prefs::upperCase() && Prefs::selectedLanguage() =="de")
-    {
-        m_word = m_word.toUpper();// only for German currently
-
+    khangman->hintAct->setEnabled(m_hintExists);
+    if ( Prefs::selectedLanguage() =="de") {
         // Replace ß with SS in German
         if (m_word.contains(QString::fromUtf8("ß"))) {
             int index = m_word.indexOf(QString::fromUtf8("ß"),0);
@@ -657,8 +695,9 @@ void KHangManView::game()
     kDebug() << m_word;
 
     // Display the number of letters to guess with _
-    for (int i = 0; i < m_word.length(); i++)
+    for (int i = 0; i < m_word.length(); i++) {
         goodWord.append("_ ");
+    }
 
     // Remove the last trailing space.
     goodWord.remove(goodWord.length()-1);
@@ -673,10 +712,12 @@ void KHangManView::game()
         goodWord.replace(2*posOfFirstDash, 1, "-");
 
         int posOfSecondDash = m_word.indexOf( "-", posOfFirstDash+1);
-        if (posOfSecondDash>0)
+        if (posOfSecondDash>0) {
             goodWord.replace(2*posOfSecondDash, 3, "-");
-        if (posOfSecondDash>1)
+        }
+        if (posOfSecondDash>1) {
             goodWord.append("_");
+        }
     }
 
     // 2. Find white space.
@@ -690,13 +731,15 @@ void KHangManView::game()
 
     // 3. Find ·
     int posOfDot = m_word.indexOf( QString::fromUtf8("·") );
-    if (posOfDot>0)
+    if (posOfDot>0) {
         goodWord.replace(2*posOfDot, 1, QString::fromUtf8("·") );
+    }
 
     // 4. Find '
     int posOfApos = m_word.indexOf( "'" );
-    if (posOfApos>0)
+    if (posOfApos>0) {
         goodWord.replace(2*posOfApos, 1, "'");
+    }
 }
 
 
@@ -704,13 +747,13 @@ void KHangManView::slotSetWordsSequence()
 {
     kDebug() << "in read kvtml file ";
 
-    if (m_doc != 0)
-    {
+    if (m_doc != 0) {
         delete m_doc;
         m_doc = 0;
     }
 
     m_doc = new KEduVocDocument(this);
+    ///@todo open returns KEduVocDocument::ErrorCode
     m_doc->open(Prefs::levelFile());
 
     //how many words in the file
@@ -719,16 +762,14 @@ void KHangManView::slotSetWordsSequence()
     //get the words+hints
     KRandomSequence randomSequence;
     m_randomList.clear();
-    for (int j = 0; j < NumberOfWords; j++)
-    {
+    for (int j = 0; j < NumberOfWords; j++) {
         QString hint = m_doc->entry(j)->translation(0).comment();
-        if (hint.isEmpty() && m_doc->identifierCount() > 0)
-        {
+        if (hint.isEmpty() && m_doc->identifierCount() > 0) {
             // if there is no comment or it's empty, use the first translation if
             // there is one
             hint = m_doc->entry(j)->translation(1).text();
         }
-        
+
         m_randomList.append(qMakePair(m_doc->entry(j)->translation(0).text(), m_doc->entry(j)->translation(0).comment()));
     }
     //shuffle the list
@@ -741,5 +782,14 @@ void KHangManView::setGameCount()
     emit signalChangeStatusbar(i18n("Losses: %1", lossCount), IDS_LOSSES);
 }
 
+void KHangManView::slotSetHint(bool hintBool)
+{
+    m_showhint=hintBool;
+    update();
+}
+
 #include "khangmanview.moc"
 
+// kate: space-indent on; tab-width 4; indent-width 4; mixed-indent off; replace-tabs on;
+// vim: set et sw=4 ts=4 cino=l1,cs,U1:
+
