@@ -146,7 +146,7 @@ KigDocument* KigFilterCabri::load( const QString& file )
   }
 
   QString s = CabriNS::readLine( f );
-  QRegExp header( "Figure Cabri\\s?II (Plus )?vers\\. (MS-Windows|DOS) ((\\d+)\\.(\\d+)(\\.(\\d+))?(\\.(\\d+))?)(, ([^,]+), t=(\\d+)s)?" );
+  QRegExp header( "Figure Cabri\\s?II (Plus )?vers\\. (MS-Windows|DOS) ((\\d+)\\.(\\d+)(\\.(\\d+))?(\\.(\\d+))?|(\\d+)\\.x \\((\\d+)\\.(\\d+)\\))(, ([^,]+), t=(\\d+)s)?" );
   header.setCaseSensitivity( Qt::CaseInsensitive );
   if ( !header.exactMatch( s ) )
   {
@@ -160,9 +160,15 @@ KigDocument* KigFilterCabri::load( const QString& file )
   }
 
   bool ok = true;
-  int major = header.cap( 4 ).toInt( &ok );
+  QString majorstr = header.cap( 4 );
+  if ( majorstr.isEmpty() )
+    majorstr = header.cap( 11 );
+  int major = majorstr.toInt( &ok );
   bool ok2 = true;
-  int minor = header.cap( 5 ).toInt( &ok2 );
+  QString minorstr = header.cap( 5 );
+  if ( minorstr.isEmpty() )
+    minorstr = header.cap( 12 );
+  int minor = minorstr.toInt( &ok2 );
   if ( !ok || !ok2 )
     KIG_FILTER_PARSE_ERROR;
 
@@ -171,11 +177,11 @@ KigDocument* KigFilterCabri::load( const QString& file )
 
 #ifdef CABRI_DEBUG
   kDebug() << ">>>>>>>>> version: " << header.cap( 3 );
-  if ( !header.cap( 10 ).isEmpty() )
+  if ( !header.cap( 13 ).isEmpty() )
   {
     kDebug() << ">>>>>>>>> session file:";
-    kDebug() << ">>>>>>>>>  -> time = " << header.cap( 12 ) << " sec";
-    kDebug() << ">>>>>>>>>  -> action = \"" << header.cap( 11 ) << "\"";
+    kDebug() << ">>>>>>>>>  -> time = " << header.cap( 15 ) << " sec";
+    kDebug() << ">>>>>>>>>  -> action = \"" << header.cap( 14 ) << "\"";
   }
 #endif
   if ( ( major == 1 ) && ( minor == 0 ) )
@@ -183,7 +189,7 @@ KigDocument* KigFilterCabri::load( const QString& file )
     curVer = CabriNS::CV_1_0;
     reader = new CabriReader_v10( this );
   }
-  else if ( ( major == 1 ) && ( minor == 2 ) )
+  else if ( ( major == 1 ) && ( minor == 2 || minor == 4 ) )
   {
     curVer = CabriNS::CV_1_2;
     reader = new CabriReader_v12( this );
@@ -481,10 +487,10 @@ KigDocument* KigFilterCabri::load( const QString& file )
     }
     else if ( obj->type == "Angle" )
     {
-      if ( args.size() != 3 || obj->data.size() != 2 ) KIG_FILTER_PARSE_ERROR;
+      if ( args.size() != 3 ) KIG_FILTER_PARSE_ERROR;
       oc = new ObjectTypeCalcer( AngleType::instance(), args );
     }
-    else if ( obj->type == "Text" && curVer == CabriNS::CV_1_2 ) // Cabri1.2
+    else if ( obj->type == "Text" )
     {
 //      if ( !args.empty() /* || obj->data.size() != 2 */ ) KIG_FILTER_PARSE_ERROR;
       if ( !obj->textRect.valid() ) KIG_FILTER_PARSE_ERROR;
