@@ -100,11 +100,27 @@ void ISGetProperties (const char *dev) {
 	IDDefNumber (&geoNum, NULL);
 }
 
-void ISNewBLOB (const char *dev, const char *name, int sizes[], int blobsizes[], char *blobs[], char *formats[], char *names[], int n) {}
-void ISSnoopDevice (XMLEle *root) {}
+void ISNewBLOB (const char *dev, const char *name, int sizes[], int blobsizes[], char *blobs[], char *formats[], char *names[], int n) 
+{
+  INDI_UNUSED(dev);
+  INDI_UNUSED(name);
+  INDI_UNUSED(sizes);
+  INDI_UNUSED(blobsizes);
+  INDI_UNUSED(blobs);
+  INDI_UNUSED(formats);
+  INDI_UNUSED(names);
+  INDI_UNUSED(n);
+}
+void ISSnoopDevice (XMLEle *root) 
+{
+  INDI_UNUSED(root);
+}
 
-void ISNewText (const char *dev, const char *name, char *texts[], char *names[], int n) {
-        /*IText *tp;*/
+void ISNewText (const char *dev, const char *name, char *texts[], char *names[], int n) 
+{
+
+	INDI_UNUSED(names);
+	INDI_UNUSED(n);
 
 	if (strcmp (dev, mydev))
 		return;
@@ -251,7 +267,7 @@ void ISNewSwitch (const char *dev, const char *name, ISState *states, char *name
 
 	if (!strcmp (name, OnCoordSetSw.name)) {
 		if (POWSW){
-			IUResetSwitches(&OnCoordSetSw);
+			IUResetSwitch(&OnCoordSetSw);
 			sp = IUFindSwitch (&OnCoordSetSw, names[0]);
 			if (!sp)
 				return;
@@ -260,8 +276,8 @@ void ISNewSwitch (const char *dev, const char *name, ISState *states, char *name
 
 			sp->s = states[0];
 
-			IUResetSwitches(&OnCoordSetSw);
-			IUUpdateSwitches(&OnCoordSetSw, states, names, n);
+			IUResetSwitch(&OnCoordSetSw);
+			IUUpdateSwitch(&OnCoordSetSw, states, names, n);
 			/*			    currentSet = getOnSwitch(&OnCoordSetSw); */
 			OnCoordSetSw.s = IPS_OK;
 			IDSetSwitch(&OnCoordSetSw, NULL);
@@ -308,6 +324,8 @@ double calcLST(char *strlst){
 /* update the mount over time */
 void readMountcurrentpos (void *p)
 {
+	INDI_UNUSED(p);
+
 	char result[32];
 
 	if(POWSW){
