@@ -20,6 +20,7 @@
 #include <QPainter>
 #include <QBitmap>
 #include <QCheckBox>
+#include <QStackedWidget>
 
 #include <kactioncollection.h>
 #include <kapplication.h>
@@ -44,6 +45,7 @@
 #include <kicon.h>
 #include <knewstuff2/engine.h>
 #include <knewstuff2/ui/knewstuffaction.h>
+#include <KPageWidget>
 
 #include "keduvocdocument.h"
 #include "kwqtablemodel.h"
@@ -64,6 +66,8 @@ KWordQuizApp * KWordQuizApp::m_self = 0;
 KWordQuizApp::KWordQuizApp(QWidget*):KXmlGuiWindow(0)
 {
   m_self = this;
+  m_quiz = 0;
+  m_prefDialog = 0;
 
   initStatusBar();
   initActions();
@@ -75,20 +79,12 @@ KWordQuizApp::KWordQuizApp(QWidget*):KXmlGuiWindow(0)
   initView();
 
   m_dirWatch = KDirWatch::self();
-  m_quizType = KWQQuiz::QuizEditor;
-  m_quiz = 0;
-  m_flashView = 0;
-  m_multipleView = 0;
-  m_qaView = 0;
 
-  slotQuizEditor();
   slotUndoChange(i18n("Cannot &Undo"), false);
 
   QAction *a = actionCollection()->action(QString("mode_%1").arg(QString::number(Prefs::mode())));
   slotModeActionGroupTriggered(a);
 
-  m_prefDialog = 0;
-
   editMarkBlank->setEnabled(Prefs::enableBlanks());
   editUnmarkBlank->setEnabled(Prefs::enableBlanks());
 
@@ -276,72 +272,31 @@ void KWordQuizApp::initActions()
   vocabConfigLeitner->setEnabled(false);
   connect(vocabConfigLeitner, SIGNAL(triggered(bool)), this, SLOT(slotConfigLeitner()));
 
-  mode = actionCollection()->add<KActionMenu>("mode_0");
-  mode->setIcon(KIcon("mode1"));
-  mode->setText(i18n("Change Mode"));
-  mode->setIconText(i18n("Mode"));
-  mode->setWhatsThis(i18n("Changes the mode used in quiz sessions"));
-  mode->setToolTip(mode->whatsThis());
-  mode->setStatusTip(mode->whatsThis());
-  connect(mode, SIGNAL(triggered(bool)), this, SLOT(slotModeChange()));
+  m_modeActionMenu = actionCollection()->add<KActionMenu>("mode_0");
+  m_modeActionMenu->setIcon(KIcon("mode1"));
+  m_modeActionMenu->setText(i18n("Change Mode"));
+  m_modeActionMenu->setIconText(i18n("Mode"));
+  m_modeActionMenu->setWhatsThis(i18n("Changes the mode used in quiz sessions"));
+  m_modeActionMenu->setToolTip(m_modeActionMenu->whatsThis());
+  m_modeActionMenu->setStatusTip(m_modeActionMenu->whatsThis());
+  connect(m_modeActionMenu, SIGNAL(triggered(bool)), this, SLOT(slotModeChange()));
 
   m_modeActionGroup = new QActionGroup(this);
   connect(m_modeActionGroup, SIGNAL(triggered(QAction *)), this, SLOT(slotModeActionGroupTriggered(QAction *)));
 
-  mode1 = actionCollection()->addAction("mode_1");
-  mode1->setData(1);
-  mode1->setCheckable(true);
-  mode1->setIcon(KIcon("mode1"));
-  mode1->setText("");
-  mode1->setWhatsThis(i18n("Selects this mode"));
-  mode1->setToolTip(mode1->whatsThis());
-  mode1->setStatusTip(mode1->whatsThis());
-  mode->addAction(mode1);
-  m_modeActionGroup->addAction(mode1);
+  QAction * a;
 
-  mode2 = actionCollection()->addAction("mode_2");
-  mode2->setData(2);
-  mode2->setCheckable(true);
-  mode2->setIcon(KIcon("mode2"));
-  mode2->setText("");
-  mode2->setWhatsThis(i18n("Selects this mode"));
-  mode2->setToolTip(mode2->whatsThis());
-  mode2->setStatusTip(mode2->whatsThis());
-  mode->addAction(mode2);
-  m_modeActionGroup->addAction(mode2);
-
-  mode3 = actionCollection()->addAction("mode_3");
-  mode3->setData(3);
-  mode3->setCheckable(true);
-  mode3->setIcon(KIcon("mode3"));
-  mode3->setText("");
-  mode3->setWhatsThis(i18n("Selects this mode"));
-  mode3->setToolTip(mode3->whatsThis());
-  mode3->setStatusTip(mode3->whatsThis());
-  mode->addAction(mode3);
-  m_modeActionGroup->addAction(mode3);
-
-  mode4 = actionCollection()->addAction("mode_4");
-  mode4->setData(4);
-  mode4->setCheckable(true);
-  mode4->setIcon(KIcon("mode4"));
-  mode4->setText("");
-  mode4->setWhatsThis(i18n("Selects this mode"));
-  mode4->setToolTip(mode4->whatsThis());
-  mode4->setStatusTip(mode4->whatsThis());
-  mode->addAction(mode4);
-  m_modeActionGroup->addAction(mode4);
-
-  mode5 = actionCollection()->addAction("mode_5");
-  mode5->setData(5);
-  mode5->setCheckable(true);
-  mode5->setIcon(KIcon("mode5"));
-  mode5->setText("");
-  mode5->setWhatsThis(i18n("Selects this mode"));
-  mode5->setToolTip(mode5->whatsThis());
-  mode5->setStatusTip(mode5->whatsThis());
-  mode->addAction(mode5);
-  m_modeActionGroup->addAction(mode5);
+  for (int i = 1; i <=5; ++i) {
+    a = actionCollection()->addAction(QString("mode_%1").arg(QString::number(i)));
+    a->setData(i);
+    a->setCheckable(true);
+    a->setIcon(KIcon(QString("mode%1").arg(QString::number(i))));
+    a->setWhatsThis(i18n("Selects this mode"));
+    a->setToolTip(a->whatsThis());
+    a->setStatusTip(a->whatsThis());
+    m_modeActionMenu->addAction(a);
+    m_modeActionGroup->addAction(a);
+  }
 
   quizEditor = actionCollection()->addAction("quiz_editor");
   quizEditor->setIcon(KIcon("editor"));
@@ -448,87 +403,13 @@ void KWordQuizApp::initActions()
   charMapper = new QSignalMapper(this);
   connect(charMapper, SIGNAL(mapped(int)), this, SLOT(slotInsertChar(int)));
 
-  specialChar1 = actionCollection()->addAction("char_1") ;
-  specialChar1->setText(i18n("Special Character 1"));
-  specialChar1->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_1));
-  specialChar1->setWhatsThis(i18n("Insert this character"));
-  specialChar1->setToolTip(specialChar1->whatsThis());
-  specialChar1->setStatusTip(specialChar1->whatsThis());
-  connect(specialChar1, SIGNAL(triggered(bool)), charMapper, SLOT(map()));
-
-  specialChar2 = actionCollection()->addAction("char_2") ;
-  specialChar2->setText(i18n("Special Character 2"));
-  specialChar2->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_2));
-  specialChar2->setWhatsThis(i18n("Insert this character"));
-  specialChar2->setToolTip(specialChar2->whatsThis());
-  specialChar2->setStatusTip(specialChar2->whatsThis());
-  connect(specialChar2, SIGNAL(triggered(bool)), charMapper, SLOT(map()));
-
-  specialChar3 = actionCollection()->addAction("char_3") ;
-  specialChar3->setText(i18n("Special Character 3"));
-  specialChar3->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_3));
-  specialChar3->setWhatsThis(i18n("Insert this character"));
-  specialChar3->setToolTip(specialChar3->whatsThis());
-  specialChar3->setStatusTip(specialChar3->whatsThis());
-  connect(specialChar3, SIGNAL(triggered(bool)), charMapper, SLOT(map()));
-
-  specialChar4 = actionCollection()->addAction("char_4") ;
-  specialChar4->setText(i18n("Special Character 4"));
-  specialChar4->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_4));
-  specialChar4->setWhatsThis(i18n("Insert this character"));
-  specialChar4->setToolTip(specialChar4->whatsThis());
-  specialChar4->setStatusTip(specialChar4->whatsThis());
-  connect(specialChar4, SIGNAL(triggered(bool)), charMapper, SLOT(map()));
-
-  specialChar5 = actionCollection()->addAction("char_5") ;
-  specialChar5->setText(i18n("Special Character 5"));
-  specialChar5->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_5));
-  specialChar5->setWhatsThis(i18n("Insert this character"));
-  specialChar5->setToolTip(specialChar5->whatsThis());
-  specialChar5->setStatusTip(specialChar5->whatsThis());
-  connect(specialChar5, SIGNAL(triggered(bool)), charMapper, SLOT(map()));
-
-  specialChar6 = actionCollection()->addAction("char_6") ;
-  specialChar6->setText(i18n("Special Character 6"));
-  specialChar6->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_6));
-  specialChar6->setWhatsThis(i18n("Insert this character"));
-  specialChar6->setToolTip(specialChar6->whatsThis());
-  specialChar6->setStatusTip(specialChar6->whatsThis());
-  connect(specialChar6, SIGNAL(triggered(bool)), charMapper, SLOT(map()));
-
-  specialChar7 = actionCollection()->addAction("char_7") ;
-  specialChar7->setText(i18n("Special Character 7"));
-  specialChar7->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_7));
-  specialChar7->setWhatsThis(i18n("Insert this character"));
-  specialChar7->setToolTip(specialChar7->whatsThis());
-  specialChar7->setStatusTip(specialChar7->whatsThis());
-  connect(specialChar7, SIGNAL(triggered(bool)), charMapper, SLOT(map()));
-
-  specialChar8 = actionCollection()->addAction("char_8") ;
-  specialChar8->setText(i18n("Special Character 8"));
-  specialChar8->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_8));
-  specialChar8->setWhatsThis(i18n("Insert this character"));
-  specialChar8->setToolTip(specialChar8->whatsThis());
-  specialChar8->setStatusTip(specialChar8->whatsThis());
-  connect(specialChar8, SIGNAL(triggered(bool)), charMapper, SLOT(map()));
-
-  specialChar9 = actionCollection()->addAction("char_9") ;
-  specialChar9->setText(i18n("Special Character 9"));
-  specialChar9->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_9));
-  specialChar9->setWhatsThis(i18n("Insert this character"));
-  specialChar9->setToolTip(specialChar9->whatsThis());
-  specialChar9->setStatusTip(specialChar9->whatsThis());
-  connect(specialChar9, SIGNAL(triggered(bool)), charMapper, SLOT(map()));
-
-  charMapper->setMapping(specialChar1, 1);
-  charMapper->setMapping(specialChar2, 2);
-  charMapper->setMapping(specialChar3, 3);
-  charMapper->setMapping(specialChar4, 4);
-  charMapper->setMapping(specialChar5, 5);
-  charMapper->setMapping(specialChar6, 6);
-  charMapper->setMapping(specialChar7, 7);
-  charMapper->setMapping(specialChar8, 8);
-  charMapper->setMapping(specialChar9, 9);
+  for (int i = 1; i <=9; ++i) {
+    a = actionCollection()->addAction(QString("char_%1").arg(QString::number(i)));
+    a->setText(i18n("Special Character %1", QString::number(i)));
+    a->setShortcut(QKeySequence(QString("Ctrl+%1").arg(QString::number(i))));
+    connect(a, SIGNAL(triggered(bool)), charMapper, SLOT(map()));
+    charMapper->setMapping(a, i);
+  }
 
   updateSpecialCharIcons();
 
@@ -554,8 +435,15 @@ void KWordQuizApp::initStatusBar()
 void KWordQuizApp::initDocument()
 {
   m_doc = new KEduVocDocument(this);
-  m_doc->appendIdentifier(i18n("Column 1"));
-  m_doc->appendIdentifier(i18n("Column 2"));
+  ///@todo New kvtml documents have a locale setting which should contain the language they are written in.
+  /* it would be nice to support that somehow. get hot new stuff for KHangman and KAnagram already uses this. */
+  m_doc->appendIdentifier();
+  m_doc->identifier(0).setName(i18n("Column 1"));
+  // m_doc->identifier(0).setLocale("en");
+  m_doc->appendIdentifier();
+  m_doc->identifier(1).setName(i18n("Column 2"));
+  // m_doc->identifier(1).setLocale("en");
+
   for (int i=0; i<20; i++)
   {
     m_doc->appendEntry(new KEduVocExpression());
@@ -575,11 +463,14 @@ void KWordQuizApp::initModel()
 
 void KWordQuizApp::initView()
 {
-  QWidget * mainWidget = new QWidget(this);
-  setCentralWidget(mainWidget);
-  m_topLayout = new QVBoxLayout(mainWidget);
-  m_topLayout->setMargin(0);
-  m_topLayout->setSpacing(KDialog::spacingHint());
+  m_pageWidget = new KPageWidget(this);
+  m_pageWidget->setFaceType( KPageView::List );
+  setCentralWidget(m_pageWidget);
+  connect(m_pageWidget, SIGNAL(currentPageChanged(KPageWidgetItem *, KPageWidgetItem *)), this, SLOT(slotCurrentPageChanged(KPageWidgetItem *, KPageWidgetItem *)));
+
+  QVBoxLayout *editorLayout = new QVBoxLayout(this);
+  editorLayout->setMargin(0);
+  editorLayout->setSpacing(KDialog::spacingHint());
 
   m_searchLine = new KLineEdit(this);
   m_searchLine->show();
@@ -599,21 +490,52 @@ void KWordQuizApp::initView()
   layout->addWidget(label);
   layout->addWidget(m_searchLine);
 
-  m_tableView = new KWQTableView(centralWidget());
-  m_tableView->setFrameStyle(QFrame::NoFrame);
-  m_topLayout->addWidget(m_searchWidget);
-  m_topLayout->addWidget(m_tableView);
-
+  m_tableView = new KWQTableView(this);
+  editorLayout->addWidget(m_searchWidget);
+  editorLayout->addWidget(m_tableView);
   m_tableView->setModel(m_sortFilterModel);
   m_tableView->setColumnWidth(0, qvariant_cast<QSize>(m_tableModel->headerData(0, Qt::Horizontal, Qt::SizeHintRole)).width());
   m_tableView->setColumnWidth(1, qvariant_cast<QSize>(m_tableModel->headerData(1, Qt::Horizontal, Qt::SizeHintRole)).width());
   setCaption(m_doc->url().fileName(),false);
+  m_tableView->addActions(guiFactory()->container("editor_popup", this)->actions());
+  m_tableView->setContextMenuPolicy(Qt::ActionsContextMenu);
   connect(m_tableView, SIGNAL(undoChange(const QString&, bool )), this, SLOT(slotUndoChange(const QString&, bool)));
   connect(m_tableModel, SIGNAL(modelReset()), m_tableView, SLOT(slotModelReset()));
 
   m_searchWidget->setVisible(Prefs::showSearch());
   configShowSearchBar->setChecked(Prefs::showSearch());
 
+  m_editorView = new QWidget(this);
+  m_flashView = new FlashView(this);
+  m_multipleView = new MultipleView(this);
+  m_qaView = new QAView(this);
+
+  m_editorPage = m_pageWidget->addPage(m_editorView, i18n("Editor"));
+  m_editorPage->setIcon(KIcon("editor"));
+  m_editorPage->setHeader(i18n("Vocabulary Editor"));
+  m_editorPage->widget()->setLayout(editorLayout);
+
+  m_flashPage = m_pageWidget->addPage(m_flashView, i18n("Flashcard"));
+  m_flashPage->setIcon(KIcon("flash"));
+  m_flashPage->setHeader(i18n("Flashcard Quiz"));
+
+  m_multiplePage = m_pageWidget->addPage(m_multipleView, i18n("Multiple\nChoice"));
+  m_multiplePage->setIcon(KIcon("multiple"));
+  m_multiplePage->setHeader(i18n("Multiple Choice Quiz"));
+
+  m_qaPage = m_pageWidget->addPage(m_qaView, i18n("Q&A"));
+  m_qaPage->setIcon(KIcon("qa"));
+  m_qaPage->setHeader(i18n("Question and Answer Quiz"));
+
+  m_pageWidget->addAction(quizEditor);
+  m_pageWidget->addAction(quizFlash);
+  m_pageWidget->addAction(quizMultiple);
+  m_pageWidget->addAction(quizQA);
+
+  m_pageWidget->setCurrentPage(m_editorPage);
+  m_tableView->setFocus();
+  updateActions();
+
   m_doc->setModified(false);
 }
 
@@ -659,14 +581,17 @@ void KWordQuizApp::openDocumentFile(const KUrl& url)
 {
   slotStatusMsg(i18n("Opening file..."));
   if (!url.isEmpty()) {
-    m_doc->open(url);
-    m_tableModel->reset();
-    m_dirWatch->addFile(url.path());
-    setCaption(m_doc->url().fileName(), false);
-    fileOpenRecent->addUrl(url);
-    QAction *a = actionCollection()->action(QString("mode_%1").arg(QString::number(Prefs::mode())));
-    slotModeActionGroupTriggered(a);
-    m_doc->setModified(false);
+    int result = m_doc->open(url);
+    if (result == KEduVocDocument::NoError) {
+      m_tableModel->reset();
+      m_dirWatch->addFile(url.path());
+      setCaption(m_doc->url().fileName(), false);
+      fileOpenRecent->addUrl(url);
+      slotModeActionGroupTriggered(m_modeActionGroup->checkedAction());
+      m_doc->setModified(false);
+    }
+    else
+      KMessageBox::error(this, KEduVocDocument::errorDescription(result));
   }
   slotStatusMsg(i18n("Ready"));
 }
@@ -781,27 +706,6 @@ bool KWordQuizApp::queryExit()
   return true;
 }
 
-bool KWordQuizApp::checkSyntax(bool blanks)
-{
-  int errorCount = 0;
-
-  for (int r = 0; r < m_tableModel->rowCount(QModelIndex()); ++r)
-  {
-    QString s = m_tableModel->data(m_tableModel->index(r, 0, QModelIndex()), Qt::DisplayRole).toString();
-    if (s.length() > 0)
-      for (int i = 0; i <= s.length(); ++i)
-        if (s[i] == delim_start || s[i] == delim_end)
-          if (!m_tableView->checkForBlank(s, blanks))
-            errorCount++;
-    s = m_tableModel->data(m_tableModel->index(r, 1, QModelIndex()), Qt::DisplayRole).toString();
-    if (s.length() > 0)
-      for (int i = 0; i <= s.length(); ++i)
-        if (s[i] == delim_start || s[i] == delim_end)
-          if (!m_tableView->checkForBlank(s, blanks))
-            errorCount++;
-    }
-  return (errorCount == 0);
-}
 
 void KWordQuizApp::slotFileNew()
 {
@@ -904,7 +808,6 @@ void KWordQuizApp::slotFileSaveAs()
 
 bool KWordQuizApp::saveAsFileName( )
 {
-
   bool success = false;
 
   QString filter = KEduVocDocument::pattern(KEduVocDocument::Writing);
@@ -933,13 +836,19 @@ bool KWordQuizApp::saveAsFileName( )
       }
       else
       {
-        if (m_dirWatch ->contains(m_doc->url().path()))
-          m_dirWatch ->removeFile(m_doc->url().path());
-        m_doc->saveAs(url, KEduVocDocument::Automatic, QString("kwordquiz %1").arg(KWQ_VERSION));
-        m_dirWatch->addFile(url.path());
-        fileOpenRecent->addUrl(url);
-        setCaption(m_doc->url().fileName(), m_doc->isModified());
-        success = true;
+        if (m_dirWatch->contains(m_doc->url().path()))
+          m_dirWatch->removeFile(m_doc->url().path());
+        int result = m_doc->saveAs(url, KEduVocDocument::Automatic, QString("kwordquiz %1").arg(KWQ_VERSION));
+        if (result == KEduVocDocument::NoError) {
+          m_dirWatch->addFile(url.path());
+          fileOpenRecent->addUrl(url);
+          setCaption(m_doc->url().fileName(), m_doc->isModified());
+          success = true;
+        }
+        else {
+          KMessageBox::error(this, KEduVocDocument::errorDescription(result));
+          success = false;
+        }
       }
     }
   }
@@ -994,7 +903,7 @@ void KWordQuizApp::slotFileQuit()
   KMainWindow* w;
   if(!memberList().isEmpty())
   {
-  for (int i = 0; i < memberList().size(); ++i)
+    for (int i = 0; i < memberList().size(); ++i)
     {
     w = memberList().at(i);
       // only close the window if the closeEvent is accepted. If the user presses Cancel on the saveModified() dialog,
@@ -1163,170 +1072,143 @@ void KWordQuizApp::slotModeChange()
 void KWordQuizApp::slotQuizEditor()
 {
   slotStatusMsg(i18n("Starting editor session..."));
-  updateSession(KWQQuiz::QuizEditor);
+  kDebug() << "I'm here" << endl;
+  m_pageWidget->setCurrentPage(m_editorPage);
   slotStatusMsg(i18n("Ready"));
 }
 
 void KWordQuizApp::slotQuizFlash()
 {
   slotStatusMsg(i18n("Starting flashcard session..."));
-  updateSession(KWQQuiz::QuizFlashCard);
+  m_pageWidget->setCurrentPage(m_flashPage);
   slotStatusMsg(i18n("Ready"));
 }
 
 void KWordQuizApp::slotQuizMultiple()
 {
   slotStatusMsg(i18n("Starting multiple choice session..."));
-  updateSession(KWQQuiz::QuizMultipleChoice);
+  m_pageWidget->setCurrentPage(m_multiplePage);
   slotStatusMsg(i18n("Ready"));
 }
 
 void KWordQuizApp::slotQuizQA()
 {
   slotStatusMsg(i18n("Starting question & answer session..."));
-  updateSession(KWQQuiz::QuizQuestionAnswer);
+  m_pageWidget->setCurrentPage(m_qaPage);
   slotStatusMsg(i18n("Ready"));
 }
 
-void KWordQuizApp::updateSession(KWQQuiz::QuizType qt)
+void KWordQuizApp::slotCurrentPageChanged(KPageWidgetItem *current, KPageWidgetItem *before)
 {
+  Q_UNUSED(before);
   if (m_quiz != 0)
   {
     delete(m_quiz);
     m_quiz = 0;
   }
+  disconnect(quizCheck, 0, 0, 0);
+  disconnect(flashKnow, 0, 0, 0);
+  disconnect(flashDontKnow, 0, 0, 0);
+  disconnect(quizRestart, 0, 0, 0);
+  disconnect(quizRepeatErrors, 0, 0, 0);
+  disconnect(qaHint, 0, 0, 0);
 
-  switch( m_quizType ){
-    case KWQQuiz::QuizEditor:
-      //
-      break;
-    case KWQQuiz::QuizFlashCard:
-      if (m_flashView != 0)
-      {
-        delete(m_flashView);
-        m_flashView = 0;
-      }
-      break;
-    case KWQQuiz::QuizMultipleChoice:
-      if (m_multipleView != 0)
-      {
-        delete(m_multipleView);
-        m_multipleView = 0;
-      }
-      break;
-    case KWQQuiz::QuizQuestionAnswer:
-      if (m_qaView != 0)
-      {
-        delete(m_qaView);
-        m_qaView = 0;
-      }
-      break;
+  if (current == m_editorPage) {
+    m_tableView->setFocus();
+    m_searchWidget->setVisible(Prefs::showSearch());
   }
 
-  if ((qt != KWQQuiz::QuizEditor) && Prefs::enableBlanks()) {
-    if (!checkSyntax(true))
-      return;
+  else  if (current == m_flashPage) {
+    if (Prefs::enableBlanks()) {
+      if (!m_tableModel->checkSyntax())
+        m_pageWidget->setCurrentPage(m_editorPage);
+    }
+    m_quiz = new KWQQuiz(this);
+    m_quiz->setModel(m_sortFilterModel);
+    connect(m_quiz, SIGNAL(checkingAnswer(int )), m_tableView, SLOT(slotCheckedAnswer(int )));
+    m_quiz ->setQuizType(KWQQuiz::QuizFlashCard);
+    m_quiz->setQuizMode(Prefs::mode());
+    if (m_quiz->init())
+    {
+      connect(quizCheck, SIGNAL(triggered(bool)), m_flashView, SLOT(slotFlip()));
+      connect(flashKnow, SIGNAL(triggered(bool)), m_flashView, SLOT(slotKnow()));
+      connect(flashDontKnow, SIGNAL(triggered(bool)), m_flashView, SLOT(slotDontKnow()));
+      connect(quizRestart, SIGNAL(triggered(bool)), m_flashView, SLOT(slotRestart()));
+      connect(quizRepeatErrors, SIGNAL(triggered(bool)), m_flashView, SLOT(slotRepeat()));
+      connect(this, SIGNAL(settingsChanged()), m_flashView, SLOT(slotApplySettings()));
+
+      m_flashView->setQuiz(m_quiz);
+      m_flashView->init();
+      m_flashView->stackedWidget->setCurrentIndex(0);
+    }
+    else
+    {
+      delete(m_quiz);
+      m_quiz = 0;
+      m_flashView->stackedWidget->setCurrentIndex(1);;
+    }
   }
 
-  switch (qt){
-    case KWQQuiz::QuizEditor:
-      m_tableView->show();
-      m_tableView -> setFocus();
-      m_searchWidget->setVisible(Prefs::showSearch());
-      m_quizType = qt;
-      break;
-    case KWQQuiz::QuizFlashCard:
-      m_quiz = new KWQQuiz(this);
-      m_quiz->setModel(m_sortFilterModel);
-      connect(m_quiz, SIGNAL(checkingAnswer(int )), m_tableView, SLOT(slotCheckedAnswer(int )));
-      m_quiz ->setQuizType(KWQQuiz::QuizFlashCard);
-      m_quiz->setQuizMode(Prefs::mode());
-      if (m_quiz -> init())
-      {
-        m_tableView->hide();
-        m_searchWidget->hide();
-        m_flashView = new FlashView(this);
-        connect(quizCheck, SIGNAL(triggered(bool)), m_flashView, SLOT(slotFlip()));
-        connect(flashKnow, SIGNAL(triggered(bool)), m_flashView, SLOT(slotKnow()));
-        connect(flashDontKnow, SIGNAL(triggered(bool)), m_flashView, SLOT(slotDontKnow()));
-        connect(quizRestart, SIGNAL(triggered(bool)), m_flashView, SLOT(slotRestart()));
-        connect(quizRepeatErrors, SIGNAL(triggered(bool)), m_flashView, SLOT(slotRepeat()));
-        connect(this, SIGNAL(settingsChanged()), m_flashView, SLOT(slotApplySettings()));
-
-        m_flashView->setQuiz(m_quiz);
-        m_flashView->init();
-        m_flashView->show();
-        m_topLayout->addWidget(m_flashView);
-        m_quizType = qt;
-      }
-      else
-      {
-        delete(m_quiz);
-        m_quiz = 0;
-      }
-      break;
-    case KWQQuiz::QuizMultipleChoice:
-      m_quiz = new KWQQuiz(this);
-      m_quiz->setModel(m_sortFilterModel);
-      connect(m_quiz, SIGNAL(checkingAnswer(int )), m_tableView, SLOT(slotCheckedAnswer(int )));
-      m_quiz ->setQuizType(KWQQuiz::QuizMultipleChoice);
-      m_quiz->setQuizMode(Prefs::mode());
-      if (m_quiz -> init())
-      {
-        //m_tableView->saveCurrentSelection(true);
-        m_tableView->hide();
-        m_searchWidget->hide();
-        m_multipleView = new MultipleView(this);
-        connect(quizCheck, SIGNAL(triggered(bool)), m_multipleView, SLOT(slotCheck()));
-        connect(quizRestart, SIGNAL(triggered(bool)), m_multipleView, SLOT(slotRestart()));
-        connect(quizRepeatErrors, SIGNAL(triggered(bool)), m_multipleView, SLOT(slotRepeat()));
-        connect(this, SIGNAL(settingsChanged()), m_multipleView, SLOT(slotApplySettings()));
-
-        m_multipleView -> setQuiz(m_quiz);
-        m_multipleView ->init();
-        m_multipleView->show();
-        m_topLayout->addWidget(m_multipleView);
-        m_quizType = qt;
-      }
-      else
-      {
-        delete(m_quiz);
-        m_quiz = 0;
-      }
-      break;
-    case KWQQuiz::QuizQuestionAnswer:
-      m_quiz = new KWQQuiz(this);
-      m_quiz->setModel(m_sortFilterModel);
-      connect(m_quiz, SIGNAL(checkingAnswer(int )), m_tableView, SLOT(slotCheckedAnswer(int )));
-      m_quiz ->setQuizType(KWQQuiz::QuizQuestionAnswer);
-      m_quiz->setQuizMode(Prefs::mode());
-      if (m_quiz -> init())
-      {
-        //m_tableView->saveCurrentSelection(true);
-        m_tableView->hide();
-        m_searchWidget->hide();
-        m_qaView = new QAView(this);
-        connect(quizCheck, SIGNAL(triggered(bool)), m_qaView, SLOT(slotCheck()));
-        connect(qaHint, SIGNAL(triggered(bool)), m_qaView, SLOT(slotHint()));
-        connect(quizRestart, SIGNAL(triggered(bool)), m_qaView, SLOT(slotRestart()));
-        connect(quizRepeatErrors, SIGNAL(triggered(bool)), m_qaView, SLOT(slotRepeat()));
-        connect(this, SIGNAL(settingsChanged()), m_qaView, SLOT(slotApplySettings()));
-
-        m_qaView -> setQuiz(m_quiz);
-        m_qaView ->init();
-        m_qaView->show();
-        m_topLayout->addWidget(m_qaView);
-        m_quizType = qt;
-      }
-      else
-      {
-        delete(m_quiz);
-        m_quiz = 0;
-      }
-      break;
+  else if (current == m_multiplePage) {
+    if (Prefs::enableBlanks()) {
+      if (!m_tableModel->checkSyntax())
+        m_pageWidget->setCurrentPage(m_editorPage);
+    }
+    m_quiz = new KWQQuiz(this);
+    m_quiz->setModel(m_sortFilterModel);
+    connect(m_quiz, SIGNAL(checkingAnswer(int )), m_tableView, SLOT(slotCheckedAnswer(int )));
+    m_quiz ->setQuizType(KWQQuiz::QuizMultipleChoice);
+    m_quiz->setQuizMode(Prefs::mode());
+    if (m_quiz->init())
+    {
+      connect(quizCheck, SIGNAL(triggered(bool)), m_multipleView, SLOT(slotCheck()));
+      connect(quizRestart, SIGNAL(triggered(bool)), m_multipleView, SLOT(slotRestart()));
+      connect(quizRepeatErrors, SIGNAL(triggered(bool)), m_multipleView, SLOT(slotRepeat()));
+      connect(this, SIGNAL(settingsChanged()), m_multipleView, SLOT(slotApplySettings()));
 
+      m_multipleView->setQuiz(m_quiz);
+      m_multipleView->init();
+      m_multipleView->stackedWidget->setCurrentIndex(0);
+    }
+    else
+    {
+      delete(m_quiz);
+      m_quiz = 0;
+      m_multipleView->stackedWidget->setCurrentIndex(1);
+    }
   }
-  updateActions(m_quizType);
+
+  else if (current == m_qaPage) {
+    if (Prefs::enableBlanks()) {
+      if (!m_tableModel->checkSyntax())
+        m_pageWidget->setCurrentPage(m_editorPage);
+    }
+    m_quiz = new KWQQuiz(this);
+    m_quiz->setModel(m_sortFilterModel);
+    connect(m_quiz, SIGNAL(checkingAnswer(int )), m_tableView, SLOT(slotCheckedAnswer(int )));
+    m_quiz ->setQuizType(KWQQuiz::QuizQuestionAnswer);
+    m_quiz->setQuizMode(Prefs::mode());
+    if (m_quiz->init())
+    {
+      connect(quizCheck, SIGNAL(triggered(bool)), m_qaView, SLOT(slotCheck()));
+      connect(qaHint, SIGNAL(triggered(bool)), m_qaView, SLOT(slotHint()));
+      connect(quizRestart, SIGNAL(triggered(bool)), m_qaView, SLOT(slotRestart()));
+      connect(quizRepeatErrors, SIGNAL(triggered(bool)), m_qaView, SLOT(slotRepeat()));
+      connect(this, SIGNAL(settingsChanged()), m_qaView, SLOT(slotApplySettings()));
+
+      m_qaView->setQuiz(m_quiz);
+      m_qaView->init();
+      m_qaView->stackedWidget->setCurrentIndex(0);
+    }
+    else
+    {
+      delete(m_quiz);
+      m_quiz = 0;
+      m_qaView->stackedWidget->setCurrentIndex(1);
+    }
+  }
+
+  updateActions();
 }
 
 /** Configure notifications */
@@ -1356,12 +1238,15 @@ void KWordQuizApp::slotApplyPreferences()
   emit settingsChanged();
 }
 
-void KWordQuizApp::updateSpecialCharIcons( )
+void KWordQuizApp::updateSpecialCharIcons()
 {
   for (int i = 0; i < 9; i++){
-    QAction * act = actionCollection()->action(QString("char_" + QString::number(i + 1)));
-    act->setIcon(KIcon(charIcon(Prefs::specialCharacters()[i])));
-    act->setToolTip(i18n("Inserts the character %1", Prefs::specialCharacters()[i]));
+    QAction * a = actionCollection()->action(QString("char_" + QString::number(i + 1)));
+    a->setIcon(KIcon(charIcon(Prefs::specialCharacters()[i])));
+    a->setIconText(i18n("Insert %1", Prefs::specialCharacters()[i]));
+    a->setWhatsThis(i18n("Inserts the character %1", Prefs::specialCharacters()[i]));
+    a->setToolTip(a->whatsThis());
+    a->setStatusTip(a->whatsThis());
   }
 }
 
@@ -1419,11 +1304,11 @@ void KWordQuizApp::slotModeActionGroupTriggered(QAction *act)
     if (KMessageBox::warningContinueCancel(this, i18n("This will restart your quiz. Do you wish to continue?"), QString(), KStandardGuiItem::cont(), KStandardGuiItem::cancel(), "askModeQuiz")
       != KMessageBox::Continue)
       {
-        mode1->setChecked(Prefs::mode() == 1);
-        mode2->setChecked(Prefs::mode() == 2);
-        mode3->setChecked(Prefs::mode() == 3);
-        mode4->setChecked(Prefs::mode() == 4);
-        mode5->setChecked(Prefs::mode() == 5);
+        m_modeActionGroup->actions()[0]->setChecked(Prefs::mode() == 1);
+        m_modeActionGroup->actions()[1]->setChecked(Prefs::mode() == 2);
+        m_modeActionGroup->actions()[2]->setChecked(Prefs::mode() == 3);
+        m_modeActionGroup->actions()[3]->setChecked(Prefs::mode() == 4);
+        m_modeActionGroup->actions()[4]->setChecked(Prefs::mode() == 5);
         return;
       }
 
@@ -1432,14 +1317,14 @@ void KWordQuizApp::slotModeActionGroupTriggered(QAction *act)
   QString s1 = m_tableModel->headerData(0, Qt::Horizontal, Qt::DisplayRole).toString();
   QString s2 = m_tableModel->headerData(1, Qt::Horizontal, Qt::DisplayRole).toString();
 
-  mode1->setText(i18n("&1 %1 -> %2 In Order", s1, s2));
-  mode2->setText(i18n("&2 %1 -> %2 In Order", s2, s1));
-  mode3->setText(i18n("&3 %1 -> %2 Randomly", s1, s2));
-  mode4->setText(i18n("&4 %1 -> %2 Randomly", s2, s1));
-  mode5->setText(i18n("&5 %1 &lt;-&gt; %2 Randomly", s1, s2));
+  m_modeActionGroup->actions()[0]->setText(i18n("&1 %1 -> %2 In Order", s1, s2));
+  m_modeActionGroup->actions()[1]->setText(i18n("&2 %1 -> %2 In Order", s2, s1));
+  m_modeActionGroup->actions()[2]->setText(i18n("&3 %1 -> %2 Randomly", s1, s2));
+  m_modeActionGroup->actions()[3]->setText(i18n("&4 %1 -> %2 Randomly", s2, s1));
+  m_modeActionGroup->actions()[4]->setText(i18n("&5 %1 &lt;-&gt; %2 Randomly", s1, s2));
 
   Prefs::setMode(act->data().toInt());
-  mode->setIcon(KIcon("mode" + QString::number(Prefs::mode())));
+  m_modeActionMenu->setIcon(KIcon("mode" + QString::number(Prefs::mode())));
 
   switch (Prefs::mode()){
   case 1:
@@ -1460,31 +1345,24 @@ void KWordQuizApp::slotModeActionGroupTriggered(QAction *act)
   }
 
   if (m_quiz !=0)
-    updateSession(m_quizType);
+    slotCurrentPageChanged(m_pageWidget->currentPage(), m_pageWidget->currentPage());
 }
 
 
 void KWordQuizApp::slotInsertChar( int i )
 {
-  if (m_qaView != 0)
+  if (m_pageWidget->currentPage() == m_qaPage)
     m_qaView->slotSpecChar(Prefs::specialCharacters()[i - 1]);
   else
-    if (m_tableView->isVisible())
+    if (m_pageWidget->currentPage() == m_editorPage) {
       m_tableView->slotSpecChar(Prefs::specialCharacters()[i - 1]);
+    }
 }
 
 
-void KWordQuizApp::contextMenuEvent(QContextMenuEvent * event)
+void KWordQuizApp::updateActions()
 {
-  QWidget * w = guiFactory()->container("editor_popup", this);
-  KMenu *popup = static_cast<KMenu *>(w);
-
-  popup->exec( QPoint( event->globalX(), event->globalY() ) );
-}
-
-void KWordQuizApp::updateActions(KWQQuiz::QuizType qt)
-{
-  bool fEdit = (qt == KWQQuiz::QuizEditor);
+  bool fEdit = (m_pageWidget->currentPage() == m_editorPage);
 
   fileSave->setEnabled(fEdit);
   fileSaveAs->setEnabled(fEdit);
@@ -1502,19 +1380,14 @@ void KWordQuizApp::updateActions(KWQQuiz::QuizType qt)
   vocabAdjustRows->setEnabled(fEdit);
   vocabShuffle->setEnabled(fEdit);
 
-  quizEditor->setEnabled(qt != KWQQuiz::QuizEditor);
-  quizFlash->setEnabled(qt != KWQQuiz::QuizFlashCard);
-  quizMultiple->setEnabled(qt != KWQQuiz::QuizMultipleChoice);
-  quizQA->setEnabled(qt != KWQQuiz::QuizQuestionAnswer);
-
-  quizCheck->setEnabled(qt != KWQQuiz::QuizEditor);
-  quizRestart->setEnabled(qt != KWQQuiz::QuizEditor);
+  quizCheck->setEnabled(!fEdit);
+  quizRestart->setEnabled(!fEdit);
   quizRepeatErrors->setEnabled(false);
 
-  flashKnow->setEnabled(qt == KWQQuiz::QuizFlashCard);
-  flashDontKnow->setEnabled(qt == KWQQuiz::QuizFlashCard);
+  flashKnow->setEnabled(m_pageWidget->currentPage() == m_flashPage);
+  flashDontKnow->setEnabled(m_pageWidget->currentPage() == m_flashPage);
 
-  qaHint->setEnabled(qt == KWQQuiz::QuizQuestionAnswer);
+  qaHint->setEnabled(m_pageWidget->currentPage() == m_qaPage);
 
   configShowSearchBar->setEnabled(fEdit);
 
