@@ -2,7 +2,7 @@
                         Vocabulary Document for KDE Edu
     -----------------------------------------------------------------------
     copyright      : (C) 1999-2001 Ewald Arnold <kvoctrain@ewald-arnold.de>
-                     (C) 2001 The KDE-EDU team
+
                      (C) 2005-2007 Peter Hedlund <peter.hedlund@kdemail.net>
                      (C) 2007 Frederik Gladhorn <frederik.gladhorn@kdemail.net>
  ***************************************************************************/
@@ -25,7 +25,6 @@
 
 #include <klocale.h>
 #include <kdebug.h>
-#include <kmessagebox.h>
 #include <kio/netaccess.h>
 #include <krandomsequence.h>
 #include <kfilterdev.h>
@@ -60,17 +59,18 @@ public:
         init();
     }
 
+    ~KEduVocDocumentPrivate();
+
     void init();
 
     KEduVocDocument* q;
 
     bool                      m_dirty;
     KUrl                      m_url;
-    QList<bool>               m_sortIdentifier;
-    bool                      m_sortLesson;
 
     // save these to document
-    QStringList               m_identifiers;      //0= origin, 1,.. translations
+    QList<KEduVocIdentifier>  m_identifiers;
+
     int                       m_currentLesson;
     QList<int>                m_extraSizeHints;
     QList<int>                m_sizeHints;
@@ -79,7 +79,6 @@ public:
     QString                   m_queryorg;
     QString                   m_querytrans;
     QList<KEduVocExpression>  m_vocabulary;
-    QList<int>                m_lessonsInQuery;
 
     QStringList               m_tenseDescriptions;
     QSet<QString>             m_usages;
@@ -95,31 +94,28 @@ public:
       */
     QString                   m_category;
 
-    QList<KEduVocArticle>     m_articles;
-    QList<KEduVocConjugation> m_conjugations;
+    // A map is too error prone. Lesson order is very important.
+    QList<KEduVocLesson>      m_lessons;
 
-    // make this a map so removals don't require renumbering :)
-    QMap<int, KEduVocLesson*>  m_lessons;
-
-    KEduVocWordType*          m_wordTypes;
+    KEduVocWordType           m_wordTypes;
 
     LeitnerSystem*            m_leitnerSystem;
     bool                      m_activeLeitnerSystem;
 };
 
+KEduVocDocument::KEduVocDocumentPrivate::~KEduVocDocumentPrivate()
+{
+}
 
 void KEduVocDocument::KEduVocDocumentPrivate::init()
 {
     m_lessons.clear();
-    m_articles.clear();
     m_tenseDescriptions.clear();
     m_identifiers.clear();
-    m_sortIdentifier.clear();
     m_extraSizeHints.clear();
     m_sizeHints.clear();
     m_vocabulary.clear();
     m_dirty = false;
-    m_sortLesson = false;
     m_currentLesson = 0;
     m_queryorg = "";
     m_querytrans = "";
@@ -132,8 +128,6 @@ void KEduVocDocument::KEduVocDocumentPrivate::init()
     m_csvDelimiter = QString( '\t' );
     m_activeLeitnerSystem = false;
     m_leitnerSystem = NULL;
-
-    m_wordTypes = new KEduVocWordType();
 }
 
 
@@ -144,7 +138,6 @@ KEduVocDocument::KEduVocDocument( QObject *parent )
 
 KEduVocDocument::~KEduVocDocument()
 {
-    delete d->m_wordTypes;
     delete d;
 }
 
@@ -158,14 +151,23 @@ void KEduVocDocument::setModified( bool dirty )
 
 void KEduVocDocument::appendEntry( KEduVocExpression *expression )
 {
-    d->m_vocabulary.append( *expression );
-    setModified();
+    insertEntry(expression, d->m_vocabulary.count());
 }
 
 
 void KEduVocDocument::insertEntry( KEduVocExpression *expression, int index )
 {
     d->m_vocabulary.insert( index, *expression );
+
+    // now we need to go fix the entryids that are greater than index in the lessons
+    for (int i = 0; i < d->m_lessons.size(); ++i)
+    {
+        d->m_lessons[i].incrementEntriesAbove(index);
+    }
+    // if the expression is added and the lesson already exists (not at doc loading time, but added later) make sure it ends up in the lesson as well.
+    if ( expression->lesson() >= 0 && expression->lesson() < d->m_lessons.count() ) {
+        d->m_lessons[expression->lesson()].addEntry(index);
+    }
     setModified();
 }
 
@@ -173,59 +175,58 @@ void KEduVocDocument::insertEntry( KEduVocExpression *expression, int index )
 KEduVocDocument::FileType KEduVocDocument::detectFileType( const QString &fileName )
 {
     QIODevice * f = KFilterDev::deviceForFile( fileName );
-    if ( !f->open( QIODevice::ReadOnly ) )
+    if ( !f->open( QIODevice::ReadOnly ) ) {
+        kDebug() << "Warning, could not open QIODevice for file: " << fileName;
         return Csv;
-
-    QDataStream is( f );
-
-    qint8 c1, c2, c3, c4, c5;
-    is >> c1
-    >> c2
-    >> c3
-    >> c4
-    >> c5;  // guess filetype by first x bytes
-
-    QTextStream ts( f );
-    QString line;
-    QString line2;
-
-    line = ts.readLine();
-    if ( !ts.atEnd() )
-        line2 = ts.readLine();
-    line.prepend( c5 );
-    line.prepend( c4 );
-    line.prepend( c3 );
-    line.prepend( c2 );
-    line.prepend( c1 );
-
-    if ( !is.device()->isOpen() )
-        return KvdNone;
-
-    f->close();
-    if ( c1 == '<' && c2 == '?' && c3 == 'x' && c4 == 'm' && c5 == 'l' ) {
-        if ( line2.indexOf( "pauker", 0 ) >  0 )
-            return Pauker;
-        else if ( line2.indexOf( "xdxf", 0 ) >  0 )
-            return Xdxf;
-        else
-            return Kvtml;
     }
 
-    if ( line == WQL_IDENT )
-        return Wql;
+    QTextStream ts( f );
+    QString line1;
+    QString line2;
 
-    if ( c1 == '"' && ( line.contains( '"' ) == 1 || line.contains( QRegExp( "\",[0-9]" ) ) ) )
+    line1 = ts.readLine();
+    if ( !ts.atEnd() ) {
+        line2 = ts.readLine();
+    }
+
+    f->close();
+    if ( line1.startsWith(QString::fromLatin1("<?xml")) ) {
+        if ( line2.indexOf( "pauker", 0 ) >  0 ) {
+            return Pauker;
+        }
+        if ( line2.indexOf( "xdxf", 0 ) >  0 ) {
+            return Xdxf;
+        } else {
+            return Kvtml;
+        }
+    }
+
+    if ( line1 == WQL_IDENT ) {
+        return Wql;
+    }
+
+    /*
+    Vokabeln.de files:
+    The header seems to be something like this:
+    "Name
+    Lang1 - Lang2",123,234,456
+    */
+    if ( line1.startsWith(QChar::fromLatin1('"')) &&
+        ((line1.contains(QRegExp( "\",[0-9]" ))) ||
+        (line2.contains(QRegExp( "\",[0-9]" ))))) {
         return Vokabeln;
+    }
 
     return Csv;
 }
 
 
-bool KEduVocDocument::open( const KUrl& url )
+int KEduVocDocument::open( const KUrl& url )
 {
     d->init();
-    if ( !url.isEmpty() )
+    if ( !url.isEmpty() ) {
         d->m_url = url;
+    }
 
     bool read = false;
     QString errorMessage = i18n( "<qt>Cannot open file<br /><b>%1</b></qt>", url.path() );
@@ -234,86 +235,123 @@ bool KEduVocDocument::open( const KUrl& url )
         QIODevice * f = KFilterDev::deviceForFile( temporaryFile );
 
         if ( !f->open( QIODevice::ReadOnly ) ) {
-            KMessageBox::error( 0, errorMessage );
-            return false;
+            kError() << errorMessage;
+            return FileCannotRead;
         }
 
-        FileType ft = detectFileType( url.path() );
+        FileType ft = detectFileType( temporaryFile );
 
         switch ( ft ) {
             case Kvtml: {
+                kDebug() << "Reading KVTML document...";
                 KEduVocKvtml2Reader kvtmlReader( f );
                 read = kvtmlReader.readDoc( this );
-                if ( !read )
+                if ( !read ) {
                     errorMessage = kvtmlReader.errorMessage();
+                }
             }
             break;
 
             case Wql: {
+                kDebug() << "Reading WordQuiz (WQL) document...";
                 KEduVocWqlReader wqlReader( f );
                 d->m_url.setFileName( i18n( "Untitled" ) );
                 read = wqlReader.readDoc( this );
-                if ( !read )
+                if ( !read ) {
                     errorMessage = wqlReader.errorMessage();
+                }
             }
             break;
 
             case Pauker: {
+                kDebug() << "Reading Pauker document...";
                 KEduVocPaukerReader paukerReader( this );
                 d->m_url.setFileName( i18n( "Untitled" ) );
                 read = paukerReader.read( f );
-                if ( !read )
+                if ( !read ) {
                     errorMessage = i18n( "Parse error at line %1, column %2:\n%3", paukerReader.lineNumber(), paukerReader.columnNumber(), paukerReader.errorString() );
+                }
             }
             break;
 
             case Vokabeln: {
+                kDebug() << "Reading Vokabeln document...";
                 KEduVocVokabelnReader vokabelnReader( f );
                 d->m_url.setFileName( i18n( "Untitled" ) );
                 read = vokabelnReader.readDoc( this );
-                if ( !read )
+                if ( !read ) {
                     errorMessage = vokabelnReader.errorMessage();
+                }
             }
             break;
 
             case Csv: {
+                kDebug() << "Reading CVS document...";
                 KEduVocCsvReader csvReader( f );
                 read = csvReader.readDoc( this );
-                if ( !read )
+                if ( !read ) {
                     errorMessage = csvReader.errorMessage();
+                }
             }
             break;
 
             case Xdxf: {
+                kDebug() << "Reading XDXF document...";
                 KEduVocXdxfReader xdxfReader( this );
                 d->m_url.setFileName( i18n( "Untitled" ) );
                 read = xdxfReader.read( f );
-                if ( !read )
+                if ( !read ) {
                     errorMessage = i18n( "Parse error at line %1, column %2:\n%3", xdxfReader.lineNumber(), xdxfReader.columnNumber(), xdxfReader.errorString() );
+                }
             }
             break;
 
             default: {
+                kDebug() << "Reading KVTML document (fallback)...";
                 KEduVocKvtml2Reader kvtmlReader( f );
                 read = kvtmlReader.readDoc( this );
-                if ( !read )
+                if ( !read ) {
                     errorMessage = kvtmlReader.errorMessage();
+                }
             }
         }
 
         if ( !read ) {
             QString msg = i18n( "Could not open or properly read \"%1\"\n(Error reported: %2)", url.path(), errorMessage );
-            KMessageBox::error( 0, msg, i18n( "Error Opening File" ) );
+            kError() << msg << i18n( "Error Opening File" );
+            ///@todo make the readers return int, pass on the error message properly
+            return FileReaderFailed;
         }
 
         f->close();
         KIO::NetAccess::removeTempFile( temporaryFile );
     }
-    return read;
+
+    // Additional cleanup: Put entries without a lesson into a default lesson.
+    int defaultLessonNumber = appendLesson(i18n("Default Lesson"));
+    // now make sure we don't have any orphan entries (lesson -1)
+    for (int i = 0; i < entryCount(); ++i)
+    {
+        if (entry(i)->lesson() == -1)
+        {
+            entry(i)->setLesson(defaultLessonNumber);
+            lesson(defaultLessonNumber).addEntry(i);
+        }
+    }
+    if (lesson(defaultLessonNumber).entries().size() == 0)
+    {
+        removeLesson(defaultLessonNumber, DeleteEmptyLesson);
+    }
+
+    if ( !read ) {
+        return FileReaderFailed;
+    }
+
+    return 0;
 }
 
 
-bool KEduVocDocument::saveAs( const KUrl & url, FileType ft, const QString & generator )
+int KEduVocDocument::saveAs( const KUrl & url, FileType ft, const QString & generator )
 {
     KUrl tmp( url );
 
@@ -323,59 +361,55 @@ bool KEduVocDocument::saveAs( const KUrl & url, FileType ft, const QString & gen
         else if ( tmp.path().right( strlen( "." CSV_EXT ) ) == "." CSV_EXT )
             ft = Csv;
         else {
-            return false;
+            return FileTypeUnknown;
         }
     }
 
+    QFile f( tmp.path() );
+
+    if ( !f.open( QIODevice::WriteOnly ) ) {
+        kError() << i18n( "Cannot write to file %1", tmp.path() );
+        return FileCannotWrite;
+    }
+
     bool saved = false;
 
-    while ( !saved ) {
-        QFile f( tmp.path() );
-
-        if ( !f.open( QIODevice::WriteOnly ) ) {
-            KMessageBox::error( 0, i18n( "<qt>Cannot write to file<br /><b>%1</b></qt>", tmp.path() ) );
-            return false;
+    switch ( ft ) {
+        case Kvtml: {
+            // write version 2 file
+            KEduVocKvtml2Writer kvtmlWriter( &f );
+            saved = kvtmlWriter.writeDoc( this, generator );
         }
-
-        switch ( ft ) {
-            case Kvtml: {
-                // write version 2 file
-                KEduVocKvtml2Writer kvtmlWriter( &f );
-                saved = kvtmlWriter.writeDoc( this, generator );
-            }
-            break;
-            case Kvtml1: {
-                // write old version 1 file
-                KEduVocKvtmlWriter kvtmlWriter( &f );
-                saved = kvtmlWriter.writeDoc( this, generator );
-            }
-            break;
-            case Csv: {
-                KEduVocCsvWriter csvWriter( &f );
-                saved = csvWriter.writeDoc( this, generator );
-            }
-            break;
-            default: {
-                kError() << "kvcotrainDoc::saveAs(): unknown filetype" << endl;
-            }
-            break;
+        break;
+        case Kvtml1: {
+            // write old version 1 file
+            KEduVocKvtmlWriter kvtmlWriter( &f );
+            saved = kvtmlWriter.writeDoc( this, generator );
         }
-        f.close();
-
-        if ( !saved ) {
-            QString msg = i18n( "Could not save \"%1\"\nDo you want to try again?", tmp.path() );
-            int result = KMessageBox::warningContinueCancel( 0, msg, i18n( "Error Saving File" ), KGuiItem( i18n( "&Retry" ) ) );
-            if ( result == KMessageBox::Cancel )
-                return false;
+        break;
+        case Csv: {
+            KEduVocCsvWriter csvWriter( &f );
+            saved = csvWriter.writeDoc( this, generator );
         }
+        break;
+        default: {
+            kError() << "kvcotrainDoc::saveAs(): unknown filetype" << endl;
+        }
+        break;
+    } // switch
+
+    f.close();
+
+    if ( !saved ) {
+        kError() << "Error Saving File" << tmp.path();
+        return FileWriterFailed;
     }
+
     d->m_url = tmp;
     setModified( false );
-    return true;
+    return 0;
 }
 
-
-
 void KEduVocDocument::merge( KEduVocDocument *docToMerge, bool matchIdentifiers )
 {
     kDebug() << "Merging of docs is not implemented"; /// @todo IMPLEMENT ME
@@ -602,37 +636,41 @@ KEduVocExpression *KEduVocDocument::entry( int index )
 
 void KEduVocDocument::removeEntry( int index )
 {
-    if ( index >= 0 && index < d->m_vocabulary.size() )
+    if ( index >= 0 && index < d->m_vocabulary.size() ) {
         d->m_vocabulary.removeAt( index );
+    }
+
+    // now we need to go fix the entryids that are greater than index in the lessons
+    for (int i = 0; i < d->m_lessons.size(); ++i)
+    {
+        d->m_lessons[i].decrementEntriesAbove(index);
+    }
 }
 
 
-int KEduVocDocument::indexOfIdentifier( const QString &lang ) const
+int KEduVocDocument::indexOfIdentifier( const QString& name ) const
 {
-    QStringList::const_iterator first = d->m_identifiers.begin();
-    int count = 0;
-    while ( first != d->m_identifiers.end() ) {
-        if ( *first == lang )
-            return count;
-        first++;
-        count++;
+    for ( int i=0; i < d->m_identifiers.count(); i++ ) {
+        if ( d->m_identifiers.value(i).name() == name ) {
+            return i;
+        }
     }
     return -1;
 }
 
 
-QString KEduVocDocument::identifier( int index ) const
+KEduVocIdentifier& KEduVocDocument::identifier( int index )
 {
-    if ( index >= d->m_identifiers.size() || index < 0 )
-        return "";
-    else
-        return d->m_identifiers[index];
+    if ( index < 0 || index >= d->m_identifiers.size() ) {
+        kError() << " Error: Invalid identifier index: " << index;
+    }
+    return d->m_identifiers[index];
 }
 
 
-void KEduVocDocument::setIdentifier( int idx, const QString &id )
+void KEduVocDocument::setIdentifier( int idx, const KEduVocIdentifier &id )
 {
-    if ( idx < d->m_identifiers.size() && idx >= 0 ) {
+    if ( idx >= 0 && idx < d->m_identifiers.size() ) {
         d->m_identifiers[idx] = id;
     }
 }
@@ -669,64 +707,6 @@ void KEduVocDocument::setTenseDescriptions( const QStringList &names )
 }
 
 
-void KEduVocDocument::setConjugation( int idx, const KEduVocConjugation &con )
-{
-    if ( idx < 0 ) return;
-
-    // extend conjugation with empty elements
-    if ( d->m_conjugations.size() <= idx )
-        for ( int i = d->m_conjugations.size(); i < idx+1; i++ )
-            d->m_conjugations.append( KEduVocConjugation() );
-
-    d->m_conjugations[idx] = con;
-}
-
-
-int KEduVocDocument::conjugationCount() const
-{
-    return d->m_conjugations.count();
-}
-
-
-KEduVocConjugation KEduVocDocument::conjugation( int idx ) const
-{
-    if ( idx >= d->m_conjugations.size() || idx < 0 ) {
-        return KEduVocConjugation();
-    } else {
-        return d->m_conjugations[idx];
-    }
-}
-
-
-void KEduVocDocument::setArticle( int idx, const KEduVocArticle &art )
-{
-    if ( idx < 0 ) return;
-
-    // extend conjugation with empty elements
-    if ( d->m_articles.size() <= idx )
-        for ( int i = d->m_articles.size(); i < idx+1; i++ )
-            d->m_articles.append( KEduVocArticle() );
-
-    d->m_articles[idx] = art;
-}
-
-
-KEduVocArticle KEduVocDocument::article( int idx ) const
-{
-    if ( idx >= d->m_articles.size() || idx < 0 ) {
-        return KEduVocArticle();
-    } else {
-        return d->m_articles[idx];
-    }
-}
-
-
-int KEduVocDocument::articleCount() const
-{
-    return d->m_articles.count();
-}
-
-
 int KEduVocDocument::sizeHint( int idx ) const
 {
     if ( idx < 0 ) {
@@ -771,10 +751,14 @@ void KEduVocDocument::setSizeHint( int idx, const int width )
 
 void KEduVocDocument::removeIdentifier( int index )
 {
-    if ( index < d->m_identifiers.size() && index >= 1 ) {
+    if ( index < d->m_identifiers.size() && index >= 0 ) {
         d->m_identifiers.removeAt( index );
-        for ( int i = 0; i < d->m_vocabulary.count(); i++ )
+        for ( int i = 0; i < d->m_vocabulary.count(); i++ ) {
             d->m_vocabulary[i].removeTranslation( index );
+            for ( int j = index; j < d->m_identifiers.size(); j++ ) {
+                d->m_vocabulary[i].translation(j) = d->m_vocabulary[i].translation(j+1);
+            }
+        }
     }
 }
 
@@ -785,7 +769,7 @@ bool KEduVocDocument::isModified() const
 }
 
 
-bool KEduVocDocument::leitnerSystemActive()
+bool KEduVocDocument::leitnerSystemActive() const
 {
     return d->m_activeLeitnerSystem;
 }
@@ -863,95 +847,48 @@ int KEduVocDocument::identifierCount() const
     return d->m_identifiers.count();  // number of translations
 }
 
-int KEduVocDocument::appendIdentifier( const QString & id )
+int KEduVocDocument::appendIdentifier( const KEduVocIdentifier& id )
 {
+    int i = d->m_identifiers.size();
+kDebug() << "appendIdentifier: " << i << id.name() << id.locale();
     d->m_identifiers.append( id );
-    return d->m_identifiers.size() - 1;
-}
-
-//QString KEduVocDocument::lessonDescription(int idx) const
-//{
-//  if (idx == 0)
-//    return i18nc("@label:listbox","<placeholder>no lesson</placeholder>");
-
-//  if (idx <= 0 || idx > d->m_lessons.size() )
-//    return "";
-
-//  return d->m_lessons[idx-1].description();
-//}
-
-//int KEduVocDocument::lessonIndex(const QString &description) const
-//{
-//  return d->m_lessonDescriptions.indexOf(description) +1;
-//}
-
-
-int KEduVocDocument::addLesson( const QString &lessonName, int position )
-{
-    if ( position == -1 ) {
-        // no position was specified, so put it wherever there's a slot
-        position = 1;
-        while ( d->m_lessons.contains( position ) ) {
-            ++position;
+    if ( id.name().isEmpty() ) {
+        if ( i == 0 ) {
+            identifier(i).setName(i18nc("The name of the first language/column of vocabulary, if we have to guess it.", "Original"));
+        } else {
+            identifier(i).setName(i18nc( "The name of the second, third ... language/column of vocabulary, if we have to guess it.", "Translation %1", i ) );
         }
     }
 
-    KEduVocLesson *lesson = new KEduVocLesson;
-    lesson->setName( lessonName );
-    d->m_lessons.insert( position, lesson );
-    return position;
+    if ( i > 0 ) {
+        for (int i = 0; i < entryCount(); i++) {
+            entry(i)->translation(i).setType(entry(i)->translation(0).type());
+        }
+    }
+
+    return i;
 }
 
-QMap<int, KEduVocLesson *> KEduVocDocument::lessons() const
+
+int KEduVocDocument::appendLesson( const QString &lessonName, bool inQuery )
+{
+    KEduVocLesson lesson;
+    lesson.setName( lessonName );
+    lesson.setInQuery( inQuery );
+    d->m_lessons.append( lesson );
+    return d->m_lessons.count() - 1;
+}
+
+QList<KEduVocLesson> & KEduVocDocument::lessons() const
 {
     return d->m_lessons;
 }
 
-KEduVocLesson * KEduVocDocument::lesson( int index )
+KEduVocLesson & KEduVocDocument::lesson( int index )
 {
-    KEduVocLesson * retval( NULL );
-    if ( d->m_lessons.contains( index ) ) {
-        retval = d->m_lessons[index];
-    }
-    return retval;
+    return d->m_lessons[index];
 }
 
-//void KEduVocDocument::renameLesson(const int lessonIndex, const QString &lessonName)
-//{
-//  d->m_lessonDescriptions.replace(lessonIndex-1, lessonName); // counting from 1
-//}
-
-
-bool KEduVocDocument::lessonInQuery( int lessonIndex ) const
-{
-    return d->m_lessonsInQuery.contains( lessonIndex );
-}
-
-
-void KEduVocDocument::addLessonToQuery( int lessonIndex )
-{
-    if ( !lessonInQuery( lessonIndex ) )
-        d->m_lessonsInQuery.append( lessonIndex );
-}
-
-
-void KEduVocDocument::removeLessonFromQuery( int lessonIndex )
-{
-    if ( lessonInQuery( lessonIndex ) )
-        d->m_lessonsInQuery.removeAt( d->m_lessonsInQuery.indexOf( lessonIndex ) );
-}
-
-
-QList<int> KEduVocDocument::lessonsInQuery() const
-{
-    return d->m_lessonsInQuery;
-}
-
-
-void KEduVocDocument::setLessonsInQuery( const QList<int> &lesson_iq )
-{
-    d->m_lessonsInQuery = lesson_iq;
-}
 
 KUrl KEduVocDocument::url() const
 {
@@ -991,6 +928,16 @@ QString KEduVocDocument::documentComment() const
     return d->m_comment;
 }
 
+void KEduVocDocument::setCategory( const QString & category )
+{
+    d->m_category = category;
+}
+
+QString KEduVocDocument::category() const
+{
+    return d->m_category;
+    ///@todo make writer/reader use this
+}
 
 void KEduVocDocument::queryIdentifier( QString &org, QString &trans ) const
 {
@@ -1069,31 +1016,34 @@ void KEduVocDocument::setCurrentLesson( int lesson )
 QStringList KEduVocDocument::lessonNames() const
 {
     QStringList descriptions;
-    QList<KEduVocLesson*> lessonObjects = lessons().values();
-    for ( int i = 0; i < lessonObjects.count(); ++i ) {
-        descriptions.append( lessonObjects[i]->name() );
+    foreach ( KEduVocLesson lesson, d->m_lessons ) {
+        descriptions.append(lesson.name());
     }
     return descriptions;
 }
 
-
 int KEduVocDocument::lessonCount() const
 {
     return d->m_lessons.count();
 }
 
-bool KEduVocDocument::deleteLesson( int lessonIndex, int deleteMode )
-{  // too bad we count from one!
-    lessonIndex++;
-    for ( int ent = entryCount() - 1; ent  >= 0 ; ent-- ) {
-        if ( entry( ent )->lesson() == lessonIndex ) {
-            if ( deleteMode == DeleteEmptyLesson )
-                return false; // stop if there are vocabs left in the lesson
-            if ( deleteMode == DeleteEntriesAndLesson )
-                // delete entries of this lesson with this lesson
-                removeEntry( ent );
+bool KEduVocDocument::removeLesson( int lessonIndex, int deleteMode )
+{
+    if (deleteMode == DeleteEmptyLesson) {
+        if (d->m_lessons[lessonIndex].entryCount() > 0) {
+            return false; // stop if there are vocabs left in the lesson
         }
-    }//for entries
+    }
+    else if (deleteMode == DeleteEntriesAndLesson) {
+        while (d->m_lessons[lessonIndex].entryCount() > 0) {
+            // get the next entryid
+            int entry = d->m_lessons[lessonIndex].entries()[0];
+            // take it out of this lesson
+            d->m_lessons[lessonIndex].removeEntry(entry);
+            // delete the entry from the document
+            removeEntry(entry);
+        }
+    }
 
     // for all above this lesson number - reduce lesson by one.
     for ( int ent = 0; ent < entryCount(); ent++ ) {
@@ -1102,18 +1052,9 @@ bool KEduVocDocument::deleteLesson( int lessonIndex, int deleteMode )
         }
     } // reduce lesson
 
-    // finally just remove the lesson name
-    //d->m_lessonDescriptions.removeAt(lessonIndex-1); // because of the damned 0 arghh
+    // finally just remove the lesson
+    d->m_lessons.removeAt(lessonIndex);
 
-    int currentInQuery = d->m_lessonsInQuery.indexOf( lessonIndex );
-    if ( currentInQuery != -1 )
-        d->m_lessonsInQuery.removeAt( currentInQuery );
-
-    // move query entries
-    for ( int queryLesson = 0; queryLesson < d->m_lessonsInQuery.count(); queryLesson++ ) {
-        if ( d->m_lessonsInQuery.at( queryLesson ) > lessonIndex )
-            d->m_lessonsInQuery.replace( queryLesson, d->m_lessonsInQuery.at( queryLesson )-1 );
-    }
     return true;
 }
 
@@ -1270,7 +1211,7 @@ void KEduVocDocument::shuffle()
 }
 
 
-QString KEduVocDocument::pattern( Mode mode )
+QString KEduVocDocument::pattern( FileDialogMode mode )
 {
     static const struct SupportedFilter {
         bool reading;
@@ -1284,7 +1225,7 @@ QString KEduVocDocument::pattern( Mode mode )
                     { true, false, "*.xml.qz *.pau.gz", I18N_NOOP( "Pauker Lesson" ) },
                     { true, false, "*.voc", I18N_NOOP( "Vokabeltrainer" ) },
                     { true, false, "*.xdxf", I18N_NOOP( "XML Dictionary Exchange Format" ) },
-                    { true, true, "*.csv", I18N_NOOP( "Text" ) },
+                    { true, true, "*.csv", I18N_NOOP( "Comma Separated Values (CSV)" ) },
                     // last is marker for the end, do not remove it
                     { false, false, 0, 0 }
                 };
@@ -1297,18 +1238,45 @@ QString KEduVocDocument::pattern( Mode mode )
             allext.append( QLatin1String( filters[i].extensions ) );
         }
     }
-    newfilters.prepend( allext.join( " " ) + '|' + i18n( "All supported documents" ) );
+    if ( mode == Reading ) {
+        newfilters.prepend( allext.join( " " ) + '|' + i18n( "All supported documents" ) );
+    }
     return newfilters.join( "\n" );
 }
 
+QString KEduVocDocument::errorDescription( int errorCode )
+{
+    switch (errorCode) {
+    case NoError:
+        return i18n("No error found.");
 
-KEduVocWordType* KEduVocDocument::wordTypes()
+    case InvalidXml:
+        return i18n("Invalid XML in document.");
+    case FileTypeUnknown:
+        return i18n("Unknown file type.");
+    case FileCannotWrite:
+        return i18n("File is not writeable.");
+    case FileWriterFailed:
+        return i18n("File writer failed.");
+    case FileCannotRead:
+        return i18n("File is not readable.");
+    case FileReaderFailed:
+        return i18n("The file reader failed.");
+    case FileDoesNotExist:
+        return i18n("The file does not exist.");
+    case Unknown:
+    default:
+        return i18n("Unknown error.");
+    }
+}
+
+KEduVocWordType& KEduVocDocument::wordTypes()
 {
     return d->m_wordTypes;
 }
 
 
-QStringList KEduVocDocument::usages()
+QStringList KEduVocDocument::usages() const
 {
     return d->m_usages.values();
 }
@@ -1352,6 +1320,4 @@ void KEduVocDocument::removeUsage( const QString &name )
 }
 
 
-
-
 #include "keduvocdocument.moc"
