@@ -2,7 +2,7 @@
                      read a KEduVocDocument from a KVTML file
     -----------------------------------------------------------------------
     copyright           : (C) 1999-2001 Ewald Arnold <kvoctrain@ewald-arnold.de>
-                          (C) 2001 The KDE-EDU team
+
                           (C) 2005 Eric Pignet <eric at erixpage.com>
                           (C) 2007 Peter Hedlund <peter.hedlund@kdemail.net>
                           (C) 2007 Frederik Gladhorn <frederik.gladhorn@kdemail.net>
@@ -124,6 +124,12 @@ bool KEduVocKvtml2Reader::readInformation( QDomElement &informationElement )
         m_doc->setDocumentComment( currentElement.text() );
     }
 
+    // read the category
+    currentElement = informationElement.firstChildElement( KVTML_CATEGORY );
+    if ( !currentElement.isNull() ) {
+        m_doc->setCategory( currentElement.text() );
+    }
+
     return true;
 }
 
@@ -169,11 +175,6 @@ bool KEduVocKvtml2Reader::readGroups( QDomElement &domElementParent )
     groupElement = domElementParent.firstChildElement( KVTML_ENTRIES );
     if ( !groupElement.isNull() ) {
         QDomNodeList entryList = groupElement.elementsByTagName( KVTML_ENTRY );
-        if ( entryList.length() <= 0 ) {
-            m_errorMessage = i18n( "no entries found in 'entries' tag" );
-            return false; // at least one entry is required
-        }
-
         for ( int i = 0; i < entryList.count(); ++i ) {
             currentElement = entryList.item( i ).toElement();
             if ( currentElement.parentNode() == groupElement ) {
@@ -214,24 +215,21 @@ bool KEduVocKvtml2Reader::readIdentifier( QDomElement &identifierElement )
         return false;
     }
 
-    QDomElement currentElement = identifierElement.firstChildElement( KVTML_IDENTIFIERTYPE );
-    if ( !currentElement.isNull() ) {
-        // TODO: do something with the type
+    // generate empty identifiers in the doc
+    for ( int i = m_doc->identifierCount(); i <= id; i++ ) {
+        m_doc->appendIdentifier( KEduVocIdentifier() );
     }
 
+    // the first element, create the identifier, even if empty
+    QDomElement currentElement = identifierElement.firstChildElement( KVTML_NAME );
+    m_doc->identifier(id).setName( currentElement.text() );
+
     currentElement = identifierElement.firstChildElement( KVTML_LOCALE );
-    if ( !currentElement.isNull() ) {
-        // TODO: do we want to use this for the identifier, or the name?
-        int index = m_doc->appendIdentifier( currentElement.text() );
-        if ( index != id ) {
-            m_errorMessage = i18n( "identifiers out of order" );
-            return false;
-        }
-    }
+    m_doc->identifier(id).setLocale( currentElement.text() );
 
-    currentElement = identifierElement.firstChildElement( KVTML_NAME );
+    currentElement = identifierElement.firstChildElement( KVTML_IDENTIFIERTYPE );
     if ( !currentElement.isNull() ) {
-        // TODO: do something with the name
+        // TODO: do something with the type
     }
 
     currentElement = identifierElement.firstChildElement( KVTML_SIZEHINT );
@@ -247,9 +245,9 @@ bool KEduVocKvtml2Reader::readIdentifier( QDomElement &identifierElement )
 
     currentElement = identifierElement.firstChildElement( KVTML_PERSONALPRONOUNS );
     if ( !currentElement.isNull() ) {
-        KEduVocConjugation personalPronouns;
-        readConjugation( currentElement, personalPronouns );
-        m_doc->setConjugation( id, personalPronouns );
+        KEduVocPersonalPronoun personalPronoun;
+        readPersonalPronoun( currentElement, personalPronoun );
+        m_doc->identifier(id).setPersonalPronouns( personalPronoun );
     }
     return result;
 }
@@ -278,15 +276,15 @@ bool KEduVocKvtml2Reader::readEntry( QDomElement &entryElement )
         }
     }
 
-    currentElement = entryElement.firstChildElement( KVTML_INQUERY );
-    if ( !currentElement.isNull() ) {
-        // set the inquery information
-        if ( currentElement.text() == KVTML_TRUE ) {
-            expr.setInQuery( true );
-        } else {
-            expr.setInQuery( false );
-        }
-    }
+//     currentElement = entryElement.firstChildElement( KVTML_INQUERY );
+//     if ( !currentElement.isNull() ) {
+//         // set the inquery information
+//         if ( currentElement.text() == KVTML_TRUE ) {
+//             expr.setInQuery( true );
+//         } else {
+//             expr.setInQuery( false );
+//         }
+//     }
 
     currentElement = entryElement.firstChildElement( KVTML_SIZEHINT );
     if ( !currentElement.isNull() ) {
@@ -386,16 +384,15 @@ bool KEduVocKvtml2Reader::readTranslation( QDomElement &translationElement,
 
     // conjugations
     currentElement = translationElement.firstChildElement( KVTML_CONJUGATION );
-    KEduVocConjugation conjugation;
     while ( !currentElement.isNull() ) {
         // read any conjugations (NOTE: this will overwrite any conjugations of the same type for this
         // translation, as the type is used as the key
-        readConjugation( currentElement, conjugation );
+        QDomElement tenseElement = currentElement.firstChildElement( KVTML_TENSE );
+        QString tense = tenseElement.text();
+
+        readConjugation( currentElement, expr.translation(index).conjugation(tense) );
         currentElement = currentElement.nextSiblingElement( KVTML_CONJUGATION );
     }
-    if ( conjugation.entryCount() > 0 ) {
-        expr.translation( index ).setConjugation( conjugation );
-    }
 
     // grade elements
     currentElement = translationElement.firstChildElement( KVTML_GRADE );
@@ -424,13 +421,13 @@ bool KEduVocKvtml2Reader::readTranslation( QDomElement &translationElement,
     // image
     currentElement = translationElement.firstChildElement( KVTML_IMAGE );
     if ( !currentElement.isNull() ) {
-        // TODO: do something with the image
+        expr.translation( index ).setImageUrl( KUrl( m_doc->url(), currentElement.text() ) );
     }
 
     // sound
     currentElement = translationElement.firstChildElement( KVTML_SOUND );
     if ( !currentElement.isNull() ) {
-        // TODO: do something with the sound
+        expr.translation( index ).setSoundUrl( KUrl( m_doc->url(), currentElement.text() ) );
     }
 
     return true;
@@ -440,12 +437,11 @@ bool KEduVocKvtml2Reader::readLesson( QDomElement &lessonElement )
 {
     // NOTE: currently this puts an identifier into the last lesson it is in, once support for multiple lessons
     // is in the entry class, all lessons that include an entry will be in there
-    int lessonId = 0;
-
+    int lessonId;
     //<name>Lesson name</name>
     QDomElement currentElement = lessonElement.firstChildElement( KVTML_NAME );
     if ( !currentElement.isNull() ) {
-        lessonId = m_doc->addLesson( currentElement.text() );
+        lessonId = m_doc->appendLesson( currentElement.text() );
     } else {
         m_errorMessage = i18n( "each lesson must have a name" );
         return false;
@@ -453,11 +449,7 @@ bool KEduVocKvtml2Reader::readLesson( QDomElement &lessonElement )
 
     //<query>true</query>
     currentElement = lessonElement.firstChildElement( KVTML_QUERY );
-    if ( !currentElement.isNull() ) {
-        if ( currentElement.text() == KVTML_TRUE ) {
-            m_doc->addLessonToQuery( lessonId );
-        }
-    }
+    m_doc->lesson(lessonId).setInQuery(currentElement.text() == KVTML_TRUE);
 
     //<current>true</current>
     currentElement = lessonElement.firstChildElement( KVTML_CURRENT );
@@ -474,7 +466,7 @@ bool KEduVocKvtml2Reader::readLesson( QDomElement &lessonElement )
         // TODO: once we have a lesson class, add each of these entryids to the lesson
         // set this lesson for the given enty
         m_doc->entry( entryId )->setLesson( lessonId );
-        m_doc->lesson( lessonId )->addEntry( entryId );
+        m_doc->lesson( lessonId ).addEntry( entryId );
         currentElement = currentElement.nextSiblingElement( KVTML_ENTRYID );
     }
 
@@ -505,7 +497,7 @@ bool KEduVocKvtml2Reader::readArticle( QDomElement &articleElement, int identifi
     QString mal_indef = "";
     QString nat_indef = "";
 
-    QDomElement currentElement = articleElement.firstChildElement( KVTML_DEFINITE );
+    QDomElement currentElement = articleElement.firstChildElement( KVTML_SINGULAR ).firstChildElement( KVTML_DEFINITE );
     if ( !currentElement.isNull() )
     {
         QDomElement subElement = currentElement.firstChildElement( KVTML_MALE );
@@ -524,7 +516,7 @@ bool KEduVocKvtml2Reader::readArticle( QDomElement &articleElement, int identifi
         }
     }
 
-    currentElement = articleElement.firstChildElement( KVTML_INDEFINITE );
+    currentElement = articleElement.firstChildElement( KVTML_SINGULAR ).firstChildElement( KVTML_INDEFINITE );
     if ( !currentElement.isNull() )
     {
         QDomElement subElement = currentElement.firstChildElement( KVTML_MALE );
@@ -542,158 +534,12 @@ bool KEduVocKvtml2Reader::readArticle( QDomElement &articleElement, int identifi
             nat_indef = subElement.text();
         }
     }
-    m_doc->setArticle( identifierNum, KEduVocArticle( fem_def, fem_indef, mal_def, mal_indef, nat_def, nat_indef ) );
+    m_doc->identifier(identifierNum).setArticle( KEduVocArticle( fem_def, fem_indef, mal_def, mal_indef, nat_def, nat_indef ) );
 
     return true;
 }
 
 
-bool KEduVocKvtml2Reader::readConjugation( QDomElement &conjugElement, KEduVocConjugation &curr_conjug )
-/*
- <conjugation>
-  <tenseid>1</tenseid>
-  <type>regular</type>
-  <singular>
-    <first person></first person>
-    <second person></second person>
-    <third person>
-      <male></male>
-      <female></female>
-      <neutral></neutral>
-    </third person>
-  </singular>
-  <plural>
-    <first person></first person>
-    <second person></second person>
-    <third person>
-      <common></common>
-    </third person>
-  </plural>
- </conjugation>
-*/
-{
-    bool p3_common;
-    bool s3_common;
-    QString singfirst;
-    QString singsecond;
-    QString singthirdmale;
-    QString singthirdfemale;
-    QString singthirdneutral;
-    QString plurfirst;
-    QString plursecond;
-    QString plurthirdmale;
-    QString plurthirdfemale;
-    QString plurthirdneutral;
-    QString type;
-
-    QDomElement typeElement = conjugElement.firstChildElement( KVTML_CONJUGATIONTYPE );
-    if ( !typeElement.isNull() )
-    {
-        type = typeElement.text();
-    }
-
-    // TODO: add something here to link and/or store tense information
-
-    QDomElement currentGroup = conjugElement.firstChildElement( KVTML_SINGULAR );
-    if ( !currentGroup.isNull() )
-    {
-        QDomElement currentElement = currentGroup.firstChildElement( KVTML_1STPERSON );
-        if ( !currentElement.isNull() ) {
-            singfirst = currentElement.text();
-        }
-
-        currentElement = currentGroup.firstChildElement( KVTML_2NDPERSON );
-        if ( !currentElement.isNull() ) {
-            singsecond = currentElement.text();
-        }
-
-        currentGroup = currentGroup.firstChildElement( KVTML_3RDPERSON );
-        if ( !currentGroup.isNull() ) {
-            currentElement = currentGroup.firstChildElement( KVTML_COMMON );
-            if ( !currentElement.isNull() ) {
-                s3_common = true;
-                singthirdmale = currentElement.text();
-                singthirdfemale = singthirdmale;
-                singthirdneutral = singthirdmale;
-            } else {
-                s3_common = false;
-                currentElement = currentGroup.firstChildElement( KVTML_MALE );
-                if ( !currentElement.isNull() ) {
-                    singthirdmale = currentElement.text();
-                }
-
-                currentElement = currentGroup.firstChildElement( KVTML_FEMALE );
-                if ( !currentElement.isNull() ) {
-                    singthirdfemale = currentElement.text();
-                }
-
-                currentElement = currentGroup.firstChildElement( KVTML_NEUTRAL );
-                if ( !currentElement.isNull() ) {
-                    singthirdneutral = currentElement.text();
-                }
-            }
-
-        }
-    }
-
-    currentGroup = conjugElement.firstChildElement( KVTML_PLURAL );
-    if ( !currentGroup.isNull() )
-    {
-        QDomElement currentElement = currentGroup.firstChildElement( KVTML_1STPERSON );
-        if ( !currentElement.isNull() ) {
-            plurfirst = currentElement.text();
-        }
-
-        currentElement = currentGroup.firstChildElement( KVTML_2NDPERSON );
-        if ( !currentElement.isNull() ) {
-            plursecond = currentElement.text();
-        }
-
-        currentGroup = currentGroup.firstChildElement( KVTML_3RDPERSON );
-        if ( !currentGroup.isNull() ) {
-            currentElement = currentGroup.firstChildElement( KVTML_COMMON );
-            if ( !currentElement.isNull() ) {
-                p3_common = true;
-                plurthirdmale = currentElement.text();
-                plurthirdfemale = singthirdmale;
-                plurthirdneutral = singthirdmale;
-            } else {
-                p3_common = false;
-                currentElement = currentGroup.firstChildElement( KVTML_MALE );
-                if ( !currentElement.isNull() ) {
-                    plurthirdmale = currentElement.text();
-                }
-
-                currentElement = currentGroup.firstChildElement( KVTML_FEMALE );
-                if ( !currentElement.isNull() ) {
-                    plurthirdfemale = currentElement.text();
-                }
-
-                currentElement = currentGroup.firstChildElement( KVTML_NEUTRAL );
-                if ( !currentElement.isNull() ) {
-                    plurthirdneutral = currentElement.text();
-                }
-            }
-
-        }
-    }
-
-    curr_conjug.setPers3SingularCommon( type, s3_common );
-    curr_conjug.setPers3PluralCommon( type, p3_common );
-    curr_conjug.setPers1Singular( type, singfirst );
-    curr_conjug.setPers2Singular( type, singsecond );
-    curr_conjug.setPers3FemaleSingular( type, singthirdfemale );
-    curr_conjug.setPers3MaleSingular( type, singthirdmale );
-    curr_conjug.setPers3NaturalSingular( type, singthirdneutral );
-    curr_conjug.setPers1Plural( type, plurfirst );
-    curr_conjug.setPers2Plural( type, plursecond );
-    curr_conjug.setPers3FemalePlural( type, plurthirdfemale );
-    curr_conjug.setPers3MalePlural( type, plurthirdmale );
-    curr_conjug.setPers3NaturalPlural( type, plurthirdneutral );
-
-    return true;
-}
-
 bool KEduVocKvtml2Reader::readTypes( QDomElement &typesElement )
 {
     QString mainTypeName;
@@ -704,56 +550,49 @@ bool KEduVocKvtml2Reader::readTypes( QDomElement &typesElement )
         // set type and specialtype
         mainTypeName =
             currentTypeElement.firstChildElement( KVTML_TYPENAME ).text();
-        m_doc->wordTypes()->addType( mainTypeName,
-                                     currentTypeElement.firstChildElement( KVTML_SPECIALWORDTYPE ).text() );
+
+        QString specialType = currentTypeElement.firstChildElement( KVTML_SPECIALWORDTYPE ).text();
+        if ( !specialType.isEmpty() ) {
+            // get the localized version
+            if ( specialType == KVTML_SPECIALWORDTYPE_NOUN ) {
+                specialType = m_doc->wordTypes().specialTypeNoun();
+            }
+            if ( specialType == KVTML_SPECIALWORDTYPE_VERB ) {
+                specialType = m_doc->wordTypes().specialTypeVerb();
+            }
+            if ( specialType == KVTML_SPECIALWORDTYPE_ADVERB ) {
+                specialType = m_doc->wordTypes().specialTypeAdverb();
+            }
+            if ( specialType == KVTML_SPECIALWORDTYPE_ADJECTIVE ) {
+                specialType = m_doc->wordTypes().specialTypeAdjective();
+            }
+        }
+        m_doc->wordTypes().addType( mainTypeName, specialType );
 
         // iterate sub type elements <subwordtypedefinition>
-        QDomElement currentSubTypeElement =    currentTypeElement.firstChildElement( KVTML_SUBWORDTYPEDEFINITION );
+        QDomElement currentSubTypeElement = currentTypeElement.firstChildElement( KVTML_SUBWORDTYPEDEFINITION );
         while ( !currentSubTypeElement.isNull() ) {
+            QString specialSubType = currentSubTypeElement.firstChildElement( KVTML_SPECIALWORDTYPE ).text();
+            if ( !specialSubType.isEmpty() ) {
+                // get the localized version
+                if ( specialSubType == KVTML_SPECIALWORDTYPE_NOUN_MALE ) {
+                    specialSubType = m_doc->wordTypes().specialTypeNounMale();
+                }
+                if ( specialSubType == KVTML_SPECIALWORDTYPE_NOUN_FEMALE ) {
+                    specialSubType = m_doc->wordTypes().specialTypeNounFemale();
+                }
+                if ( specialSubType == KVTML_SPECIALWORDTYPE_NOUN_NEUTRAL ) {
+                    specialSubType = m_doc->wordTypes().specialTypeNounNeutral();
+                }
+            }
             // set type and specialtype
-            m_doc->wordTypes()->addSubType( mainTypeName,
+            m_doc->wordTypes().addSubType( mainTypeName,
                                             currentSubTypeElement.firstChildElement( KVTML_SUBTYPENAME ).text(),
-                                            currentSubTypeElement.firstChildElement( KVTML_SPECIALWORDTYPE ).text() );
-
-
+                                            specialSubType );
             currentSubTypeElement = currentSubTypeElement.nextSiblingElement( KVTML_SUBWORDTYPEDEFINITION );
         }
-
-
-
-
-
         currentTypeElement = currentTypeElement.nextSiblingElement( KVTML_WORDTYPEDEFINITION );
     }
-
-
-    /*
-    for (int i = 0; i < typeNodes.count(); ++i)
-    {
-      QDomElement currentElement = typeNodes.item(i).toElement();
-      if (currentElement.parentNode() == typesElement)
-      {
-        m_doc->wordTypes()->addType(currentElement.text());
-      }
-    }*/
-
-
-
-
-
-
-
-//   QDomNodeList typeNodes = typesElement.elementsByTagName(KVTML_WORDTYPEDEFINITION);
-//   for (int i = 0; i < typeNodes.count(); ++i)
-//   {
-//     QDomElement currentElement = typeNodes.item(i).toElement();
-//     if (currentElement.parentNode() == typesElement)
-//     {
-//       m_doc->wordTypes()->addType(currentElement.text());
-//     }
-//   }
-
-//   m_doc->setTypeDescriptions(types);
     return true;
 }
 
@@ -878,132 +717,138 @@ bool KEduVocKvtml2Reader::readGrade( QDomElement &gradeElement, KEduVocExpressio
 
     currentElement = gradeElement.firstChildElement( KVTML_DATE );
     if ( !currentElement.isNull() ) {
-        QDateTime value = QDateTime::fromTime_t( currentElement.text().toUInt() );
-        expr.translation( index ).gradeFrom( id ).setQueryDate( value );
+        QString dateString = currentElement.text();
+        if ( !dateString.isEmpty() ) {
+            QDateTime value = QDateTime::fromString( dateString, Qt::ISODate );
+            expr.translation( index ).gradeFrom( id ).setQueryDate( value );
+        }
     }
 
     return true;
 }
 
 
-//**** old code from kvtmlreader
-//  grade = KV_NORM_GRADE;
-//  rev_grade = KV_NORM_GRADE;
-//  attribute = domElementExpressionChild.attributeNode(KV_GRADE);
-//  if (!attribute.isNull())
-//  {
-//    QString s = attribute.value();
-//    if ((pos = s.indexOf(';')) >= 1)
-//    {
-//      grade = s.left(pos).toInt();
-//      rev_grade = s.mid(pos + 1, s.length()).toInt();
-//    }
-//    else
-//      grade = s.toInt();
-//  }
 
-//  count = 0;
-//  rev_count = 0;
-//  attribute = domElementExpressionChild.attributeNode(KV_COUNT);
-//  if (!attribute.isNull())
-//  {
-//    QString s = attribute.value();
-//    if ((pos = s.indexOf(';')) >= 1)
-//    {
-//      count = s.left(pos).toInt();
-//      rev_count = s.mid(pos + 1, s.length()).toInt();
-//    }
-//    else
-//      count = s.toInt();
-//  }
+bool KEduVocKvtml2Reader::readConjugation( QDomElement &conjugElement, KEduVocConjugation &conjugation )
+/*
+ <conjugation>
+  <tense>Futurepastperfekt:)</tense>
+  <singular>
+    <firstperson></firstperson>
+    <secondperson></secondperson>
+    <thirdperson>
+      <male></male>
+      <female></female>
+      <neutral></neutral>
+    </thirdperson>
+  </singular>
+  <plural>
+    <firstperson></firstperson>
+    <secondperson></secondperson>
+    <thirdsperson>
+      <common></common>
+    </third person>
+  </plural>
+ </conjugation>
+*/
+{
+    QDomElement personElement = conjugElement.firstChildElement( KVTML_SINGULAR );
+    if ( !personElement.isNull() )
+    {
+        readConjugationPerson( personElement, conjugation, KEduVocConjugation::Singular );
+    }
 
-//  bcount = 0;
-//  rev_bcount = 0;
-//  attribute = domElementExpressionChild.attributeNode(KV_BAD);
-//  if (!attribute.isNull())
-//  {
-//    QString s = attribute.value();
-//    if ((pos = s.indexOf(';')) >= 1)
-//    {
-//      bcount = s.left(pos).toInt();
-//      rev_bcount = s.mid(pos + 1, s.length()).toInt();
-//    }
-//    else
-//      bcount = s.toInt();
-//  }
+    personElement = conjugElement.firstChildElement( KVTML_DUAL );
+    if ( !personElement.isNull() )
+    {
+        readConjugationPerson( personElement, conjugation, KEduVocConjugation::Dual );
+    }
 
-//  date.setTime_t(0);
-//  rev_date.setTime_t(0);
-//  attribute = domElementExpressionChild.attributeNode(KV_DATE);
-//  if (!attribute.isNull())
-//  {
-//    QString s = attribute.value();
-//    if ((pos = s.indexOf(';')) >= 1)
-//    {
-//      date.setTime_t(s.left(pos).toInt());
-//      rev_date.setTime_t(s.mid(pos + 1, s.length()).toInt());
-//    }
-//    else
-//      date.setTime_t(s.toInt());
-//  }
+    personElement = conjugElement.firstChildElement( KVTML_PLURAL );
+    if ( !personElement.isNull() )
+    {
+        readConjugationPerson( personElement, conjugation, KEduVocConjugation::Plural );
+    }
 
-//  attribute = domElementExpressionChild.attributeNode(KV_DATE2);
-//  if (!attribute.isNull())
-//  {
-//    //this format is deprecated and ignored.
-//  }
+    return true;
+}
 
-//  faux_ami_f = "";
-//  attribute = domElementExpressionChild.attributeNode(KV_FAUX_AMI_F);
-//  if (!attribute.isNull())
-//    faux_ami_f = attribute.value();
 
-//  faux_ami_t = "";
-//  attribute = domElementExpressionChild.attributeNode(KV_FAUX_AMI_T);
-//  if (!attribute.isNull())
-//    faux_ami_t = attribute.value();
+bool KEduVocKvtml2Reader::readConjugationPerson(QDomElement & personElement, KEduVocConjugation & conjugation, KEduVocConjugation::ConjugationNumber number)
+{
+    QDomElement currentElement = personElement.firstChildElement( KVTML_1STPERSON );
+    conjugation.setConjugation( currentElement.text(),
+        KEduVocConjugation::First, number );
 
-//  usage = "";
-//  attribute = domElementExpressionChild.attributeNode(KV_USAGE);
-//  if (!attribute.isNull())
-//  {
-//    usage = attribute.value();
-//    if (usage.length() != 0 && usage.left(1) == UL_USER_USAGE)
-//    {
-//      int num = qMin(usage.mid (1, 40).toInt(), 1000); // paranioa check
-//      if (num > m_doc->usageDescriptions().count())
-//      {
-//        // description missing ?
-//        QStringList sl = m_doc->usageDescriptions();
-//        QString s;
-//        for (int i = m_doc->usageDescriptions().count(); i < num; i++)
-//        {
-//          s.setNum(i + 1);
-//          s.prepend("#");  // invent descr according to number
-//          sl.append(s);
-//        }
-//        m_doc->setUsageDescriptions(sl);
-//      }
-//    }
-//  }
+    currentElement = personElement.firstChildElement( KVTML_2NDPERSON );
+    conjugation.setConjugation( currentElement.text(),
+        KEduVocConjugation::Second, number );
+
+    currentElement = personElement.firstChildElement( KVTML_THIRD_MALE );
+    conjugation.setConjugation( currentElement.text(),
+        KEduVocConjugation::ThirdMale, number );
+
+    currentElement = personElement.firstChildElement( KVTML_THIRD_FEMALE );
+    conjugation.setConjugation( currentElement.text(),
+        KEduVocConjugation::ThirdFemale, number );
+
+    currentElement = personElement.firstChildElement( KVTML_THIRD_NEUTRAL_COMMON );
+    conjugation.setConjugation( currentElement.text(),
+        KEduVocConjugation::ThirdNeutralCommon, number );
+    return true;
+}
+
+
+bool KEduVocKvtml2Reader::readPersonalPronoun(QDomElement & pronounElement, KEduVocPersonalPronoun & pronoun)
+{
+    pronoun.setMaleFemaleDifferent(!pronounElement.firstChildElement(
+        KVTML_THIRD_PERSON_MALE_FEMALE_DIFFERENT).isNull());
+    pronoun.setNeutralExists( !pronounElement.firstChildElement(
+        KVTML_THIRD_PERSON_NEUTRAL_EXISTS).isNull() );
+    pronoun.setDualExists( !pronounElement.firstChildElement(
+        KVTML_DUAL_EXISTS).isNull() );
+
+    QDomElement personElement = pronounElement.firstChildElement( KVTML_SINGULAR );
+    if ( !personElement.isNull() ) {
+        readPersonalPronounChild( personElement, pronoun, KEduVocConjugation::Singular );
+    }
+
+    personElement = pronounElement.firstChildElement( KVTML_DUAL );
+    if ( !personElement.isNull() ) {
+        readPersonalPronounChild( personElement, pronoun, KEduVocConjugation::Dual );
+    }
+
+    personElement = pronounElement.firstChildElement( KVTML_PLURAL );
+    if ( !personElement.isNull() ) {
+        readPersonalPronounChild( personElement, pronoun, KEduVocConjugation::Plural );
+    }
+    return true;
+}
+
+
+bool KEduVocKvtml2Reader::readPersonalPronounChild(QDomElement & personElement, KEduVocPersonalPronoun & pronoun, KEduVocConjugation::ConjugationNumber number)
+{
+    QDomElement currentElement = personElement.firstChildElement( KVTML_1STPERSON );
+    pronoun.setPersonalPronoun( currentElement.text(),
+        KEduVocConjugation::First, number );
+
+    currentElement = personElement.firstChildElement( KVTML_2NDPERSON );
+    pronoun.setPersonalPronoun( currentElement.text(),
+        KEduVocConjugation::Second, number );
+
+    currentElement = personElement.firstChildElement( KVTML_THIRD_MALE );
+    pronoun.setPersonalPronoun( currentElement.text(),
+        KEduVocConjugation::ThirdMale, number );
+
+    currentElement = personElement.firstChildElement( KVTML_THIRD_FEMALE );
+    pronoun.setPersonalPronoun( currentElement.text(),
+        KEduVocConjugation::ThirdFemale, number );
+
+    currentElement = personElement.firstChildElement( KVTML_THIRD_NEUTRAL_COMMON );
+    pronoun.setPersonalPronoun( currentElement.text(),
+        KEduVocConjugation::ThirdNeutralCommon, number );
+    return true;
+}
 
-//    if (type.length() != 0 && type.left(1) == QM_USER_TYPE)
-//    {
-//      int num = qMin(type.mid (1, 40).toInt(), 1000); // paranoia check
-//      if (num > m_doc->typeDescriptions().count())
-//      {
-//        // description missing ?
-//        QString s;
-//        QStringList sl = m_doc->typeDescriptions();
-//        for (int i = m_doc->typeDescriptions().count(); i < num; i++)
-//        {
-//          s.setNum(i + 1);
-//          s.prepend("#");  // invent descr according to number
-//          sl.append(s);
-//        }
-//        m_doc->setTypeDescriptions(sl);
-//      }
-//    }
-//  }
 
 #include "keduvockvtml2reader.moc"
