@@ -128,6 +128,11 @@ bool KEduVocKvtml2Writer::writeInformation( QDomElement &informationElement, con
         informationElement.appendChild( newTextElement( KVTML_COMMENT, m_doc->documentComment() ) );
     }
 
+    // category
+    if ( !m_doc->category().isEmpty() ) {
+        informationElement.appendChild( newTextElement( KVTML_CATEGORY, m_doc->category() ) );
+    }
+
     return true;
 }
 
@@ -143,7 +148,9 @@ bool KEduVocKvtml2Writer::writeIdentifiers( QDomElement &identifiersElement )
 
         // record the identifier as the locale for now
         // TODO: when support for more parts of the identifier is in the document class (name, type, etc.) store those here as well
-        identifier.appendChild( newTextElement( KVTML_LOCALE, m_doc->identifier( i ) ) );
+        identifier.appendChild( newTextElement( KVTML_NAME, m_doc->identifier( i ).name() ) );
+
+        identifier.appendChild( newTextElement( KVTML_LOCALE, m_doc->identifier( i ).locale() ) );
 
         // record articles
         QDomElement article = m_domDoc.createElement( KVTML_ARTICLE );
@@ -154,7 +161,7 @@ bool KEduVocKvtml2Writer::writeIdentifiers( QDomElement &identifiersElement )
 
         // record personalpronouns
         QDomElement personalpronouns = m_domDoc.createElement( KVTML_PERSONALPRONOUNS );
-        writeConjugation( personalpronouns, m_doc->conjugation( i ), QString() );
+        writePersonalPronoun( personalpronouns, m_doc->identifier(i).personalPronouns() );
         if ( personalpronouns.hasChildNodes() ) {
             identifier.appendChild( personalpronouns );
         }
@@ -167,29 +174,26 @@ bool KEduVocKvtml2Writer::writeIdentifiers( QDomElement &identifiersElement )
 
 bool KEduVocKvtml2Writer::writeLessons( QDomElement &lessonsElement )
 {
-    if ( m_doc->lessonCount() == 0 )
-        return true;
-
-    QMap<int, KEduVocLesson*> lessons = m_doc->lessons();
-
-    foreach( int lessonid, lessons.keys() ) {
-        KEduVocLesson * thisLesson = lessons[lessonid];
-
+    for( int lessonId = 0; lessonId < m_doc->lessonCount(); lessonId++ ) {
         // make lesson element
         QDomElement thisLessonElement = m_domDoc.createElement( KVTML_LESSON );
 
         // add a name
-        thisLessonElement.appendChild( newTextElement( KVTML_NAME, thisLesson->name() ) );
+        thisLessonElement.appendChild( newTextElement( KVTML_NAME, m_doc->lesson(lessonId).name() ) );
 
         // add a inquery tag
-        thisLessonElement.appendChild( newTextElement( KVTML_QUERY, m_doc->lessonInQuery( lessonid ) ? KVTML_TRUE : KVTML_FALSE ) );
+        if ( m_doc->lesson(lessonId).inQuery() ) {
+            thisLessonElement.appendChild( newTextElement( KVTML_QUERY, KVTML_TRUE ) );
+        }
 
         // add a current tag
-        thisLessonElement.appendChild( newTextElement( KVTML_CURRENT, m_doc->currentLesson() == lessonid ? KVTML_TRUE : KVTML_FALSE ) );
+        if ( lessonId == m_doc->currentLesson() ) {
+            thisLessonElement.appendChild( newTextElement( KVTML_CURRENT, KVTML_TRUE ) );
+        }
 
         // TODO: add the entryids...
         for ( int i = 0; i < m_doc->entryCount(); ++i ) {
-            if ( m_doc->entry( i )->lesson() == lessonid ) {
+            if ( m_doc->entry( i )->lesson() == lessonId ) {
                 thisLessonElement.appendChild( newTextElement( KVTML_ENTRYID, QString::number( i ) ) );
             }
         }
@@ -203,84 +207,110 @@ bool KEduVocKvtml2Writer::writeLessons( QDomElement &lessonsElement )
 
 bool KEduVocKvtml2Writer::writeArticle( QDomElement &articleElement, int article )
 {
+    ///@todo: this is ugly as hell and writes only singular
+
+    QDomElement singular = m_domDoc.createElement( KVTML_SINGULAR );
+
     QDomElement definite = m_domDoc.createElement( KVTML_DEFINITE );
     QDomElement indefinite = m_domDoc.createElement( KVTML_INDEFINITE );
     QString def;
     QString indef;
 
     // male
-    m_doc->article( article ).getMale( &def, &indef );
-    if ( !def.isEmpty() ) {
-        definite.appendChild( newTextElement( KVTML_MALE, def ) );
+    QString articleString;
+    articleString = m_doc->identifier(article).article().article( KEduVocArticle::Singular, KEduVocArticle::Definite, KEduVocArticle::Masculine );
+    if ( !articleString.isEmpty() ) {
+        definite.appendChild( newTextElement( KVTML_MALE, articleString ) );
     }
-    if ( !indef.isEmpty() ) {
-        indefinite.appendChild( newTextElement( KVTML_MALE, indef ) );
+    articleString = m_doc->identifier(article).article().article( KEduVocArticle::Singular, KEduVocArticle::Indefinite, KEduVocArticle::Masculine );
+    if ( !articleString.isEmpty() ) {
+        indefinite.appendChild( newTextElement( KVTML_MALE, articleString ) );
     }
 
     // female
-    m_doc->article( article ).getFemale( &def, &indef );
-    if ( !def.isEmpty() ) {
-        definite.appendChild( newTextElement( KVTML_FEMALE, def ) );
+    articleString = m_doc->identifier(article).article().article( KEduVocArticle::Singular, KEduVocArticle::Definite, KEduVocArticle::Feminine );
+    if ( !articleString.isEmpty() ) {
+        definite.appendChild( newTextElement( KVTML_FEMALE, articleString ) );
     }
-    if ( !indef.isEmpty() ) {
-        indefinite.appendChild( newTextElement( KVTML_FEMALE, indef ) );
+    articleString = m_doc->identifier(article).article().article( KEduVocArticle::Singular, KEduVocArticle::Indefinite, KEduVocArticle::Feminine );
+    if ( !articleString.isEmpty() ) {
+        indefinite.appendChild( newTextElement( KVTML_FEMALE, articleString ) );
     }
 
     // neutral
-    m_doc->article( article ).getNatural( &def, &indef );
-    if ( !def.isEmpty() ) {
-        definite.appendChild( newTextElement( KVTML_NEUTRAL, def ) );
+    articleString = m_doc->identifier(article).article().article( KEduVocArticle::Singular, KEduVocArticle::Definite, KEduVocArticle::Neutral );
+    if ( !articleString.isEmpty() ) {
+        definite.appendChild( newTextElement( KVTML_NEUTRAL, articleString ) );
     }
-    if ( !indef.isEmpty() ) {
-        indefinite.appendChild( newTextElement( KVTML_NEUTRAL, indef ) );
+    articleString = m_doc->identifier(article).article().article( KEduVocArticle::Singular, KEduVocArticle::Indefinite, KEduVocArticle::Neutral );
+    if ( !articleString.isEmpty() ) {
+        indefinite.appendChild( newTextElement( KVTML_NEUTRAL, articleString ) );
     }
 
+
+
+
     if ( definite.hasChildNodes() ) {
-        articleElement.appendChild( definite );
+        singular.appendChild( definite );
     }
 
     if ( indefinite.hasChildNodes() ) {
-        articleElement.appendChild( indefinite );
+        singular.appendChild( indefinite );
+    }
+
+    if ( singular.hasChildNodes() ) {
+        articleElement.appendChild( singular );
     }
     return true;
 }
 
 bool KEduVocKvtml2Writer::writeTypes( QDomElement &typesElement )
 {
-    KEduVocWordType* wt = m_doc->wordTypes();
-    foreach( QString mainTypeName, wt->typeNameList() ) {
+    KEduVocWordType wt = m_doc->wordTypes();
+    foreach( QString mainTypeName, wt.typeNameList() ) {
         kDebug() << "Writing type: " << mainTypeName;
         QDomElement typeDefinitionElement = m_domDoc.createElement( KVTML_WORDTYPEDEFINITION );
         typeDefinitionElement.appendChild( newTextElement( KVTML_TYPENAME, mainTypeName ) );
-        if ( !wt->specialType( mainTypeName ).isEmpty() ) {
-            typeDefinitionElement.appendChild( newTextElement( KVTML_SPECIALWORDTYPE, wt->specialType( mainTypeName ) ) );
+
+        QString specialType = wt.specialType( mainTypeName );
+        if ( !specialType.isEmpty() ) {
+            // get the NOT localized version for the doc
+            if ( specialType == m_doc->wordTypes().specialTypeNoun() ) {
+                specialType = KVTML_SPECIALWORDTYPE_NOUN;
+            }
+            if ( specialType == m_doc->wordTypes().specialTypeVerb()) {
+                specialType =  KVTML_SPECIALWORDTYPE_VERB;
+            }
+            if ( specialType == m_doc->wordTypes().specialTypeAdverb()) {
+                specialType = KVTML_SPECIALWORDTYPE_ADVERB;
+            }
+            if ( specialType ==  m_doc->wordTypes().specialTypeAdjective()) {
+                specialType = KVTML_SPECIALWORDTYPE_ADJECTIVE;
+            }
+            typeDefinitionElement.appendChild( newTextElement( KVTML_SPECIALWORDTYPE, specialType ) );
         }
 
         // subtypes
-        foreach( QString subTypeName, wt->subTypeNameList( mainTypeName ) ) {
+        foreach( QString subTypeName, wt.subTypeNameList( mainTypeName ) ) {
             QDomElement subTypeDefinitionElement = m_domDoc.createElement( KVTML_SUBWORDTYPEDEFINITION );
             subTypeDefinitionElement.appendChild( newTextElement( KVTML_SUBTYPENAME, subTypeName ) );
-            if ( !wt->specialSubType( mainTypeName, subTypeName ).isEmpty() ) {
-                subTypeDefinitionElement.appendChild( newTextElement( KVTML_SPECIALWORDTYPE, wt->specialSubType( mainTypeName, subTypeName ) ) );
+            QString specialSubType = wt.specialSubType( mainTypeName, subTypeName );
+            if ( !specialSubType.isEmpty() ) {
+                if ( specialSubType == m_doc->wordTypes().specialTypeNounMale() ) {
+                    specialSubType = KVTML_SPECIALWORDTYPE_NOUN_MALE;
+                }
+                if ( specialSubType == m_doc->wordTypes().specialTypeNounFemale() ) {
+                    specialSubType = KVTML_SPECIALWORDTYPE_NOUN_FEMALE;
+                }
+                if ( specialSubType == m_doc->wordTypes().specialTypeNounNeutral() ) {
+                    specialSubType = KVTML_SPECIALWORDTYPE_NOUN_NEUTRAL;
+                }
+                subTypeDefinitionElement.appendChild( newTextElement( KVTML_SPECIALWORDTYPE, specialSubType ) );
             }
             typeDefinitionElement.appendChild( subTypeDefinitionElement );
         }
         typesElement.appendChild( typeDefinitionElement );
     }
-
-
-
-
-
-    /*
-      foreach(QString type, m_doc->typeDescriptions())
-      {
-        if (!(type.isNull()) )
-        {
-          typesElement.appendChild(newTextElement(KVTML_WORDTYPE, type));
-        }
-      }*/
-
     return true;
 }
 
@@ -321,8 +351,9 @@ bool KEduVocKvtml2Writer::writeEntries( QDomElement &entriesElement )
         // write inactive
         entryElement.appendChild( newTextElement( KVTML_INACTIVE, thisEntry->isActive() ? KVTML_FALSE : KVTML_TRUE ) );
 
-        // write inquery
-        entryElement.appendChild( newTextElement( KVTML_INQUERY, thisEntry->isInQuery() ? KVTML_TRUE : KVTML_FALSE ) );
+// kvtml 1 relic no longer used
+//         // write inquery
+//         entryElement.appendChild( newTextElement( KVTML_INQUERY, thisEntry->isInQuery() ? KVTML_TRUE : KVTML_FALSE ) );
 
         // write sizehint
         if ( thisEntry->sizeHint() > 0 ) {
@@ -422,20 +453,17 @@ bool KEduVocKvtml2Writer::writeTranslation( QDomElement &translationElement, KEd
             gradeElement.appendChild( newTextElement( KVTML_ERRORCOUNT, QString::number( thisGrade.badCount() ) ) );
 
             //<date>949757271</date>
-            gradeElement.appendChild( newTextElement( KVTML_DATE, QString::number( thisGrade.queryDate().toTime_t() ) ) );
+            gradeElement.appendChild( newTextElement( KVTML_DATE,  thisGrade.queryDate().toString( Qt::ISODate ) ) );
 
             translationElement.appendChild( gradeElement );
         }
     }
 
     // conjugation
-    if ( translation.conjugation().entryCount() <= 0 ) {
-        KEduVocConjugation conjugation = translation.conjugation();
-        for ( int i = 0; i < conjugation.entryCount(); ++i ) {
-            QDomElement thisElement = m_domDoc.createElement( KVTML_CONJUGATION );
-            writeConjugation( thisElement, conjugation, conjugation.getType( i ) );
-            translationElement.appendChild( thisElement );
-        }
+    foreach ( QString tense, translation.conjugationTenses() ) {
+        QDomElement thisElement = m_domDoc.createElement( KVTML_CONJUGATION );
+        writeConjugation( thisElement, translation.conjugation(tense), tense );
+        translationElement.appendChild( thisElement );
     }
 
     // comparison
@@ -453,7 +481,28 @@ bool KEduVocKvtml2Writer::writeTranslation( QDomElement &translationElement, KEd
     }
 
     // image
+    if ( !translation.imageUrl().isEmpty() ) {
+        QString urlString;
+        if ( translation.imageUrl().url().startsWith(m_doc->url().upUrl().url()) ) {
+            // try to save as relative url
+            urlString = KUrl::relativeUrl( m_doc->url() , translation.imageUrl() );
+        } else {
+            urlString =  translation.imageUrl().url();
+        }
+        translationElement.appendChild( newTextElement( KVTML_IMAGE, urlString ) );
+    }
+
     // sound
+    if ( !translation.soundUrl().isEmpty() ) {
+        QString urlString;
+        if ( translation.soundUrl().url().startsWith(m_doc->url().upUrl().url()) ) {
+            // try to save as relative url
+            urlString = KUrl::relativeUrl( m_doc->url() , translation.soundUrl() );
+        } else {
+            urlString =  translation.soundUrl().url();
+        }
+        translationElement.appendChild( newTextElement( KVTML_SOUND, urlString ) );
+    }
 
     return true;
 }
@@ -495,56 +544,46 @@ bool KEduVocKvtml2Writer::writeMultipleChoice( QDomElement &multipleChoiceElemen
 }
 
 bool KEduVocKvtml2Writer::writeConjugation( QDomElement &conjugationElement,
-        const KEduVocConjugation &conjugation, const QString &type )
+        const KEduVocConjugation &conjugation, const QString &tense )
 {
-    // first singular conjugations
-    QString first = conjugation.pers1Singular( type );
-    QString second = conjugation.pers2Singular( type );
-    bool third_common = conjugation.pers3SingularCommon( type );
-    QString third_male = conjugation.pers3MaleSingular( type );
-    QString third_female = conjugation.pers3FemaleSingular( type );
-    QString third_neutral = conjugation.pers3NaturalSingular( type );
+    // write the tense tag
+    conjugationElement.appendChild( newTextElement(KVTML_TENSE, tense) );
 
-    if ( !first.isEmpty() || !second.isEmpty() || !third_female.isEmpty() ||
-            !third_male.isEmpty() || !third_neutral.isEmpty() ) {
-        QDomElement singular = m_domDoc.createElement( KVTML_SINGULAR );
+    for ( KEduVocConjugation::ConjugationNumber num = KEduVocConjugation::Singular; num < KEduVocConjugation::NumberMAX; num = KEduVocConjugation::ConjugationNumber(num +1) ) {
+        QString first = conjugation.conjugation(
+            KEduVocConjugation::First, num );
+        QString second = conjugation.conjugation(
+            KEduVocConjugation::Second, num );
+        QString third_male = conjugation.conjugation(
+            KEduVocConjugation::ThirdMale, num );
+        QString third_female = conjugation.conjugation(
+            KEduVocConjugation::ThirdFemale, num );
+        QString third_neutral = conjugation.conjugation(
+            KEduVocConjugation::ThirdNeutralCommon, num );
 
-        singular.appendChild( newTextElement( KVTML_1STPERSON, first ) );
-        singular.appendChild( newTextElement( KVTML_2NDPERSON, second ) );
+        if ( !first.isEmpty() || !second.isEmpty() || !third_female.isEmpty() ||
+                !third_male.isEmpty() || !third_neutral.isEmpty() ) {
+            QDomElement number;
+            switch (num) {
+            case KEduVocConjugation::Singular:
+                number = m_domDoc.createElement( KVTML_SINGULAR );
+                break;
+            case KEduVocConjugation::Dual:
+                number = m_domDoc.createElement( KVTML_DUAL );
+                break;
+            case KEduVocConjugation::Plural:
+                number = m_domDoc.createElement( KVTML_PLURAL );
+                break;
+            }
 
-        if ( third_common ) {
-            singular.appendChild( newTextElement( KVTML_COMMON, third_female ) );
-        } else {
-            singular.appendChild( newTextElement( KVTML_MALE, third_male ) );
-            singular.appendChild( newTextElement( KVTML_FEMALE, third_female ) );
-            singular.appendChild( newTextElement( KVTML_NEUTRAL, third_neutral ) );
+            number.appendChild( newTextElement( KVTML_1STPERSON, first ) );
+            number.appendChild( newTextElement( KVTML_2NDPERSON, second ) );
+            number.appendChild( newTextElement( KVTML_THIRD_MALE, third_male ) );
+            number.appendChild( newTextElement( KVTML_THIRD_FEMALE, third_female ) );
+            number.appendChild( newTextElement( KVTML_THIRD_NEUTRAL_COMMON, third_neutral ) );
+
+            conjugationElement.appendChild( number );
         }
-        conjugationElement.appendChild( singular );
-    }
-
-    // now for plurals
-    first = conjugation.pers1Plural( type );
-    second = conjugation.pers2Plural( type );
-    third_common = conjugation.pers3PluralCommon( type );
-    third_male = conjugation.pers3MalePlural( type );
-    third_female = conjugation.pers3FemalePlural( type );
-    third_neutral = conjugation.pers3NaturalPlural( type );
-
-    if ( !first.isEmpty() || !second.isEmpty() || !third_female.isEmpty() ||
-            !third_male.isEmpty() || !third_neutral.isEmpty() ) {
-        QDomElement plural = m_domDoc.createElement( KVTML_PLURAL );
-
-        plural.appendChild( newTextElement( KVTML_1STPERSON, first ) );
-        plural.appendChild( newTextElement( KVTML_2NDPERSON, second ) );
-
-        if ( third_common ) {
-            plural.appendChild( newTextElement( KVTML_COMMON, third_female ) );
-        } else {
-            plural.appendChild( newTextElement( KVTML_MALE, third_male ) );
-            plural.appendChild( newTextElement( KVTML_FEMALE, third_female ) );
-            plural.appendChild( newTextElement( KVTML_NEUTRAL, third_neutral ) );
-        }
-        conjugationElement.appendChild( plural );
     }
 
     return true;
@@ -557,3 +596,61 @@ QDomElement KEduVocKvtml2Writer::newTextElement( const QString &elementName, con
     retval.appendChild( textNode );
     return retval;
 }
+
+bool KEduVocKvtml2Writer::writePersonalPronoun(QDomElement & pronounElement, const KEduVocPersonalPronoun & pronoun)
+{
+    // general pronoun properties
+    if ( pronoun.maleFemaleDifferent() ) {
+        pronounElement.appendChild( m_domDoc.createElement( KVTML_THIRD_PERSON_MALE_FEMALE_DIFFERENT ) );
+    }
+    if ( pronoun.neutralExists() ) {
+        pronounElement.appendChild( m_domDoc.createElement( KVTML_THIRD_PERSON_NEUTRAL_EXISTS ) );
+    }
+    if ( pronoun.dualExists() ) {
+        pronounElement.appendChild( m_domDoc.createElement( KVTML_DUAL_EXISTS ) );
+    }
+
+    for ( KEduVocConjugation::ConjugationNumber num = KEduVocConjugation::Singular; num < KEduVocConjugation::NumberMAX; num = KEduVocConjugation::ConjugationNumber(num +1) ) {
+        QString first = pronoun.personalPronoun(
+            KEduVocConjugation::First, num );
+        QString second = pronoun.personalPronoun(
+            KEduVocConjugation::Second, num );
+        QString third_male = pronoun.personalPronoun(
+            KEduVocConjugation::ThirdMale, num );
+        QString third_female = pronoun.personalPronoun(
+            KEduVocConjugation::ThirdFemale, num );
+        QString third_neutral = pronoun.personalPronoun(
+            KEduVocConjugation::ThirdNeutralCommon, num );
+
+        if ( !first.isEmpty() || !second.isEmpty() || !third_female.isEmpty() ||
+                !third_male.isEmpty() || !third_neutral.isEmpty() ) {
+            QDomElement number;
+            switch (num) {
+            case KEduVocConjugation::Singular:
+                number = m_domDoc.createElement( KVTML_SINGULAR );
+                break;
+            case KEduVocConjugation::Dual:
+                number = m_domDoc.createElement( KVTML_DUAL );
+                break;
+            case KEduVocConjugation::Plural:
+                number = m_domDoc.createElement( KVTML_PLURAL );
+                break;
+            }
+
+            number.appendChild( newTextElement( KVTML_1STPERSON, first ) );
+            number.appendChild( newTextElement( KVTML_2NDPERSON, second ) );
+            number.appendChild( newTextElement( KVTML_THIRD_MALE, third_male ) );
+            number.appendChild( newTextElement( KVTML_THIRD_FEMALE, third_female ) );
+            number.appendChild( newTextElement( KVTML_THIRD_NEUTRAL_COMMON, third_neutral ) );
+
+            if ( pronoun.maleFemaleDifferent() ) {
+                number.appendChild( m_domDoc.createElement( KVTML_THIRD_PERSON_MALE_FEMALE_DIFFERENT ) );
+            }
+            if ( pronoun.neutralExists() ) {
+                number.appendChild( m_domDoc.createElement( KVTML_THIRD_PERSON_NEUTRAL_EXISTS ) );
+            }
+            pronounElement.appendChild( number );
+        }
+    }
+    return true;
+}
