@@ -2,7 +2,7 @@
                      read a KEduVocDocument from a KVTML file
     -----------------------------------------------------------------------
     copyright           : (C) 1999-2001 Ewald Arnold <kvoctrain@ewald-arnold.de>
-                          (C) 2001 The KDE-EDU team
+
                           (C) 2005 Eric Pignet <eric at erixpage.com>
                           (C) 2007 Peter Hedlund <peter.hedlund@kdemail.net>
                           (C) 2007 Frederik Gladhorn <frederik.gladhorn@kdemail.net>
@@ -25,6 +25,7 @@
 
 #include <kdebug.h>
 #include <klocale.h>
+#include <kglobal.h>
 
 #include "keduvocdocument.h"
 #include "keduvoclesson.h"
@@ -37,6 +38,7 @@ KEduVocKvtmlReader::KEduVocKvtmlReader( QIODevice *file )
     // the file must be already open
     m_inputFile = file;
     m_errorMessage = "";
+    kDebug() << "KEduVocKvtmlReader for kvtml version 1 files started.";
 }
 
 
@@ -132,21 +134,31 @@ bool KEduVocKvtmlReader::readBody( QDomElement &domElementParent )
 
     currentElement = domElementParent.firstChildElement( KV_CONJUG_GRP );
     if ( !currentElement.isNull() ) {
-        QList<KEduVocConjugation> conjugations;
-        result = readConjug( currentElement, conjugations );
-        if ( result ) {
-            KEduVocConjugation conjug;
-            for ( int i = 0; i< conjugations.count(); i++ ) {
-                conjug = conjugations[i];
-                m_doc->setConjugation( i, conjug );
+        int count = 0;
+
+        QDomElement domElementConjugChild = currentElement.firstChildElement(KV_CON_ENTRY);
+        while ( !domElementConjugChild.isNull() ) {
+            QString lang;
+            QDomAttr domAttrLang = domElementConjugChild.attributeNode( KV_LANG ); // "l"
+            // make sure, the identifier is there
+            if (!addLanguage(count, domAttrLang.value())) {
+                return false;
             }
-        } else
-            return false;
+
+            KEduVocPersonalPronoun pronouns;
+            if (! readPersonalPronouns( domElementConjugChild, pronouns ) ) {
+                return false;
+            }
+            m_doc->identifier(count).setPersonalPronouns( pronouns );
+
+            count ++;
+
+            domElementConjugChild = domElementConjugChild.nextSiblingElement( KV_CON_ENTRY );
+        }
     }
 
-
     // initialize the list of predefined types
-    m_doc->wordTypes()->createDefaultWordTypes();
+    m_doc->wordTypes().createDefaultWordTypes();
     currentElement = domElementParent.firstChildElement( KV_TYPE_GRP );
     if ( !currentElement.isNull() ) {
         result = readType( currentElement );
@@ -181,6 +193,8 @@ bool KEduVocKvtmlReader::readBody( QDomElement &domElementParent )
         }
     }
 
+    m_doc->setTenseDescriptions(m_compability.documentTenses());
+
     return true;
 }
 
@@ -207,40 +221,37 @@ bool KEduVocKvtmlReader::readLesson( QDomElement &domElementParent )
     if ( entryList.length() <= 0 )
         return false;
 
-    QList<int> inQueryList;
-
     for ( int i = 0; i < entryList.count(); ++i ) {
         currentElement = entryList.item( i ).toElement();
         if ( currentElement.parentNode() == domElementParent ) {
-            int no = 0;
-            bool isCurr = false;
+            int no;
 
             attribute = currentElement.attributeNode( KV_LESS_NO );
-            if ( !attribute.isNull() )
+            if ( !attribute.isNull() ) {
                 no = attribute.value().toInt();
+            }
 
             attribute = currentElement.attributeNode( KV_LESS_CURR );
-            if ( !attribute.isNull() )
-                isCurr = attribute.value().toInt() != 0;
-
-            if ( isCurr && no != 0 )
-                m_doc->setCurrentLesson( no );
+            if ( !attribute.isNull() ) {
+                if ( attribute.value().toInt() != 0 ) {
+                    m_doc->setCurrentLesson( no );
+                }
+            }
 
+            bool inQuery = false;
             attribute = currentElement.attributeNode( KV_LESS_QUERY );
-            if ( !attribute.isNull() )
-                if ( attribute.value().toInt() != 0 && no > 0 )
-                    inQueryList.append( no );
+            if ( !attribute.isNull() ) {
+                inQuery = (attribute.value().toInt() != 0);
+            }
 
             s = currentElement.text();
-            if ( s.isNull() )
-                s = "";
-            m_doc->addLesson( s, no );
+            int index = m_doc->appendLesson( s, inQuery );
+            if ( index != no-1 ) {
+                kDebug() << "Warning! Lesson order may be confused. Are all lessons in order in the file?";
+            }
         }
     }
 
-    if ( inQueryList.count() > 0 )
-        m_doc->setLessonsInQuery( inQueryList );
-
     return true;
 }
 
@@ -277,20 +288,8 @@ bool KEduVocKvtmlReader::readArticle( QDomElement &domElementParent )
             QString lang;
             attribute = currentElement.attributeNode( KV_LANG );
 
-            if ( m_doc->identifierCount() <= i ) {
-                // first entry
-                if ( !attribute.isNull() )       // no definition in first entry
-                    lang = attribute.value();
-                else
-                    lang = "original";
-                m_doc->appendIdentifier( lang );
-//kDebug() << " Identifier " << i << " is " << lang;
-            } else {
-                if ( !attribute.isNull() && attribute.value() != m_doc->identifier( i ) ) {
-                    // different originals ?
-                    m_errorMessage = i18n( "Ambiguous definition of language code" );
-                    return false;
-                }
+            if (!addLanguage(i, attribute.value())) {
+                return false;
             }
 
             //---------
@@ -345,7 +344,7 @@ bool KEduVocKvtmlReader::readArticle( QDomElement &domElementParent )
                     nat_indef = "";
             }
 
-            m_doc->setArticle( i, KEduVocArticle( fem_def, fem_indef, mal_def, mal_indef, nat_def, nat_indef ) );
+            m_doc->identifier(i).setArticle( KEduVocArticle( fem_def, fem_indef, mal_def, mal_indef, nat_def, nat_indef ) );
         }
     }
 
@@ -353,7 +352,28 @@ bool KEduVocKvtmlReader::readArticle( QDomElement &domElementParent )
 }
 
 
-bool KEduVocKvtmlReader::readConjug( QDomElement &domElementParent, QList<KEduVocConjugation> &curr_conjug )
+bool KEduVocKvtmlReader::readTranslationConjugations( QDomElement &domElementParent, KEduVocTranslation &translation ) {
+
+    QString tense;
+
+    QDomElement domElementConjugChild = domElementParent.firstChildElement(KV_CON_TYPE);
+    while ( !domElementConjugChild.isNull() )
+    {
+        // "n" == is the type is the tense
+        QDomAttr domAttrLang = domElementConjugChild.attributeNode( KV_CON_NAME );
+        QString oldShortTense = domAttrLang.value();
+
+        tense = m_compability.tenseFromKvtml1( oldShortTense );
+        KEduVocConjugation conjugation;
+        readConjugation(domElementConjugChild, conjugation);
+        translation.setConjugation(tense, conjugation);
+
+        domElementConjugChild = domElementConjugChild.nextSiblingElement( KV_CON_TYPE );
+    } // while -> next tense, count++
+    return true;
+}
+
+bool KEduVocKvtmlReader::readConjugation( QDomElement &domElementParent, KEduVocConjugation& conjugation )
 /*
  <conjugation>        used in header for definiton of "prefix"
   <e l="de">          lang determines also lang order in entries !!
@@ -386,7 +406,7 @@ bool KEduVocKvtmlReader::readConjug( QDomElement &domElementParent, QList<KEduVo
  </conjugation>
 */
 {
-    QString s;
+//     QString s;
     bool p3_common;
     bool s3_common;
     QString pers1_sing;
@@ -399,173 +419,204 @@ bool KEduVocKvtmlReader::readConjug( QDomElement &domElementParent, QList<KEduVo
     QString pers3_m_plur;
     QString pers3_f_plur;
     QString pers3_n_plur;
-    QString lang;
-    QString type;
 
-    // this gets a list of keduvocconjugations, count will be the conjug number to be filled.
-    int count = 0;
+    p3_common = false;
+    s3_common = false;
 
-// reset
-    curr_conjug.clear();
-// create new empty one
-    curr_conjug.append( KEduVocConjugation() );
+    // get the individual entries for persons...
+    QDomElement domElementConjugGrandChild = domElementParent.firstChild().toElement();
+    while ( !domElementConjugGrandChild.isNull() ) {
+        if ( domElementConjugGrandChild.tagName() == KV_CON_P1S ) {
+            pers1_sing = domElementConjugGrandChild.text();
+        } else if ( domElementConjugGrandChild.tagName() == KV_CON_P2S ) {
+            pers2_sing = domElementConjugGrandChild.text();
+        } else if ( domElementConjugGrandChild.tagName() == KV_CON_P3SF ) {
+            QDomAttr domAttrCommon = domElementConjugGrandChild.attributeNode( KV_CONJ_COMMON );
+            if ( !domAttrCommon.isNull() )
+                s3_common = domAttrCommon.value().toInt();  // returns 0 if the conversion fails
+            pers3_f_sing = domElementConjugGrandChild.text();
 
-    QDomElement domElementConjugChild = domElementParent.firstChild().toElement();
-    while ( !domElementConjugChild.isNull() )
-    {
-        if ( domElementConjugChild.tagName() == KV_CON_ENTRY ) {            // if KV_CON_ENTRY == "e" is found, we are reading a personal pronun definition.
-            // CONJ_PREFIX is defined as "--"
-            type = CONJ_PREFIX;
+        } else if ( domElementConjugGrandChild.tagName() == KV_CON_P3SM ) {
+            pers3_m_sing = domElementConjugGrandChild.text();
 
-            //----------
-            // Attribute
+        } else if ( domElementConjugGrandChild.tagName() == KV_CON_P3SN ) {
+            pers3_n_sing = domElementConjugGrandChild.text();
 
-            QString lang;
-            QDomAttr domAttrLang = domElementConjugChild.attributeNode( KV_LANG ); // "l"
+        } else if ( domElementConjugGrandChild.tagName() == KV_CON_P1P ) {
+            pers1_plur = domElementConjugGrandChild.text();
 
-            if ( m_doc->identifierCount() <= count ) {
-                // first entry
-                if ( !domAttrLang.isNull() )          // no definition in first entry
-                    lang = domAttrLang.value();
-                else
-                    lang = "original";
-                m_doc->appendIdentifier( lang );
-            } else {
-                if ( !domAttrLang.isNull() && domAttrLang.value() != m_doc->identifier( count ) ) {
-                    // different originals ?
-                    m_errorMessage = i18n( "Ambiguous definition of language code" );
-                    return false;
-                }
-            }
-        } else if ( domElementConjugChild.tagName() == KV_CON_TYPE ) {                    // this means reading translations KV_CON_TYPE == "t"
-            //----------
-            // Attribute
+        } else if ( domElementConjugGrandChild.tagName() == KV_CON_P2P ) {
+            pers2_plur = domElementConjugGrandChild.text();
 
-            // "n" == is the type is the tense
-            QDomAttr domAttrLang = domElementConjugChild.attributeNode( KV_CON_NAME );
-            type = domAttrLang.value();
-            if ( type.isNull() )
-                type = "";
+        } else if ( domElementConjugGrandChild.tagName() == KV_CON_P3PF ) {
+            QDomAttr domAttrCommon = domElementConjugGrandChild.attributeNode( KV_CONJ_COMMON );
+            if ( !domAttrCommon.isNull() )
+                p3_common = domAttrCommon.value().toInt();  // returns 0 if the conversion fails
 
-            // if it starts with "#" the user typed in the tense name
-            if ( type.length() != 0 && type.left( 1 ) == UL_USER_TENSE ) {
-                int num = qMin( type.mid( 1, 40 ).toInt(), 1000 ); // paranoia check
-                if ( num > m_doc->tenseDescriptions().count() ) {
-                    // description missing ?
-                    QString s;
-                    QStringList sl = m_doc->tenseDescriptions();
-                    for ( int i = m_doc->tenseDescriptions().count(); i < num; i++ ) {
-                        s.setNum( i + 1 );
-                        s.prepend( "#" ); // invent descr according to number
-                        sl.append( s );
-                    }
-                    m_doc->setTenseDescriptions( sl );
-                }
-            }
+            pers3_f_plur = domElementConjugGrandChild.text();
+
+        } else if ( domElementConjugGrandChild.tagName() == KV_CON_P3PM ) {
+            pers3_m_plur = domElementConjugGrandChild.text();
+
+        } else if ( domElementConjugGrandChild.tagName() == KV_CON_P3PN ) {
+            pers3_n_plur = domElementConjugGrandChild.text();
+
+        } else {
+            return false;
         }
 
-        pers1_sing = "";
-        pers2_sing = "";
-        pers3_m_sing = "";
-        pers3_f_sing = "";
-        pers3_n_sing = "";
-        pers1_plur = "";
-        pers2_plur = "";
-        pers3_m_plur = "";
-        pers3_f_plur = "";
-        pers3_n_plur = "";
-        p3_common = false;
-        s3_common = false;
+        domElementConjugGrandChild = domElementConjugGrandChild.nextSibling().toElement();
+    } // while - probably to be sure, because the persons could be in any order.
+    // I guess this goes over only one set, such as:
+    // <s1>traigo</s1><s2>traes</s2><s3fcommon="1">trae</s3f>
+    // <p1>traemos</p1><p2>traÃÂ©is</p2><p3f common="1">traen</p3f>
+    // until no elements are left in that soup.
 
-        // get the individual entries for persons...
-        QDomElement domElementConjugGrandChild = domElementConjugChild.firstChild().toElement();
-        while ( !domElementConjugGrandChild.isNull() ) {
-            if ( domElementConjugGrandChild.tagName() == KV_CON_P1S ) {
-                pers1_sing = domElementConjugGrandChild.text();
-                if ( pers1_sing.isNull() )
-                    pers1_sing = "";
-            } else if ( domElementConjugGrandChild.tagName() == KV_CON_P2S ) {
-                pers2_sing = domElementConjugGrandChild.text();
-                if ( pers2_sing.isNull() )
-                    pers2_sing = "";
-            } else if ( domElementConjugGrandChild.tagName() == KV_CON_P3SF ) {
-                QDomAttr domAttrCommon = domElementConjugGrandChild.attributeNode( KV_CONJ_COMMON );
-                if ( !domAttrCommon.isNull() )
-                    s3_common = domAttrCommon.value().toInt();  // returns 0 if the conversion fails
+    // now set the data: [count] - number of conjug?
+    // type - the tense?
+    // finally the person
 
-                pers3_f_sing = domElementConjugGrandChild.text();
-                if ( pers3_f_sing.isNull() )
-                    pers3_f_sing = "";
-            } else if ( domElementConjugGrandChild.tagName() == KV_CON_P3SM ) {
-                pers3_m_sing = domElementConjugGrandChild.text();
-                if ( pers3_m_sing.isNull() )
-                    pers3_m_sing = "";
-            } else if ( domElementConjugGrandChild.tagName() == KV_CON_P3SN ) {
-                pers3_n_sing = domElementConjugGrandChild.text();
-                if ( pers3_n_sing.isNull() )
-                    pers3_n_sing = "";
-            } else if ( domElementConjugGrandChild.tagName() == KV_CON_P1P ) {
-                pers1_plur = domElementConjugGrandChild.text();
-                if ( pers1_plur.isNull() )
-                    pers1_plur = "";
-            } else if ( domElementConjugGrandChild.tagName() == KV_CON_P2P ) {
-                pers2_plur = domElementConjugGrandChild.text();
-                if ( pers2_plur.isNull() )
-                    pers2_plur = "";
-            } else if ( domElementConjugGrandChild.tagName() == KV_CON_P3PF ) {
-                QDomAttr domAttrCommon = domElementConjugGrandChild.attributeNode( KV_CONJ_COMMON );
-                if ( !domAttrCommon.isNull() )
-                    p3_common = domAttrCommon.value().toInt();  // returns 0 if the conversion fails
+    const KEduVocConjugation::ConjugationNumber numS = KEduVocConjugation::Singular;
+    const KEduVocConjugation::ConjugationNumber numP = KEduVocConjugation::Plural;
 
-                pers3_f_plur = domElementConjugGrandChild.text();
-                if ( pers3_f_plur.isNull() )
-                    pers3_f_plur = "";
-            } else if ( domElementConjugGrandChild.tagName() == KV_CON_P3PM ) {
-                pers3_m_plur = domElementConjugGrandChild.text();
-                if ( pers3_m_plur.isNull() )
-                    pers3_m_plur = "";
-            } else if ( domElementConjugGrandChild.tagName() == KV_CON_P3PN ) {
-                pers3_n_plur = domElementConjugGrandChild.text();
-                if ( pers3_n_plur.isNull() )
-                    pers3_n_plur = "";
-            } else {
-                return false;
-            }
+    conjugation.setConjugation( pers1_sing, KEduVocConjugation::First, numS);
+    conjugation.setConjugation( pers2_sing, KEduVocConjugation::Second, numS);
+    conjugation.setConjugation( pers1_plur, KEduVocConjugation::First, numP);
+    conjugation.setConjugation( pers2_plur, KEduVocConjugation::Second, numP);
 
-            domElementConjugGrandChild = domElementConjugGrandChild.nextSibling().toElement();
-        } // while - probably to be sure, because the persons could be in any order.
-        // I guess this goes over only one set, such as:
-        // <s1>traigo</s1><s2>traes</s2><s3fcommon="1">trae</s3f>
-        // <p1>traemos</p1><p2>traÃÂ©is</p2><p3f common="1">traen</p3f>
-        // until no elements are left in that soup.
+    if ( s3_common ) {
+        conjugation.setConjugation( pers3_f_sing, KEduVocConjugation::ThirdNeutralCommon, KEduVocConjugation::Singular );
+    } else  {
+        conjugation.setConjugation( pers3_m_sing,
+            KEduVocConjugation::ThirdMale, KEduVocConjugation::Singular );
+        conjugation.setConjugation( pers3_f_sing,
+            KEduVocConjugation::ThirdFemale, KEduVocConjugation::Singular );
+        conjugation.setConjugation( pers3_n_sing,
+            KEduVocConjugation::ThirdNeutralCommon, KEduVocConjugation::Singular );
+    }
+
+    if ( p3_common ) {
+        conjugation.setConjugation( pers3_f_plur, KEduVocConjugation::ThirdNeutralCommon, KEduVocConjugation::Plural );
+    } else  {
+        conjugation.setConjugation( pers3_m_plur,
+            KEduVocConjugation::ThirdMale, KEduVocConjugation::Plural );
+        conjugation.setConjugation( pers3_f_plur,
+            KEduVocConjugation::ThirdFemale, KEduVocConjugation::Plural );
+        conjugation.setConjugation( pers3_n_plur,
+            KEduVocConjugation::ThirdNeutralCommon, KEduVocConjugation::Plural );
+    }
+
+    return true;
+}
 
 
 
-        if ( domElementConjugChild.tagName() == KV_CON_ENTRY )
-            while ( count + 1 > ( int ) curr_conjug.size() )
-                curr_conjug.append( KEduVocConjugation() );
 
-        // now set the data: [count] - number of conjug?
-        // type - the tense?
-        // finally the person
-        curr_conjug[count].setPers3SingularCommon( type, s3_common );
-        curr_conjug[count].setPers3PluralCommon( type, p3_common );
-        curr_conjug[count].setPers1Singular( type, pers1_sing );
-        curr_conjug[count].setPers2Singular( type, pers2_sing );
-        curr_conjug[count].setPers3FemaleSingular( type, pers3_f_sing );
-        curr_conjug[count].setPers3MaleSingular( type, pers3_m_sing );
-        curr_conjug[count].setPers3NaturalSingular( type, pers3_n_sing );
-        curr_conjug[count].setPers1Plural( type, pers1_plur );
-        curr_conjug[count].setPers2Plural( type, pers2_plur );
-        curr_conjug[count].setPers3FemalePlural( type, pers3_f_plur );
-        curr_conjug[count].setPers3MalePlural( type, pers3_m_plur );
-        curr_conjug[count].setPers3NaturalPlural( type, pers3_n_plur );
+bool KEduVocKvtmlReader::readPersonalPronouns( QDomElement &domElementParent, KEduVocPersonalPronoun& pronouns )
+{
+//     QString s;
+    bool p3_common;
+    bool s3_common;
+    QString pers1_sing;
+    QString pers2_sing;
+    QString pers3_m_sing;
+    QString pers3_f_sing;
+    QString pers3_n_sing;
+    QString pers1_plur;
+    QString pers2_plur;
+    QString pers3_m_plur;
+    QString pers3_f_plur;
+    QString pers3_n_plur;
 
-        if ( domElementConjugChild.tagName() == KV_CON_ENTRY )
-            count++;
+    p3_common = false;
+    s3_common = false;
 
-        domElementConjugChild = domElementConjugChild.nextSibling().toElement();
-    } // while -> next type, count++
+    // get the individual entries for persons...
+    QDomElement domElementConjugGrandChild = domElementParent.firstChild().toElement();
+    while ( !domElementConjugGrandChild.isNull() ) {
+        if ( domElementConjugGrandChild.tagName() == KV_CON_P1S ) {
+            pers1_sing = domElementConjugGrandChild.text();
+        } else if ( domElementConjugGrandChild.tagName() == KV_CON_P2S ) {
+            pers2_sing = domElementConjugGrandChild.text();
+        } else if ( domElementConjugGrandChild.tagName() == KV_CON_P3SF ) {
+            QDomAttr domAttrCommon = domElementConjugGrandChild.attributeNode( KV_CONJ_COMMON );
+            if ( !domAttrCommon.isNull() )
+                s3_common = domAttrCommon.value().toInt();  // returns 0 if the conversion fails
+            pers3_f_sing = domElementConjugGrandChild.text();
+
+        } else if ( domElementConjugGrandChild.tagName() == KV_CON_P3SM ) {
+            pers3_m_sing = domElementConjugGrandChild.text();
+
+        } else if ( domElementConjugGrandChild.tagName() == KV_CON_P3SN ) {
+            pers3_n_sing = domElementConjugGrandChild.text();
+
+        } else if ( domElementConjugGrandChild.tagName() == KV_CON_P1P ) {
+            pers1_plur = domElementConjugGrandChild.text();
+
+        } else if ( domElementConjugGrandChild.tagName() == KV_CON_P2P ) {
+            pers2_plur = domElementConjugGrandChild.text();
+
+        } else if ( domElementConjugGrandChild.tagName() == KV_CON_P3PF ) {
+            QDomAttr domAttrCommon = domElementConjugGrandChild.attributeNode( KV_CONJ_COMMON );
+            if ( !domAttrCommon.isNull() )
+                p3_common = domAttrCommon.value().toInt();  // returns 0 if the conversion fails
+
+            pers3_f_plur = domElementConjugGrandChild.text();
+
+        } else if ( domElementConjugGrandChild.tagName() == KV_CON_P3PM ) {
+            pers3_m_plur = domElementConjugGrandChild.text();
+
+        } else if ( domElementConjugGrandChild.tagName() == KV_CON_P3PN ) {
+            pers3_n_plur = domElementConjugGrandChild.text();
+
+        } else {
+            return false;
+        }
+
+        domElementConjugGrandChild = domElementConjugGrandChild.nextSibling().toElement();
+    } // while - probably to be sure, because the persons could be in any order.
+    // I guess this goes over only one set, such as:
+    // <s1>traigo</s1><s2>traes</s2><s3fcommon="1">trae</s3f>
+    // <p1>traemos</p1><p2>traÃÂ©is</p2><p3f common="1">traen</p3f>
+    // until no elements are left in that soup.
+
+    // now set the data: [count] - number of conjug?
+    // type - the tense?
+    // finally the person
+
+    const KEduVocConjugation::ConjugationNumber numS = KEduVocConjugation::Singular;
+    pronouns.setMaleFemaleDifferent(false);
+    pronouns.setPersonalPronoun( pers1_sing, KEduVocConjugation::First, numS );
+    pronouns.setPersonalPronoun( pers2_sing, KEduVocConjugation::Second, numS );
+
+    // used to have common in female
+    if ( s3_common ) {
+        pronouns.setPersonalPronoun( pers3_f_sing, KEduVocConjugation::ThirdNeutralCommon, numS );
+    } else  {
+        pronouns.setPersonalPronoun( pers3_m_sing,
+            KEduVocConjugation::ThirdMale, numS );
+        pronouns.setPersonalPronoun( pers3_f_sing,
+            KEduVocConjugation::ThirdFemale, numS );
+        pronouns.setPersonalPronoun( pers3_n_sing,
+            KEduVocConjugation::ThirdNeutralCommon, numS );
+        pronouns.setMaleFemaleDifferent(true);
+    }
+
+    const KEduVocConjugation::ConjugationNumber numP = KEduVocConjugation::Plural;
+
+    pronouns.setPersonalPronoun( pers1_plur, KEduVocConjugation::First, numP );
+    pronouns.setPersonalPronoun( pers2_plur, KEduVocConjugation::Second, numP );
+    if ( p3_common ) {
+        pronouns.setPersonalPronoun( pers3_f_plur, KEduVocConjugation::ThirdNeutralCommon, numP );
+    } else  {
+        pronouns.setPersonalPronoun( pers3_m_plur,
+            KEduVocConjugation::ThirdMale, numP );
+        pronouns.setPersonalPronoun( pers3_f_plur,
+            KEduVocConjugation::ThirdFemale, numP );
+        pronouns.setPersonalPronoun( pers3_n_plur,
+            KEduVocConjugation::ThirdNeutralCommon, numP );
+        pronouns.setMaleFemaleDifferent(true);
+    }
 
     return true;
 }
@@ -588,7 +639,7 @@ bool KEduVocKvtmlReader::readType( QDomElement &domElementParent )
 
             kDebug() << "Adding old self defined type: " << currentElement.text();
             // add the type to the list of available types
-            m_doc->wordTypes()->addType( currentElement.text() );
+            m_doc->wordTypes().addType( currentElement.text() );
 
             // from this the #1 are transformed to something sensible again
             m_oldSelfDefinedTypes.append( currentElement.text() );
@@ -601,33 +652,19 @@ bool KEduVocKvtmlReader::readType( QDomElement &domElementParent )
 
 bool KEduVocKvtmlReader::readTense( QDomElement &domElementParent )
 {
-    QString s;
     QDomElement currentElement;
-    QStringList descriptions;
 
     QDomNodeList entryList = domElementParent.elementsByTagName( KV_TENSE_DESC );
     if ( entryList.length() <= 0 )
         return false;
 
-    descriptions.clear();
-
     for ( int i = 0; i < entryList.count(); ++i ) {
         currentElement = entryList.item( i ).toElement();
         if ( currentElement.parentNode() == domElementParent ) {
-            int no = 0;
-
-            QDomAttr attribute = currentElement.attributeNode( KV_TENSE_NO );
-            if ( !attribute.isNull() )
-                no = attribute.value().toInt();
-
-            s = currentElement.text();
-            if ( s.isNull() )
-                s = "";
-            descriptions.append( s );
+            m_compability.addUserdefinedTense( currentElement.text() );
         }
     }
 
-    m_doc->setTenseDescriptions( descriptions );
     return true;
 }
 
@@ -954,7 +991,7 @@ bool KEduVocKvtmlReader::readExpression( QDomElement &domElementParent )
     QString                   q_org;
     QString                   q_trans;
     QString                   query_id;
-    int                       lesson = 0;
+    int                       lesson = - 1;
     int                       width;
     QString                   type;
     QString                   subType;
@@ -965,7 +1002,7 @@ bool KEduVocKvtmlReader::readExpression( QDomElement &domElementParent )
     QString                   antonym;
     QSet<QString>             usage;
     QString                   paraphrase;
-    QList<KEduVocConjugation> conjug;
+
     KEduVocComparison         comparison;
     KEduVocMultipleChoice     mc;
     KEduVocExpression         expr;
@@ -980,13 +1017,15 @@ bool KEduVocKvtmlReader::readExpression( QDomElement &domElementParent )
 
     attribute = domElementParent.attributeNode( KV_LESS_MEMBER );
     if ( !attribute.isNull() ) {
-        lesson = attribute.value().toInt();
-    }
-
-    if ( lesson && lesson > m_doc->lessonCount() ) {
-        // it's from a lesson that hasn't been added yet
-        // so make sure this lesson is in the document
-        m_doc->addLesson( QString( "#" ) + QString::number( lesson ), lesson );
+        // we start conting from 0 in new documents
+        lesson = attribute.value().toInt() - 1;
+        if ( lesson > m_doc->lessonCount() ) {
+            ///@todo can this happen? does it need a while loop?
+            // it's from a lesson that hasn't been added yet
+            // so make sure this lesson is in the document
+            kDebug() << "Warning: lesson > m_doc->lessonCount() in readExpression.";
+            m_doc->appendLesson( i18nc("A generic name for a new lesson and its number.", "Lesson %1", lesson ));
+        }
     }
 
     attribute = domElementParent.attributeNode( KV_SELECTED );
@@ -1066,40 +1105,17 @@ bool KEduVocKvtmlReader::readExpression( QDomElement &domElementParent )
 
                 q_trans = lang;
         }
-//kDebug() << " TranslationList.count(): " << translationList.count() << "  Entry count: " << m_doc->entryCount();
-        if ( m_doc->entryCount() == 0 ) { // this is because in kvtml the languages are saved in the FIRST ENTRY ONLY.
-//kDebug() << " Read Expression with identifiers: " << lang;
-            // new translation
-            if ( lang.isEmpty() ) {
-                if ( i == 0 ) {
-                    lang = "original";
-                } else {
-                    // no definition in first entry ?
-                    lang.setNum( m_doc->identifierCount() );
-                    lang.prepend( "translation " );
-                }
 
-            }
-            if ( m_doc->identifierCount() <= i )
-                m_doc->appendIdentifier( lang );
-        } else {
-            if ( lang != m_doc->identifier( i ) && !lang.isEmpty() ) {
-                // different language ?
-                m_errorMessage = i18n( "ambiguous definition of language code" );
+        if ( m_doc->entryCount() == 0 ) { // this is because in kvtml the languages are saved in the FIRST ENTRY ONLY.
+
+            // new translation
+            if (!addLanguage(i, lang)) {
                 return false;
             }
         }
-
         //---------
         // Children
 
-        currentChild = currentElement.firstChildElement( KV_CONJUG_GRP );
-        if ( !currentChild.isNull() ) {
-            conjug.clear();
-            if ( !readConjug( currentChild, conjug ) )
-                return false;
-        }
-
         currentChild = currentElement.firstChildElement( KV_COMPARISON_GRP );
         if ( !currentChild.isNull() ) {
             comparison.clear();
@@ -1115,26 +1131,22 @@ bool KEduVocKvtmlReader::readExpression( QDomElement &domElementParent )
         }
 
         textstr = currentElement.lastChild().toText().data();
-        if ( textstr.isNull() )
-            textstr = "";
 
         if ( i == 0 ) {
-            expr = KEduVocExpression( textstr );
-            expr.setLesson( lesson );
-            expr.setInQuery( inquery );
+            expr = KEduVocExpression( textstr, lesson );
             expr.setActive( active );
         } else {
             expr.setTranslation( i, textstr );
         }
 
-        if ( conjug.size() > 0 ) {
-            for ( int conjugationIndex = 0; conjugationIndex < conjug.size(); conjugationIndex++ ) {
-                expr.translation( i ).setConjugation( conjug[conjugationIndex] );
+        // better make sure, translation(i) already exists...
+        currentChild = currentElement.firstChildElement( KV_CONJUG_GRP );
+        if ( !currentChild.isNull() ) {
+            if ( !readTranslationConjugations( currentChild, expr.translation(i) ) ) {
+                return false;
             }
-            //expr.setConjugation(i, conjug[0]); ///@todo check if this is better than the above!
-
-            conjug.clear();
         }
+
         if ( !comparison.isEmpty() ) {
             expr.translation( i ).setComparison( comparison );
             comparison.clear();
@@ -1186,16 +1198,43 @@ bool KEduVocKvtmlReader::readExpression( QDomElement &domElementParent )
         i++;
     }
 
-    if ( m_doc->entryCount() == 0 )
-        m_doc->setQueryIdentifier( q_org, q_trans );
+//     if ( m_doc->entryCount() == 0 ) {
+//         m_doc->setQueryIdentifier( q_org, q_trans );
+//     }
     m_doc->appendEntry( &expr );
 
-    // also add this entryid to the lesson it's part of
-    if ( m_doc->lesson( lesson ) != NULL ) {
-        m_doc->lesson( lesson )->addEntry( m_doc->entryCount() );
-    }
-
     return true;
 }
 
+
+bool KEduVocKvtmlReader::addLanguage( int languageId, const QString& locale)
+{
+    if ( m_doc->identifierCount() <= languageId ) {
+        m_doc->appendIdentifier();
+        // first entry
+        if ( !locale.isEmpty() ) {      // no definition in first entry
+            m_doc->identifier(languageId).setLocale(locale);
+
+            QString languageName = KGlobal::locale()->languageCodeToName(locale);
+            if ( languageName.isEmpty() ) {
+                languageName = locale;
+            }
+
+            m_doc->identifier(languageId).setName(languageName);
+            kDebug() << "addLanguage( " << languageId << ", " << locale << "): " << languageName;
+
+        }
+    } else {
+        if ( !locale.isEmpty() ) {
+            if ( locale != m_doc->identifier(languageId).locale() ) {
+                // different originals ?
+                m_errorMessage = i18n( "Ambiguous definition of language code" );
+                return false;
+            }
+        }
+    }
+    return true;
+}
+
+
 #include "keduvockvtmlreader.moc"
