@@ -25,6 +25,7 @@
 #include <QtCore/QtAlgorithms>
 #include <QtCore/QDebug>
 #include <QtCore/QTimer>
+#include <QtGui/QSortFilterProxyModel>
 #include <QtGui/QStringListModel>
 #include <QtGui/QTextFrame>
 
@@ -36,8 +37,7 @@
 #include <MarbleDirs.h>
 #include <MapTheme.h>
 #include <FileViewModel.h>
-
-// #define FLATPROJ_GSOC
+#include "gps/GpxFileModel.h"
 
 class MarbleControlBoxPrivate
 {
@@ -56,6 +56,7 @@ class MarbleControlBoxPrivate
     QWidget              *m_fileViewWidget;
 
     QStandardItemModel   *m_mapthememodel;
+    QSortFilterProxyModel *m_sortproxy;
 };
 
 
@@ -98,6 +99,9 @@ MarbleControlBox::MarbleControlBox(QWidget *parent)
     d->uiWidget.toolBox->setCurrentIndex( 4 );
     d->m_fileViewWidget = d->uiWidget.toolBox->currentWidget();
 
+    d->m_sortproxy = new QSortFilterProxyModel( d->uiWidget.locationListView );
+    d->uiWidget.locationListView->setModel( d->m_sortproxy );
+
 //  d->m_currentLocationWidget->hide(); // Current location tab is hidden
                                     //by default
  //   toolBox->removeItem( 3 );
@@ -118,6 +122,9 @@ MarbleControlBox::MarbleControlBox(QWidget *parent)
     connect( d->uiWidget.zoomOutButton, SIGNAL( clicked() ),
              this,                      SIGNAL( zoomOut() ) );
 
+    connect( d->uiWidget.zoomSlider,  SIGNAL( valueChanged( int ) ),
+             this,                      SLOT( updateButtons( int ) ) );
+
     connect( d->uiWidget.moveLeftButton,  SIGNAL( clicked() ),
              this,                        SIGNAL( moveLeft() ) );
     connect( d->uiWidget.moveRightButton, SIGNAL( clicked() ),
@@ -131,7 +138,7 @@ MarbleControlBox::MarbleControlBox(QWidget *parent)
             this,                         SLOT( searchLineChanged( const QString& ) ) );
 
     connect( d->uiWidget.locationListView, SIGNAL( centerOn( const QModelIndex& ) ),
-             this,                         SIGNAL( centerOn( const QModelIndex& ) ) );
+             this,                         SLOT( mapCenterOnSignal( const QModelIndex& ) ) );
 
     QStringList          mapthemedirs  = MapTheme::findMapThemes( "maps/earth" );
     d->m_mapthememodel = MapTheme::mapThemeModel( mapthemedirs );
@@ -142,11 +149,30 @@ MarbleControlBox::MarbleControlBox(QWidget *parent)
     connect( d->uiWidget.projectionComboBox,    SIGNAL( currentIndexChanged( int ) ),
              this,                              SIGNAL( projectionSelected( int ) ) );
 
-#ifdef FLATPROJ_GSOC
     d->uiWidget.projectionComboBox->setEnabled( true );
-#endif
 }
 
+MarbleControlBox::~MarbleControlBox()
+{
+    delete d->m_mapthememodel;
+    delete d;
+}
+
+void MarbleControlBox::updateButtons( int value )
+{
+    if ( value <= d->uiWidget.zoomSlider->minimum() ) {
+        d->uiWidget.zoomInButton->setEnabled( true );
+        d->uiWidget.zoomOutButton->setEnabled( false );
+    } else if ( value >= d->uiWidget.zoomSlider->maximum() ) {
+        d->uiWidget.zoomInButton->setEnabled( false );
+        d->uiWidget.zoomOutButton->setEnabled( true );
+    } else {
+        d->uiWidget.zoomInButton->setEnabled( true );
+        d->uiWidget.zoomOutButton->setEnabled( true );
+    }
+}
+
+
 void MarbleControlBox::setupGpsOption()
 {
     d->uiWidget.m_gpsDrawBox -> setEnabled( true );
@@ -167,19 +193,20 @@ void MarbleControlBox::addMarbleWidget(MarbleWidget *widget)
     // Make us aware of all the PlaceMarks in the MarbleModel so that
     // we can search them.
     setLocations( d->m_widget->placeMarkModel() );
+    d->uiWidget.locationListView->setSelectionModel( d->m_widget->placeMarkSelectionModel() );
 
 #ifndef KML_GSOC
     //set up everything for the FileModel
     d->uiWidget.m_fileView->setModel( widget->gpxFileModel() );
 
     connect( d->uiWidget.m_fileView->selectionModel(),
-          SIGNAL( selectionChanged( QItemSelection, QItemSelection )),
-          this,
-          SLOT( enableFileViewActions() ) );
-    connect( d->uiWidget.m_saveButton, SIGNAL( clicked() ) ,
-             widget->gpxFileModel(), SLOT( saveFile() ) );
+	     SIGNAL( selectionChanged( QItemSelection, QItemSelection )),
+	     this,
+	     SLOT( enableFileViewActions() ) );
+    connect( d->uiWidget.m_saveButton,  SIGNAL( clicked() ) ,
+             widget->gpxFileModel(),    SLOT( saveFile() ) );
     connect( d->uiWidget.m_closeButton, SIGNAL( clicked() ) ,
-             widget->gpxFileModel(), SLOT( closeFile() ) );
+             widget->gpxFileModel(),    SLOT( closeFile() ) );
 #else
     FileViewModel* model = widget->fileViewModel();
     d->uiWidget.m_fileView->setModel( model );
@@ -189,15 +216,11 @@ void MarbleControlBox::addMarbleWidget(MarbleWidget *widget)
              this,
              SLOT( enableFileViewActions() ) );
 
-    connect( d->uiWidget.m_saveButton,
-             SIGNAL( clicked() ),
-             model,
-             SLOT( saveFile() ) );
+    connect( d->uiWidget.m_saveButton,  SIGNAL( clicked() ),
+             model,                     SLOT( saveFile() ) );
 
-    connect( d->uiWidget.m_closeButton,
-             SIGNAL( clicked () ),
-             model,
-             SLOT( closeFile() ) );
+    connect( d->uiWidget.m_closeButton, SIGNAL( clicked () ),
+             model,                     SLOT( closeFile() ) );
 #endif
 
     // Initialize the MarbleLegendBrowser
@@ -207,6 +230,7 @@ void MarbleControlBox::addMarbleWidget(MarbleWidget *widget)
     d->uiWidget.marbleLegendBrowser->setCheckedBorders( d->m_widget->showBorders() );
     d->uiWidget.marbleLegendBrowser->setCheckedWaterBodies( d->m_widget->showLakes()
                                                       && d->m_widget->showRivers() );
+    d->uiWidget.marbleLegendBrowser->setCheckedOtherPlaces( d->m_widget->showOtherPlaces() );
     d->uiWidget.marbleLegendBrowser->setCheckedGrid( d->m_widget->showGrid() );
     d->uiWidget.marbleLegendBrowser->setCheckedIceLayer( d->m_widget->showIceLayer() );
     d->uiWidget.marbleLegendBrowser->setCheckedRelief( d->m_widget->showRelief() );
@@ -230,10 +254,10 @@ void MarbleControlBox::addMarbleWidget(MarbleWidget *widget)
     selectTheme( d->m_widget->mapTheme() );
 
     connect(d->m_widget, SIGNAL(zoomChanged(int)),
-	    this,     SLOT(changeZoom(int)));
-    connect(this,     SIGNAL(centerOn(const QModelIndex&)),
+	    this,        SLOT(changeZoom(int)));
+    connect(this,        SIGNAL(centerOn(const QModelIndex&)),
 	    d->m_widget, SLOT(centerOn(const QModelIndex&)));
-    connect(this,     SIGNAL(selectMapTheme(const QString&)),
+    connect(this,        SIGNAL(selectMapTheme(const QString&)),
 	    d->m_widget, SLOT(setMapTheme(const QString&)));
 
     // connect signals for the Legend
@@ -249,6 +273,8 @@ void MarbleControlBox::addMarbleWidget(MarbleWidget *widget)
              d->m_widget,                     SLOT( setShowRivers( bool ) ) );
     connect( d->uiWidget.marbleLegendBrowser, SIGNAL( toggledWaterBodies( bool ) ),
              d->m_widget,                     SLOT( setShowLakes( bool ) ) );
+    connect( d->uiWidget.marbleLegendBrowser, SIGNAL( toggledOtherPlaces( bool ) ),
+             d->m_widget,                     SLOT( setShowOtherPlaces( bool ) ) );
     connect( d->uiWidget.marbleLegendBrowser, SIGNAL( toggledGrid( bool ) ),
              d->m_widget,                     SLOT( setShowGrid( bool ) ) );
     connect( d->uiWidget.marbleLegendBrowser, SIGNAL( toggledIceLayer( bool ) ),
@@ -302,7 +328,8 @@ void MarbleControlBox::setWidgetTabShown( QWidget * widget,
 
 void MarbleControlBox::setLocations(QAbstractItemModel* locations)
 {
-    d->uiWidget.locationListView->setModel( locations );
+    d->m_sortproxy->setSourceModel( locations );
+    d->m_sortproxy->sort( 0 );
 }
 
 int MarbleControlBox::minimumZoom() const
@@ -483,4 +510,9 @@ void MarbleControlBox::selectTheme( const QString &theme )
     }
 }
 
+void MarbleControlBox::mapCenterOnSignal( const QModelIndex &index )
+{
+    emit centerOn( d->m_sortproxy->mapToSource( index ) );
+}
+
 #include "MarbleControlBox.moc"
