@@ -25,11 +25,13 @@
 //#include <QtDBus/QDBusConnection>
 
 #include "global.h"
+#include "AutoSettings.h"
 #include "Quaternion.h"
 #include "ViewParams.h"
 #include "TextureColorizer.h"
 #include "ClipPainter.h"
 #include "FileViewModel.h"
+#include "GeoPoint.h"
 #include "GpxFileViewItem.h"
 #include "MarbleDirs.h"
 #include "MarbleWidgetInputHandler.h"
@@ -77,6 +79,8 @@ class MarbleWidgetPrivate
     bool             m_showCompass;
     bool             m_showScaleBar;
 
+    bool             m_showFrameRate;
+
     // Parts of the image in the Widget
     CrossHairFloatItem  m_crosshair;
     CompassFloatItem    m_compass;  // Shown in the upper right
@@ -86,10 +90,6 @@ class MarbleWidgetPrivate
     MeasureTool     *m_measureTool;
 
     QRegion          m_activeRegion;
-
-    // The progress dialog for the tile creator.
-    TileCreatorDialog  *m_tileCreatorDlg;
-
 };
 
 
@@ -115,7 +115,10 @@ MarbleWidget::MarbleWidget(MarbleModel *model, QWidget *parent)
 
 MarbleWidget::~MarbleWidget()
 {
+    delete d->m_viewParams.m_canvasImage;
+    delete d->m_viewParams.m_coastImage;
     delete d->m_model;
+    delete d;
 }
 
 void MarbleWidget::construct(QWidget *parent)
@@ -127,10 +130,8 @@ void MarbleWidget::construct(QWidget *parent)
     // Some point that tackat defined. :-)
     setHome( -9.4, 54.8, 1050 );
 
-    connect( d->m_model, SIGNAL( creatingTilesStart( const QString&, const QString& ) ),
-             this,    SLOT( creatingTilesStart( const QString&, const QString& ) ) );
-    connect( d->m_model, SIGNAL( creatingTilesProgress( int ) ),
-             this,    SLOT( creatingTilesProgress( int ) ) );
+    connect( d->m_model, SIGNAL( creatingTilesStart( TileCreator*, const QString&, const QString& ) ),
+             this,    SLOT( creatingTilesStart( TileCreator*, const QString&, const QString& ) ) );
 
     connect( d->m_model, SIGNAL(themeChanged( QString )), SIGNAL(themeChanged( QString )) );
     connect( d->m_model, SIGNAL(modelChanged()), this, SLOT(updateChangedMap()) );
@@ -170,7 +171,7 @@ void MarbleWidget::construct(QWidget *parent)
              this,              SLOT( notifyMouseClick( int, int ) ) );
 
     connect( d->m_inputhandler, SIGNAL( mouseMoveGeoPosition( QString ) ),
-         this, SIGNAL( mouseMoveGeoPosition( QString ) ) );
+	     this,              SIGNAL( mouseMoveGeoPosition( QString ) ) );
 
     d->m_measureTool = new MeasureTool( this );
 
@@ -189,13 +190,18 @@ void MarbleWidget::construct(QWidget *parent)
     d->m_minimumzoom = 950;
     d->m_maximumzoom = 2200;
 
-    d->m_showScaleBar = true;
-    d->m_showCompass  = true;
+    d->m_showScaleBar  = true;
+    d->m_showCompass   = true;
+    d->m_showFrameRate = false;
 
+    // Widget translation
     QString      locale = QLocale::system().name();
     QTranslator  translator;
     translator.load(QString("marblewidget_") + locale);
     QCoreApplication::installTranslator(&translator);
+
+    // AutoSettings
+    AutoSettings* autoSettings = new AutoSettings( this );
 }
 
 MarbleModel *MarbleWidget::model() const
@@ -234,9 +240,14 @@ void MarbleWidget::setNeedsUpdate()
 }
 
 
-QAbstractListModel *MarbleWidget::placeMarkModel()
+QAbstractItemModel *MarbleWidget::placeMarkModel() const
 {
-    return d->m_model->getPlaceMarkModel();
+    return d->m_model->placeMarkModel();
+}
+
+QItemSelectionModel *MarbleWidget::placeMarkSelectionModel() const
+{
+    return d->m_model->placeMarkSelectionModel();
 }
 
 double MarbleWidget::moveStep()
@@ -255,12 +266,15 @@ int MarbleWidget::zoom() const
 
 double MarbleWidget::centerLatitude() const
 {
-    return d->m_viewParams.m_planetAxis.pitch() * RAD2DEG;
+    double centerLat =  d->m_viewParams.m_planetAxis.pitch() + M_PI;
+    if ( centerLat > M_PI ) centerLat -= 2 * M_PI; 
+    return centerLat * RAD2DEG;
 }
 
 double MarbleWidget::centerLongitude() const
 {
-    return - d->m_viewParams.m_planetAxis.yaw() * RAD2DEG;
+    double centerLon =  d->m_viewParams.m_planetAxis.yaw() + M_PI;
+    return centerLon * RAD2DEG;
 }
 
 void MarbleWidget::setMinimumZoom( int zoom )
@@ -308,6 +322,11 @@ bool MarbleWidget::showTerrain() const
     return d->m_viewParams.m_showTerrain;
 }
 
+bool MarbleWidget::showOtherPlaces() const
+{
+    return d->m_viewParams.m_showOtherPlaces;
+}
+
 bool MarbleWidget::showRelief() const
 {
     return d->m_viewParams.m_showRelief;
@@ -343,6 +362,11 @@ bool MarbleWidget::showGps() const
     return d->m_model->gpsLayer()->visible();
 }
 
+bool MarbleWidget::showFrameRate() const
+{
+    return d->m_showFrameRate;
+}
+
 bool  MarbleWidget::quickDirty() const
 {
     return d->m_model->textureMapper()->interlaced();
@@ -356,6 +380,7 @@ void MarbleWidget::zoomView(int zoom)
 
     d->m_logzoom = zoom;
 
+
     emit zoomChanged(zoom);
 
     int newRadius = fromLogScale(zoom);
@@ -380,6 +405,9 @@ void MarbleWidget::zoomView(int zoom)
     }
 
     setRadius( newRadius );
+
+    emit distanceChanged( distanceString() );
+
     drawAtmosphere();
 
     repaint();
@@ -455,28 +483,25 @@ void MarbleWidget::centerOn(const double& lon, const double& lat)
 
 void MarbleWidget::centerOn(const QModelIndex& index)
 {
+    QItemSelectionModel *selectionModel = d->m_model->placeMarkSelectionModel();
+    Q_ASSERT( selectionModel );
 
-    PlaceMarkModel* model = (PlaceMarkModel*) d->m_model->getPlaceMarkModel();
-    if (model == 0) qDebug( "model null" );
+    selectionModel->clear();
 
-    PlaceMark* mark = model->placeMark( index );
-
-    d->m_model->placeMarkContainer()->clearSelected();
-
-    if ( mark != 0 ) {
-	double  lon;
+    if ( index.isValid() ) {
+        const GeoPoint point = index.data( PlaceMarkModel::CoordinateRole ).value<GeoPoint>();
+  
+        double  lon;
         double  lat;
+        point.geoCoordinates( lon, lat );
 
-	mark->coordinate( lon, lat );
-	centerOn( -lon * RAD2DEG, -lat * RAD2DEG );
-	mark->setSelected( 1 );
-	d->m_crosshair.setEnabled( true );
+	      centerOn( -lon * RAD2DEG, -lat * RAD2DEG );
+
+        selectionModel->select( index, QItemSelectionModel::SelectCurrent );
+        d->m_crosshair.setEnabled( true );
     }
     else
-	d->m_crosshair.setEnabled( false );
-
-    d->m_model->placeMarkContainer()->clearTextPixmaps();
-    d->m_model->placeMarkContainer()->sort();
+        d->m_crosshair.setEnabled( false );
 
     repaint();
 }
@@ -497,6 +522,7 @@ Projection MarbleWidget::projection() const
     return d->m_viewParams.m_projection;
 }
 
+// FIXME: Actually take a real Projection as parameter.
 void MarbleWidget::setProjection( int projectionIndex )
 {
     Projection projection;
@@ -554,9 +580,9 @@ void MarbleWidget::setHome( const double lon, const double lat, const int zoom)
     d->m_homeZoom = zoom;
 }
 
-void MarbleWidget::setHome(const GeoPoint& _homePoint, int zoom)
+void MarbleWidget::setHome(const GeoPoint& homePoint, int zoom)
 {
-    d->m_homePoint = _homePoint;
+    d->m_homePoint = homePoint;
     d->m_homeZoom = zoom;
 }
 
@@ -598,6 +624,11 @@ void MarbleWidget::moveDown()
     rotateBy( 0, -moveStep() );
 }
 
+void MarbleWidget::leaveEvent (QEvent*)
+{
+    emit mouseMoveGeoPosition( NOT_AVAILABLE );
+}
+
 void MarbleWidget::resizeEvent (QResizeEvent*)
 {
     //	Redefine the area where the mousepointer becomes a navigationarrow
@@ -631,13 +662,15 @@ void MarbleWidget::resizeEvent (QResizeEvent*)
 
 void MarbleWidget::connectNotify ( const char * signal )
 {
-    if ( QLatin1String( signal ) == SIGNAL( mouseMoveGeoPosition( QString ) ) )
+    if ( QByteArray( signal ) == 
+         QMetaObject::normalizedSignature ( SIGNAL( mouseMoveGeoPosition( QString ) ) ) )
         d->m_inputhandler->setPositionSignalConnected( true );
 }
 
 void MarbleWidget::disconnectNotify ( const char * signal )
 {
-    if ( QLatin1String( signal ) == SIGNAL( mouseMoveGeoPosition( QString ) ) )
+    if ( QByteArray( signal ) == 
+         QMetaObject::normalizedSignature ( SIGNAL( mouseMoveGeoPosition( QString ) ) ) )
         d->m_inputhandler->setPositionSignalConnected( false );
 }
 
@@ -710,8 +743,11 @@ bool MarbleWidget::geoCoordinates(const int x, const int y,
         break;
 
     case Equirectangular:
-        float const centerLat =  d->m_viewParams.m_planetAxis.pitch();
-        float const centerLon = -d->m_viewParams.m_planetAxis.yaw();
+        // Calculate translation of center point
+        double centerLat =  d->m_viewParams.m_planetAxis.pitch() + M_PI;
+        if ( centerLat > M_PI ) centerLat -= 2 * M_PI; 
+        double centerLon =  d->m_viewParams.m_planetAxis.yaw() + M_PI;
+
         int yCenterOffset =  (int)((double)(2*radius()) / M_PI * centerLat);
         int yTop = imageHalfHeight - radius() + yCenterOffset;
         int yBottom = yTop + 2*radius();
@@ -825,7 +861,9 @@ void MarbleWidget::setActiveRegion()
             }
             break;
         case Equirectangular:
-            double centerLat = planetAxis().pitch();
+            // Calculate translation of center point
+            double centerLat =  planetAxis().pitch() + M_PI;
+            if ( centerLat > M_PI ) centerLat -= 2 * M_PI; 
             int yCenterOffset =  (int)((double)(2*zoom) / M_PI * centerLat);
             int yTop = height()/2 - zoom + yCenterOffset;
             d->m_activeRegion &= QRegion( 0, yTop, width(), 2*zoom, QRegion::Rectangle );
@@ -871,6 +909,9 @@ void MarbleWidget::setBoundingBox()
 
 void MarbleWidget::paintEvent(QPaintEvent *evt)
 {
+    QTime t;
+    t.start();
+
     bool  doClip = false;
     if( d->m_viewParams.m_projection == Spherical )
         doClip = ( d->m_viewParams.m_radius > d->m_viewParams.m_canvasImage->width() / 2
@@ -921,6 +962,27 @@ void MarbleWidget::paintEvent(QPaintEvent *evt)
 
     //Set the Bounding Box
     setBoundingBox();
+
+    double fps = 1000.0 / (double)( t.elapsed() );
+
+    if ( d->m_showFrameRate == true )
+    {
+        QString fpsString = QString( "Speed: %1 fps" ).arg( fps, 5, 'f', 1, QChar(' ') );
+
+        QPoint fpsLabelPos( 10, 20 );
+
+        painter.setFont( QFont( "Sans Serif", 10 ) );
+
+        painter.setPen( Qt::black );
+        painter.setBrush( Qt::black );
+        painter.drawText( fpsLabelPos, fpsString );
+
+        painter.setPen( Qt::white );
+        painter.setBrush( Qt::white );
+        painter.drawText( fpsLabelPos.x() - 1, fpsLabelPos.y() - 1, fpsString );
+    }
+
+    emit framesPerSecond( fps );
 }
 
 void MarbleWidget::paint(QPainter *painter, const QStyleOptionGraphicsItem *option,
@@ -997,6 +1059,12 @@ void MarbleWidget::setShowTerrain( bool visible )
     repaint();
 }
 
+void MarbleWidget::setShowOtherPlaces( bool visible )
+{
+    d->m_viewParams.m_showOtherPlaces = visible;
+    repaint();
+}
+
 void MarbleWidget::setShowRelief( bool visible )
 {
     d->m_viewParams.m_showRelief = visible;
@@ -1041,6 +1109,12 @@ void MarbleWidget::setShowLakes( bool visible )
     repaint();
 }
 
+void MarbleWidget::setShowFrameRate( bool visible )
+{
+    d->m_showFrameRate = visible;
+    repaint();
+}
+
 void MarbleWidget::setShowGps( bool visible )
 {
     d->m_viewParams.m_showGps = visible;
@@ -1121,29 +1195,17 @@ void MarbleWidget::setQuickDirty( bool enabled )
 
 // This slot will called when the Globe starts to create the tiles.
 
-void MarbleWidget::creatingTilesStart( const QString &name, const QString &description )
+void MarbleWidget::creatingTilesStart( TileCreator *creator, const QString &name, const QString &description )
 {
     qDebug("MarbleWidget::creatingTilesStart called... ");
 
-    d->m_tileCreatorDlg = new TileCreatorDialog( this );
+    TileCreatorDialog dlg( creator, this );
+    dlg.setSummary( name, description );
+    dlg.exec();
 
-    d->m_tileCreatorDlg->setSummary( name, description );
-
-    // The process itself is started by a timer, so an exec() is ok here.
-    d->m_tileCreatorDlg->exec();
     qDebug("MarbleWidget::creatingTilesStart exits... ");
 }
 
-// This slot will be called during the tile creation progress.  When
-// the progress goes to 100, the dialog should be closed.
-
-void MarbleWidget::creatingTilesProgress( int progress )
-{
-    d->m_tileCreatorDlg->setProgress( progress );
-
-    if ( progress == 100 )
-        delete d->m_tileCreatorDlg;
-}
 
 void MarbleWidget::updateChangedMap()
 {
@@ -1177,4 +1239,25 @@ int MarbleWidget::toLogScale(int zoom)
     return zoom;
 }
 
+QString MarbleWidget::distanceString() const
+{
+    const double VIEW_ANGLE = 110.0;
+
+    // Due to Marble's orthographic projection ("we have no focus")
+    // it's actually not possible to calculate a "real" distance.
+    // Additionally the viewing angle of the earth doesn't adjust to
+    // the window's size.
+    //
+    // So the only possible workaround is to come up with a distance
+    // definition which gives a reasonable approximation of
+    // reality. Therefore we assume that the average window width
+    // (about 800 pixels) equals the viewing angle of a human being.
+    //
+    double distance = ( EARTH_RADIUS * 400.0
+			/ (double)( radius() )
+			/ tan( 0.5 * VIEW_ANGLE * DEG2RAD ) );
+
+    return QString( "%L1 %2" ).arg( distance, 8, 'f', 1, QChar(' ') ).arg( tr("km") );
+}
+
 #include "MarbleWidget.moc"
