@@ -36,17 +36,16 @@
 #include "CompassFloatItem.h"
 #include "MapScaleFloatItem.h"
 
-
 class QStyleOptionGraphicsItem;
 class MarbleWidgetInputHandler;
 class MarbleWidgetPopupMenu;
 class TextureColorizer;
 class MeasureTool;
-class TileCreatorDialog;
+class TileCreator;
 class MarbleWidgetPrivate;
 class GpsLayer;
+class GpxFileModel;
 class FileViewModel;
-#include "gps/GpxFileModel.h"
 
 
 /**
@@ -113,6 +112,7 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
     Q_PROPERTY(bool showPlaces   READ showPlaces      WRITE setShowPlaces)
     Q_PROPERTY(bool showCities   READ showCities      WRITE setShowCities)
     Q_PROPERTY(bool showTerrain  READ showTerrain     WRITE setShowTerrain)
+    Q_PROPERTY(bool showOtherPlaces READ showOtherPlaces WRITE setShowOtherPlaces)
 
     Q_PROPERTY(bool showRelief   READ showRelief      WRITE setShowRelief)
     Q_PROPERTY(bool showElevationModel READ showElevationModel WRITE setShowElevationModel)
@@ -174,14 +174,24 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
      */
     int         zoom() const;
 
+    /**
+     * @brief Return the current distance string.
+     */
+    QString     distanceString() const;
+
     /**
      * @brief  Set the minimum value for the zoom.
      * @param  zoom  The new minimum value.
      */
     void        setMinimumZoom( int zoom );
 
-    // FIXME: Apidox
+    /**
+     * @brief return if the widget needs to be updated.
+     */
     bool needsUpdate() const;
+    /**
+     * @brief Mark the widget as needing an update.
+     */
     void setNeedsUpdate();
 
     /**
@@ -196,7 +206,21 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
     bool screenCoordinates( const double lon, const double lat,
                             int& x, int& y );
 
+    /**
+     * @brief Get the Y coordinate of the North Pole.
+     * @return the pixel address of the geographical north pole.
+     */
     int northPoleY();
+    /**
+     * @brief Get the Z coordinate of the North Pole.
+     * @return the Z coordinate of the geographical north pole.
+     *
+     * This function returns the Z coordinate of the north pole. It
+     * will return a positive value if the north pole is visible,
+     * i.e. pointing towards the viewer.  It will return a negative
+     * value if the north pole is behind the horizon, i.e. pointing
+     * away from the viewer.
+     */
     int northPoleZ();
 
     /**
@@ -212,7 +236,15 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
                          double& lon, double& lat,
                          GeoPoint::Unit=GeoPoint::Degree);
 
-    // TODO: APIDOCS
+    /**
+     * @brief Get a quaternion representing a point on the earth corresponding to a pixel in the widget.
+     * @param x  the x coordinate of the pixel
+     * @param y  the y coordinate of the pixel
+     * @return @c true  if the pixel (x, y) is within the globe
+     *         @c false if the pixel (x, y) is outside the globe, i.e. in space
+     */
+    // FIXME: Make the names of globalQuaternion() and
+    //        geoCoordinates() follow a pattern.
     bool    globalQuaternion( int x, int y, Quaternion &q);
 
     /**
@@ -228,7 +260,12 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
     /**
      * @brief returns the model for all the placemarks on the globe.
      */
-    QAbstractListModel *placeMarkModel();
+    QAbstractItemModel *placeMarkModel() const;
+
+    /**
+     * @brief returns the selection model for all the placemarks on the globe.
+     */
+    QItemSelectionModel *placeMarkSelectionModel() const;
 
     /**
      * @brief  Return how much the map will move if one of the move slots are called.
@@ -289,6 +326,12 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
      */
     bool  showTerrain() const;
 
+    /**
+     * @brief  Return whether other places are visible.
+     * @return The visibility of other places.
+     */
+    bool  showOtherPlaces() const;
+
     /**
      * @brief  Return whether the relief is visible.
      * @return The relief visibility.
@@ -331,6 +374,12 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
      */
     bool  showGps() const;
 
+    /**
+     * @brief  Return whether the frame rate gets displayed.
+     * @return the frame rates visibility
+     */
+    bool  showFrameRate() const;
+
     /**
      * @brief  Return whether quick and dirty rendering is enabled.
      * @return Quick and dirty rendering
@@ -350,6 +399,10 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
     /**
      * @brief  Zoom the view to a certain zoomlevel
      * @param  zoom  the new zoom level.
+     *
+     * The zoom level is an abstract value without physical
+     * interpretation.  A zoom value around 1000 lets the viewer see
+     * all of the earth in the default window.
      */
     void  zoomView(int zoom);
 
@@ -472,7 +525,7 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
      * @brief  Set the Projection used for the map
      * @param  projection projection type (e.g. Spherical, Equirectangular, Mercator)
      */
-    void  setProjection( int );
+    void  setProjection( int projection );
 
     /**
      * @brief  get the home point
@@ -492,7 +545,7 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
      * @brief  Set the home point
      * @param  homePoint  The new home point.
      */
-    void  setHome(const GeoPoint& _homePoint, int zoom = 1050);
+    void  setHome(const GeoPoint& homePoint, int zoom = 1050);
 
     /**
      * @brief  Move left by the moveStep.
@@ -518,7 +571,6 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
 
     /**
      * @brief Get the current map theme
-     * @param maptheme  The name of the new maptheme.
      */
     QString mapTheme() const;
     /**
@@ -563,6 +615,12 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
      */
     void setShowTerrain( bool visible );
 
+    /**
+     * @brief  Set whether the other places overlay is visible
+     * @param  visible  visibility of other places
+     */
+    void setShowOtherPlaces( bool visible );
+
     /**
      * @brief  Set whether the relief is visible
      * @param  visible  visibility of the relief
@@ -605,6 +663,12 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
      */
     void setShowGps( bool visible );
 
+    /**
+     * @brief Set whether the frame rate gets shown
+     * @param visible  visibility of the frame rate
+     */
+    void setShowFrameRate( bool visible );
+
     /**
      * @brief Set the current Gps position
      * @param lat the new latitude value
@@ -627,8 +691,14 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
      */
     void openGpxFile( QString &filename );
 
-    GpxFileModel * gpxFileModel();
+    /**
+     * @brief Return a QAbstractItemModel containing GPX files.
+     */
+    GpxFileModel  *gpxFileModel();
 
+    /**
+     * @brief Return a QAbstractItemModel containing files.
+     */
     FileViewModel* fileViewModel() const;
 
     /**
@@ -639,6 +709,7 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
 
     /**
      * @brief A slot that is called when the model starts to create new tiles.
+     * @param creator the tile creator object.
      * @param name  the name of the created theme.
      * @param description  a descriptive text that can be shown in a dialog.
      * @see    creatingTilesProgress
@@ -654,19 +725,11 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
      * calls to creatingTilesProgress, and the poup dialog can then be
      * closed.
      */
-    void creatingTilesStart( const QString& name, const QString& description );
+    void creatingTilesStart( TileCreator *creator, const QString& name, const QString& description );
 
     /**
-     * @brief A slot that is called while the model is creating new tiles.
-     * @param  progress  the percentage done
-     * @see    creatingTilesStart
-     *
-     * This function is connected to the models' signal with the same
-     * name. See creatingTilesStart for a description of how it is
-     * used.
+     * @brief Update the widget because the mdoel changed.
      */
-    void creatingTilesProgress( int progress );
-
     void updateChangedMap();
 
     /**
@@ -678,14 +741,16 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
     /**
      * @brief Signal that the zoom has changed, and to what.
      * @param zoom  The new zoom value.
+     * @see  zoomView()
      */
     void  zoomChanged( int zoom );
+    void  distanceChanged( const QString& distanceString );
 
     /**
      * @brief Signal that the theme has changed
      * @param theme  Name of the new theme.
      */
-    void  themeChanged( QString );
+    void  themeChanged( QString theme );
 
     void  mouseMoveGeoPosition( QString );
 
@@ -693,7 +758,14 @@ class MARBLE_EXPORT MarbleWidget : public QWidget
 
     void  timeout();
 
+    void  framesPerSecond( double fps );
+
  protected:
+    /**
+     * @brief Reimplementation of the leaveEvent() function in QWidget.
+     */
+    void  leaveEvent( QEvent *event );
+
     /**
      * @brief Reimplementation of the paintEvent() function in QWidget.
      */
