@@ -16,6 +16,7 @@
 
 #include <math.h>
 
+#include <QtCore/QObject>
 
 // This enum is used to choose projection shown in the view.
 
@@ -30,7 +31,11 @@ const double RAD2DEG = 180.0 / M_PI;
 
 const QString MARBLE_VERSION_STRING = "pre-0.5-SVN";
 
+const QString NOT_AVAILABLE = QObject::tr("not available");
+
 // Usage deprecated:
 const double RAD2INT = 21600.0 / M_PI;
 
+const double EARTH_RADIUS = 6378.0;
+
 #endif
